
(*
:Name: Months`
:Context: Months`
:Title: Gregorian Months of [the] Year
:Summary: Month names, days in month, etc...
:Author: John D. Hendrickson
:Mathematica Version 4.0
:Package Version: 1.0.2
:History: 1.0.2 improved packaging. 1.0.1 pretty print minor fix
:Keywords: Months.
:Requirements: None.
:Copyright: None.
*)
(*
:Warnings:
	Practically void of error checking right yet.
	The `Calendar` package assumes that before {1752, 9, 2} the
		Julian calander will be used (can be selected though)
	The below barely knows the Gregorian system and can't.
*)
(*
:Discussion:
	Mathematica seems to have a hole in its calendar!
	The `Calendar` does time system changes (its addition f[] has limits).
	Mathematica's Time/Date functions do time differences and time zone changes.
	What happened to calendar placement?  
	That is, if I have an initial date and a hand 
	full of days and months, what calendar day will I be on?
		This package has a few utilities for calendar placement.
*)
(*
The `Calendar` package allows one to specify dates that don't exist.  This \
means that days in a month can't be pinned down without extra material.	
*)
(*
The Gregorian calendar has been in use in the Western world since 1582 by \
Roman Catholic countries, and since 1752 by English\[Hyphen]speaking \
countries. The Gregorian calendar counts leap years every year divisible by \
4, except for centuries not divisible by 400, which are not leap years.
*)
(*
Thesis: I would like to make an, oh, five year Gregorian calendar forward or \
back, by easily scanning days, but all the parts to do so are not present.
*)
(*
		pre-existing system of units package (doesn't work for months exactly)
		pre-existing calander package operations (which have limitations)
			DayOfWeek[{year,month,day}] == {Monday||Tuesday||...}
			DaysBetween[{year,month,day},{year,month,day}] == n Day
			DaysPlus[{year,month,day}] == {year,month,day}
		pre-existing Time/Date functions (built-in)
			FromDate[date] == abs time
			ToDate[abs] == abs time
*)
(*
TODO:
	show a calendar using the above and the below
*)
(*
(* BeginPackage["Months`", "Miscellaneous`Units`", "Miscellaneous`SIUnits`", 
    "Calendar`"] *)
*)
BeginPackage["Months`"];
Months::usage = 
    "The months of the year and other funs like YearDay.\n\tSome allow but \
ignore extra Date[] parts and all ignore hr, min., sec.";
DaysInMonth::usage = "Days in month for given year and month.";
MonthTab::usage = "List of last days in month for given year.";
MonthDays::usage = "Sum of days in first n months for given year";
MonthOfYear::usage = "Contains the names of the months of the year.";
YearDay::usage = "Yearday given year, month and day of month.";
MonthOfYearDay::usage = "Month given year and yearday.";
IsLeapYear::usage = "True,False given year.";
PrettyDate::usage = "Readable Date[]";
MonthNames::usage = "Names of month, as Symbols.";
MonthNamesStrings::usage = "Names of month, as Strings.";
DayNames::usage = 
    "Names of days, as Symbols (context is Miscellaneous`Calendar`).";
DayNamesStrings::usage = "Names of days, as Strings.";
DaysInYear::usage = "Number of Days in given Year.";
dayofweek::usage = "Day of Week given date, numeric";
dayofweekName::usage = "Name of Day of Week given date (text)";
daysbetween::usage = 
    "Days between two dates (from noon to noon), use DaysBetween.";
daystoSunday::usage = "Days to Sunday, Jan 1 2012 ; used by dayofweek.";
IncrementToward::usage = "For [y,x] return n closer to y than x.";
IncrementAway::usage = 
    "For [y,x] return n further from y than x.\n\t(from side of y or if zero \
to sign of x).";
IncrementAwayAbove::usage = "For [y,x] return n further above y than x.";
IncrementAwayBelow::usage = "For [y,x] return n further under y than x.";
(* Miscellaneous`Calendar` defines Day name messages *)

MonthUsageText := "is a month name";
January::usage = MonthUsageText;
February::usage = MonthUsageText;
March::usage = MonthUsageText;
April::usage = MonthUsageText;
May::usage = MonthUsageText;
June::usage = MonthUsageText;
July::usage = MonthUsageText;
August::usage = MonthUsageText;
September::usage = MonthUsageText;
October::usage = MonthUsageText;
November::usage = MonthUsageText;
December::usage = MonthUsageText;
Begin["`Private`"];
MonthNames = {January, February, March, April, May, June, July, August, 
      September, October, November, December};
MonthNamesStrings = Map[SymbolName, MonthNames];
DayNames = {Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday};
DayNamesStrings = Map[SymbolName, DayNames];
IncrementToward[y_, x_] := If[(y > 0 && x > y) || (y < 0 && x > y), x - 1,
    If[x =!= y, x + 1, x]
    ]
IncrementAway[y_, x_] := If[(y > 0 && x > y) || (y < 0 && x > y), x + 1,
    If[x =!= y, x - 1, x + Sign[x]]
    ]
IncrementAwayAbove[y_, x_] := If[(y > 0 && x > y) || (y < 0 && x > y), x + 1,
    If[x =!= y, x - 1, x + 1]
    ]
IncrementAwayBelow[y_, x_] := If[(y > 0 && x > y) || (y < 0 && x > y), x + 1,
    If[x =!= y, x - 1, x - 1]
    ]
IsLeapYear[
    year_] := (Mod[year, 4] == 0) && (Mod[year, 100] != 0) || (Mod[year, 
          400] == 0)
IsLeapYear[{year_, ___}] := IsLeapYear[year]
DayTab = {{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, {31, 29, 31, 30, 
        31, 30, 31, 31, 30, 31, 30, 31} (*for leap year*)}; 
(*method from : The C Programming Language*)
DaysInMonth[year_, month_] := DayTab[[If[IsLeapYear[year], 2, 1], month]]
DaysInMonth[{year_, month_, ___}] := DaysInMonth[year, month]
MonthTab[year_] := DayTab[[If[IsLeapYear[year], 2, 1]]]
MonthTab[{year_, ___}] := MonthTab[year]
MonthDays[year_, month_] := Plus[Sequence @@ Take[MonthTab[year], month]]
MonthDays[{year_, month_, ___}] := MonthDays[year, month]
YearDay[year_, month_, day_] := MonthDays[year, month - 1] + day
YearDay[{year_, month_, day_, ___}] := YearDay[year, month, day]
DaysInYear[year_] := If[IsLeapYear[year], 366, 365]
DaysInYear[{year_, ___}] := DaysInYear[year]
MonthOfYearDay[year_, yearday_] := 
  Module[{tmonth, x}, 
    tmonth = Floor[yearday/30]; (*tentative month*)x = 
      MonthDays[year, tmonth - 1];
    If[tmonth < 12 && (x + DaysInMonth[year, tmonth]) < yearday, tmonth + 1, 
      tmonth]]
(*
# days from Sunday, January 1 2012 (or any year who's 1st day is Sunday)
*)
daystoSunday[year_, month_, day_] := Module[{d, i, dow},
    d = 0; i = 0;
    y = IncrementToward[2012, year];
    If[year != 2012 && year != 2011, d = DaysInYear[2012]];
    While[++i < 10^5,
      If[y == 2012, Break[]];
      d += DaysInYear[y];
      (* Print["d ", d, " y ", y]; *)
      y = IncrementToward[2012, y];
      ];
    (* Print["d ", d]; *)
    i = YearDay[year, month, day];
    If[year < 2012, i = DaysInYear[year] - i + 1, --i];
    d + i
    ]
daystoSunday[{year_, month_, day_, ___}] := daystoSunday[year, month, day]
dayofweek[year_, month_, day_] := Module[{d, i, dow},
    d = daystoSunday[year, month, day];
    ++d;
    (* Print["d2 ", d]; *)
    dow = Mod[d, 7];
    (* Print["d3 ", dow]; *)
    If[year < 2012 && dow > 0, dow = 7 - dow];
    If[dow == 0, dow = 7];
    (* Print["dow ", dow]; *)
    (* DayNames[[dow]] *)
    dow
    ]
dayofweekName[year_, month_, day_] := 
  DayNamesStrings[[dayofweek[year, month, day]]]
dayofweekName[{year_, month_, day_, ___}] :=
  
  DayNamesStrings[[dayofweek[year, month, day]]]
(*
Miscellaneous`Calendar` uses Friday as a symbol with no uphat value.  I can't \
see that being useful.  It only begins to matter here at dayofweek.  The \
right return is numeric unless string is asked for.
*)
dayofweek[{year_, month_, day_, ___}] := dayofweek[year, month, day]
daysbetween[dt1 : {y1 : _, m1 : _, d1 : _}, dt2 : {y2 : _, m2 : _, d2 : _}] :=
   Module[{di, df},
    di = daystoSunday[Sequence @@ dt1];
    df = daystoSunday[Sequence @@ dt2];
    If[Sign[y1 - 2012] =!= Sign[y2 - 2012],
      di + df, Abs[di - df]
      ]
    ]
daysbetween[dt1 : {y1 : _, m1 : _, d1 : _, ___}, 
    dt2 : {y2 : _, m2 : _, d2 : _, ___}] := 
  daysbetween[{y1, m1, d1}, {y2, m2, d2}]
PrettyDate[dt2___] := Module[{y, M, d, h, m, s},
    If[dt2 === Null, dt = Date[], dt = dt2];
    While[Length@dt < 6, AppendTo[dt, 0]];
    Scan[(If[\[Not] IntegerQ@#, Return[]]) &, dt];
    y = dt[[1]]; M = dt[[2]]; d = dt[[3]];
    h = dt[[4]]; m = dt[[5]]; s = dt[[6]];
    tz = TimeZone[];
    dow = DayOfWeek[Take[dt, 3]];
    dow = TodaysDayName[]; (* wrong *)
    
    dow = dayofweekName[Take[dt, 3]]; (* maybe *)
    
    dow <> ", " <>
      MonthNamesStrings[[M]] <> " " <>
      ToString[d] <>
       " " <>
      ToString[y] <> ".  " <>
      
      ToString[If[(h - 12) > 0, h - 12, h]] <> ":" <>
      
      If[m < 10, "0", ""] <>
      ToString[m] <> ":" <>
      
      If[s < 10, "0", ""] <>
      ToString[s] <> " " <>
      
      ToString[If[(h - 12) > 0, "p.m.", "a.m."]] <> " " <>
      
      ToString[Floor[tz]] <> "\[ThinSpace]" <> "GMT"
    ]
(*
(* MonthOfYearDay[] *)
*)
(*
(* Protect[MonthOfYear] *) (* Unprotect[MonthOfYear] *)
*)
End[]
EndPackage[]
(*
uci["cal -y"]
*)
(*
"                             2012                              \n\n      \
January               February               March        \nSu Mo Tu We Th Fr \
Sa  Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa\n 1  2  3  4  5  6  7          \
  1  2  3  4               1  2  3\n 8  9 10 11 12 13 14   5  6  7  8  9 10 \
11   4  5  6  7  8  9 10\n15 16 17 18 19 20 21  12 13 14 15 16 17 18  11 12 \
13 14 15 16 17\n22 23 24 25 26 27 28  19 20 21 22 23 24 25  18 19 20 21 22 23 \
24\n29 30 31              26 27 28 29           25 26 27 28 29 30 31\n        \
                                    \n       April                  May       \
            June        \nSu Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa  Su Mo \
Tu We Th Fr Sa\n 1  2  3  4  5  6  7         1  2  3  4  5                  1 \
 2\n 8  9 10 11 12 13 14   6  7  8  9 10 11 12   3  4  5  6  7  8  9\n15 16 \
17 18 19 20 21  13 14 15 16 17 18 19  10 11 12 13 14 15 16\n22 23 24 25 26 27 \
28  20 21 22 23 24 25 26  17 18 19 20 21 22 23\n29 30                 27 28 \
29 30 31        24 25 26 27 28 29 30\n                                        \
    \n        July                 August              September      \nSu Mo \
Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa  Su Mo Tu We Th Fr Sa\n 1  2  3  4  5  6 \
 7            1  2  3  4                     1\n 8  9 10 11 12 13 14   5  6  \
7  8  9 10 11   2  3  4  5  6  7  8\n15 16 17 18 19 20 21  12 13 14 15 16 17 \
18   9 10 11 12 13 14 15\n22 23 24 25 26 27 28  19 20 21 22 23 24 25  16 17 \
18 19 20 21 22\n29 30 31              26 27 28 29 30 31     23 24 25 26 27 28 \
29\n                                            30\n      October             \
  November              December      \nSu Mo Tu We Th Fr Sa  Su Mo Tu We Th \
Fr Sa  Su Mo Tu We Th Fr Sa\n    1  2  3  4  5  6               1  2  3       \
              1\n 7  8  9 10 11 12 13   4  5  6  7  8  9 10   2  3  4  5  6  \
7  8\n14 15 16 17 18 19 20  11 12 13 14 15 16 17   9 10 11 12 13 14 15\n21 22 \
23 24 25 26 27  18 19 20 21 22 23 24  16 17 18 19 20 21 22\n28 29 30 31       \
    25 26 27 28 29 30     23 24 25 26 27 28 29\n                              \
              30 31\n"
*)
(*
daysbetween[{2011, 12, 30}, {2012, 1, 2}]
*)
(*
"d "\[InvisibleSpace]0
*)
(*
"d "\[InvisibleSpace]0
*)
(*
3
*)
(*
dayofweek[2011, 12, 31]
*)
(*
Saturday
*)
(*
dayofweek[2012, 1, 8]
*)
(*
Sunday
*)
(*
dayofweek[2012, 1, 3]
*)
(*
Tuesday
*)
(*
PrettyDate[]
*)
(*
"Monday, August 13 2012.  7:17:53 p.m. -4\[ThinSpace]GMT"
*)
