
(*
Package Info
*)
(*
:Name: Months`
*)
(*
:Context: Months`Months`
*)
(*
:Title: gregorian and julian Months , a few Astronomical conversions
*)
(*
:Summary:

	Needs[Months`Months`]

	a few extra astronomical / calendar / epoch features
	
	supports multiple calendars
		(julian, gregorian, synodic, anomalistic)
		supports JDN conveniently
		NEW: supports day number from 0 of any calendar
	
	conversion all-way between supported calendars

	supports multiple epochs and epoch generation
		convert between some epochs without calendar support
			ie, J2000.0, JDN
		create new epoch with a datum + other epoch
		can work with TAI/clock epochs
			but cannot import from solar only

	note defaults: calendar->gregorian , DayNumber->julian

	has a few other convenience functions

	long names used (use Alt-K should bring up a list as you type)
	readable code, no crypic calendar formulas at all.

	It should be first noted JPL doesn't use "a calendar" for time
		(their Horizons product uses simulation maybe?)
	HOWEVER due to heavy use in (the past of) astronomy,
	one may (often) be drawn to need date conversion.

	(if made to be right within one day by adjusting orig
	 formula: it's a new calendar wrong to stored dates)

	SEE: Months.notes.nb for warnings, discusssions, tests
	
	todo: simple SHA GHA etc time conversion / print
		  synodic is NOT accurate to moon, uses average
*)
(*
:Author: John D. Hendrickson
*)
(*
:Mathematica Version 4.0
*)
(*
:Package Version: 1.0.4
*)
(*
:History:
	1.0 had a small oversight but that effeced date a year after made.
	1.0.2 improved packaging
	1.0.4 above
	see Warnings
*)
(*
:Keywords: Months, Month, Date, Day, Week
*)
(*
:Requirements: None.
*)
(*
:Copyright: None.
*)
(*
:Warnings:

	Most of Months has no thorough checking, but has some miles on it.
	But consider NewEpochFrom untested and hardly used.
	
	see Months.notes.nb for general calendar warnings
	(historic and Mathematica calendar swapping,
	 Day Number does not swap, general stuff - awareness)

	The `calendar` package assumes that at {1752,9,2}
	mathematica changes calendars (you may not know!)
	Months` stays on same calendar.
*)
(*
outer  var  decl or init
*)
(*
see notes nb about Calendar and adding a calendar
*)
(*
Clear["Months`Months`","Months`anomalistic`","Months`julian`","Months`\
gregorian`","Months`Common`MonthsCommon`"];
*)
Clear["Months`Months`"];
(*
	Months common
*)
BeginPackage["Months`Months`", "Months`Common`MonthsCommon`"];
(*
lunisolarOrder
*)
Options[Months] = {calendar -> gregorian, DayNumberBy -> julian};
Months::usage = 
    "gregorian and julian Months , a few Astronomical conversions.  if you \
load calendar` after a few help items will change in default context search \
path.";
Months::"unformatted" = "`1`";
MonthsGenericText := "used in Months`, see Months.m";
CDN::usage = "calendar day number";
DayNumberChange2::usage = 
    "[dn, calFrom, calTo], convert day number from one calendar to another";
DayNumberDifference::usage = 
    "[date1|dn, date2|dn, cal], positive when date2 is later than date1.  \
note date is always converted as from cal to julian and diff is always julian \
days.  arg may be dn (elsewhere a plain number might be fixed to be date for \
year, not here). See also DND2";
DayNumberDifference2::usage = 
    "[date1|dn, date2|dn, cal], positive when date2 is later than date1.  \
diff is in days on same calendar dates are in (use cal to specify which cal \
dt are).  see also DND ";
DayStart::usage = 
    "beginning of day for given calendar (julian starts day at 12 pm).  \
please use Epoch[cal,cal,daystart]";
DayNumber::usage = 
    "To count of days since epoch -4712 given date or day count from a date.  \
remember julian day starts at 12 pm and is read as 12 not as 0.  DN goes from \
and to julian or gregorian but always uses julian day count (convert if you \
need otherwise)";
DayNumber2::usage = 
    "DayNumber2 works cal to cal (any supported calendar) using the calendar \
day 0 epochs, while DayNumber always works from JDN epoch.  it has no classic \
\"Y<400 or no non-negative julian\" problems (see Months.notes.nb)";
DayNumberChange2::usage = 
    "[dn,from cal,to cal] change day number to other cal, like \
calendarChange2 but in dn";
DayNumberChange2Update::usage = 
    "like DayNumberChange2 except From will be updated with gleened info if \
possible, NewEpochFrom needs this";
DayNumberFrom::usage = 
    "[date, From cal , to cal] , count of days relative to julian epoch";
DayNumberFrom2::usage = 
    "[date, From cal , to cal] , 2 is a new rewrite of old other.  count of \
days relative to epoch of calendar";
DayNumberFromCenturyFraction::usage = 
    "see DayNumberToCenturyFraction, ToCenturyFraction";
DayNumberTo::usage = 
    "[date, To cal , from number] , count of days relative to julian dn \
epoch";
DayNumberTo2::usage = 
    "[date, To cal , from number] , 2 is a new rewrite of old other.  count \
of days relative to epoch of calendar";
DayNumberToCenturyFraction::usage = 
    "julian date expressed as century fraction is used as 'T' in some popular \
astronomy equations, ie J2000.0  see: julianTojulianCenturyFraction";
DaysFromCenturyFraction::usage = MonthsGenericText;
DaysToCenturyFraction::usage = MonthsGenericText;
DayNumberDotForm::usage = "Floor@y + any date == Jn.m";
Epoch::usage = 
    "[this,this|other,type] store or get data. the offset, a time from which \
a set of dates originate. note it matters if data is solar (solar) or \
clock.";
JDN::usage = "julian day number";
JDNMonths::usage = "is what julian day count algorithm thinks (12)";
MJDN::usage = "modified julian day number";
MoonPhase::usage = 
    "[date,cal,False] text for moon phase or cardinal if True";
NewEpochFrom::usage = 
    "[From,To,{args},show,cals|{},types|{}] try to fill in the skeleton of \
new Epoch[To] using Epoch[From].  NOTE: not well tested, not used much yet.  \
First assign Epoch[new,x] one date you know that has lookup in Epoch[From] \
(also daystart,CDN,JDN if you can).  Say you know gregorian JDN of new To, \
let args=={gregorian,JDN} to imply Epoch[From and To,gregorian,JDN] exist.  \
Having that the rest can be attempted (depending on avail. conversion and \
data).  show gives Output cell which can be evaluated.\n\ntodo: only one \
{args} can be given per attempt but running 2x same To it is not destructive \
(maybe running with another arg imports more).\n\ntodo: will TT import to \
solar? no unless same year.  if for some reason both solar and clock should \
import (if coincidnt) the only way will be to make two new epochs; one from \
each type, and merge.  see Months.notes.nb about solar v. clock calendaring";
PrettyDate::usage = "pretty formatted date, no options avail";
SecondsEpoch::usage = "add these to change seconds epoch on same calendar";
ToCenturyFraction::usage = 
    "convert date to century fraction (this is T for MeanTropicalYearTTfunc \
but note it does not change epoch, so julian yr -2000 is 0 (TT 2000) and \
gregorian is in wrong epoch to count days in, i beleive)";
ToCenturyFraction2::usage = 
    "uses DayNumberFrom2 otherwise same (epoch of cal assumption)";
YearFromCenturyFraction::usage = MonthsGenericText;
YearToCenturyFraction::usage = MonthsGenericText;
ZodiacNames::usage = MonthsGenericText;
ZodiacNamesStrings::usage = MonthsGenericText;
anomalistic::usage = 
    "is package Months`anomalistic` adding dates by average anomalistic year \
(begins jan 3 at noon).  it is provided to exhibit dates without leap years \
(also w/o monthless), each year is exactly an anomalistic year.";
calendarChange2::usage = 
    "[dt, calFrom, calTo], convert date from one calendar to another";
calTransformFrom::usage = "";
calTransformTo::usage = "";
dayofweekjulian::usage = "note julian day starts at 12";
gregorian::usage = "is package in Months`gregorian adding that feature set";
gregorianTojulian::usage = 
    "uses DayNumber not Daynumber2.  see calendarChange2";
gregorianTojulian2::usage = MonthsGenericText;
julian::usage = "is a package in Months`julian` adding that feature set";
julianDayNumber::usage = 
    "days passed on new julian calendar since -4712, see Epochs et al";
julianTogregorian::usage = 
    "uses DayNumber not Daynumber2.  see calendarChange2";
julianTogregorian2::usage = MonthsGenericText;
(*lunisolar::usage = 
      "was not finished to save time, not worth it, see notes";*)

moonphase::usage = "[jdn] uses synodic EpochDaysUT";
squared::usage = 
    "example calendar package, a counter example to synodicToDays and \
synodic";
synodic::usage = 
    "synodic is a supported cal that uses ave mean synodic month. it isnt' \
too accurate per lunation but is fine for showing moon phase over many \
years";
synodicEpochDaysTT::usage = "see synodicEpochDaysUT";
synodicEpochDaysUT::usage = 
  "can be used to make a new synodic calendar (new timescale and epoch).  
      current is DND2[(sedUT[0] - 1), gregorian, gregorian], 
is UT2000. with that use calendarChange2 to edit epochs for ea. cal."
      synodicFromDays::usage = 
        "[jdn] given jdn return lunations, uses synodicEpochDaysUT \
(inaccurate eq)";
synodicToDays::usage = 
    "[lun] given lunations return days, uses synodicEpochDaysUT (inaccurate \
eq).  shows a counter example to use of synodic and squared";
synodic::usage = "is a package in Months`synodic` adding synodic caledar";
DayTab::usage = 
    "list of month lengths MonthTabs needs to yeild one for given year, shape \
may differ among calendars";
DaysInMonth::usage = "Days in month for given year, 0 allowed.";
DaysInYear::usage = "Number of Days in given Year.";
FromSeconds::usage = "uses julianDayNumber epoch";
FromSeconds2::usage = "uses calendar epoch";
FromSecondsmm2::usage = "uses mathematica 1900 epoch, from gregorian only";
FromSecondsanomalistic::usage = 
    "uses y2k MeanTropicalYear[] with no further correction";
GreatestDay::usage = MonthsGenericText;
IsLeapYear::usage = "True,False given year.";
LesserYear::usage = MonthsGenericText;
MonthDays::usage = 
    "Sum of days in first n months for given year inclusive, 0 allowed.";
MonthNames::usage = "Names of month, as Symbols.";
MonthNamesjulian::usage = 
    "Intercalarus (Mercedonius) are never in julian Day Number calendar. see \
julian`DayTab for possible older layout";
MonthNamesStrings::usage = "Names of month, as Strings.";
MonthTab::usage = "List of days in each month for given year.";
MonthOfYear::usage = "Contains the names of the months of the year.";
MonthOfYearDay::usage = "returns Month given year and yearday.";
MonthsInYear::usage = "usually 12, see synodic";
ToSeconds::usage = "uses julianDayNumber epoch";
ToSeconds2::usage = "uses calendar epoch";
ToSecondsmm2::usage = "uses mathematica 1900 epoch, from gregorian only";
ToSecondsanomalistic::usage = 
    "uses y2k MeanTropicalYear[] with no further correction";
YearDay::usage = "Yearday given year, month and day of month.";
YearFromDays::usage = 
    "return signed lesseryear * Year given signed days (to find 365*Y it \
exludes all implied leaps, and knows -y time is in reverse).  see also: \
leapdays (the inverse per say)";
dayofweek::usage = 
    "Day of Week given date, numeric, given by julian Day Number conversion, \
the first cardinal day of the week is Sunday";
dayofweekName::usage = "day of week string";
daystoSunday::usage = 
    "experimental and iterative, use something else.  Days to Sunday, Jan 1 \
2012 ; used by dayofweek.";
dtsdayofweek::usage = 
    "experimental and iterative, use something else.  Day of Week given date, \
numeric, uses daystoSunday";
dtsdayofweekName::usage = 
    "experimental and iterative, use something else.  Name of Day of Week \
given date (text), uses daystoSunday";
dtsdaysbetween::usage = 
    "experimental and iterative, use something else.  Days between two dates \
(from noon to noon), use DaysBetween, uses daystoSunday";
epochlist::usage = "List of all epoch  Epoch[cal,cal,types,data]";
epochs::usage = "List of all epoch triplets Epoch[cal,cal,types]";
fixdate::usage = 
    "a minor detail to preen {y,M,d,h,m,s}.  it checks calendar.";
fixdatebegin::usage = "a minor detail of fixdate";
leapdays::usage = 
    "given year a count of all leap days, inclusive.  with right year leap \
days is known.  with days and leap formula right year can be known";
types::usage = "List of all epoch data types Epoch[cal,cal,types,data]";
(*
(* Miscellaneous`Calendar` defines Day name messages *)
(* MonthUsageText:="is a month name"; too many messages we all know *)
*)
DayNames::usage = 
    "Names of days, as Symbols (context is Miscellaneous`Calendar`).";
DayNamesStrings::usage = MonthsGenericText;
ZodiacNames = {Aries, Taurus, Gemini, Cancer, Leo, Virgo, Libra, Scorpio, 
      Sagittarius, Capricorn, Aquarius, Pisces};
julianMonths = 13; 
Clear[gregorianTommSecondsEpoch2]
gregorianTommSecondsEpoch2;
(*
these typical values are for recent epochs, ie, TT 2000, i imagine and are \
not up to date
*)
MeanTropicalYearTTfunc::usage = 
    "[ jcf, julian, Epoch[TT,TT] ]  results show diffreneces due to \
precession \"for distant past\".  est length of yr in days given julian \
century fraction from TT 2000, see Months.notes.nb example.";
IncrementToward::usage = "For [y,x] return n closer to y than x.";
IncrementAway::usage = 
    "For [y,x] return n further from y than x.\n\t(from side of y or if zero \
to sign of x).";
IncrementAwayAbove::usage = "For [y,x] return n further above y than x.";
IncrementAwayBelow::usage = "For [y,x] return n further under y than x.";
{JDN, TT, TAI, TGC, SundayYear, MTY, 
    synodicMonth,(*lunisolarMonth,*)julianToJDN, julianFromJDN, JDNtoMJDN, 
    JDNfromMJDN, date, type, solar, seasonal, clock, lunar};
(*
data  init
*)
DayStartJulian = 12;
MeanSiderealDay = 23.9344699;(*apparent to stars*)

MeanTropicalYear = 365.242190402;(*current, 
  loose - 1.9s/yr*)
(*MSY at epoch J2000.0 = January 1, 2000, 12 : 00 : 00 TT*)

MeanSolarYear = 365.256363004;(*apparent to sun*)

MeanAnomalisticYear = 365.259636;(*earth sun orbit*)

VernalEquinoxYear = 365.2424;(*see mty*)

synodicMonth = 29.530588853;(*apparent*)
(*
meanLunarYeriod=27.321582;
synodicMonthFC=(synodicMonthReal \
synodicMonth)/(synodicMonth-synodicMonthReal);
*)
centuryDays[gregorian] = 146463/4;
yearDays[gregorian] = centuryDays[gregorian]/100;
centuryDays[julian] = 36525;
yearDays[julian] = centuryDays[julian]/100;
centuryDays[anomalistic] = MeanAnomalisticYear 100;
yearDays[anomalistic] = MeanAnomalisticYear;

centuryDays[anomalistic] = MeanAnomalisticYear 100;
yearDays[anomalistic] = MeanAnomalisticYear;
(*
epochs vary if days flow through or from 0 and daystart is not known from one \
or a few epochs - it really must be said, thus daystart is doubly necessary
*)
(*
assume GMT or usual for epoch (could store else and check)
*)
(*
	how do you count seconds from 1900 on solar? eraser used?
*)
Epoch[FromDate, FromDate, text] = "FromDate ToDate, mm";
Epoch[FromDate, FromDate, type] = solar;
Epoch[FromDate, FromDate, JDN] = 2415020 + 1/2;
Epoch[FromDate, gregorian, date] = {1900, 1, 1, 0, 0, 0};
Epoch[FromDate, julian, date] = {1899, 12, 20, 0, 0, 0};
Epoch[SundayYear, SundayYear, text] = 
    "a year who's first day is Sunday NOT AN EPOCH";
Epoch[SundayYear, SundayYear, type] = solar;
Epoch[SundayYear, gregorian, date] = {2012, 1, 1, 0, 0, 0};
Epoch[SundayYear, julian, date] = {-4712, 1, 1, DayStartJulian, 0, 0};
(*
-0.000739 is ~ 63.84 sec TT
*)
Epoch[TT, TT, text] = "TT day count 0";
Epoch[TT, TT, daystart] = 1/2;
Epoch[TT, TT, type] = clock;
Epoch[TT, TT, date] = {2000, 1, 1, DayStartJulian, 0, 0};
Epoch[TT, gregorian, date] = {2000, 1, 1, DayStartJulian - 1, 58, 
    56.1503966152668`}; 
Epoch[TT, julian, date] = {1999, 12, 19, DayStartJulian - 1, 58, 
    56.1503966152668`} ;
(*
see Months.notes.nb  about  epochs
	TAI TT GPS all tick same rate, all clock. but don't use "clock" for TGC \
rename it.
*)
Epoch[TAI, TAI, text] = "TAI 1977";
Epoch[TAI, TAI, daystart] = 0;
Epoch[TAI, TAI, type] = clock;
Epoch[TAI, TAI, date] = {1977, 1, 1, 0, 0, 0};
Epoch[TAI, TAI, JDN] = 2443144.5003725;
Epoch[TAI, TT, JDN] = Epoch[TAI, TAI, JDN] + 32.184/(24 60^2);
Epoch[TAI, GPS, JDN] = Epoch[TAI, TAI, JDN] - 19/(24 60^2);
Epoch[TAI, TT, MJDN] = JDNtoMJDN[Epoch[TAI, TAI, JDN]];
Epoch[TAI, TT, date] = {1977, 1, 1, 0, 0, 32.184};
Epoch[TAI, TGC, date] = {1977, 1, 1, 0, 0, 32.184};
Epoch[TAI, gregorian, date] = {1977, 1, 1, 0, 0, 32.18401372432709`};
Epoch[TAI, julian, date] = {1976, 12, 19, 0, 0, 32.18401372432709`};
(*
	solar, specified by JDN, not a running clock
*)
Epoch[J2000, J2000, text] = 
    "J2000 is gregorian 2000 at 12 noon (J is not julian, should be G)";
Epoch[J2000, J2000, daystart] = 1/2;
Epoch[J2000, J2000, type] = solar;
Epoch[J2000, J2000, JDN] = 2451545;
Epoch[J2000, gregorian, date] = {2000, 1, 1, 12, 0, 0};
Epoch[J2000, julian, date] = {1999, 12, 19, 12, 0, 0};
(*
	what if one needed epoch that assumes any clock can be imported or offset \
from
	0 of, but not any solar ?
*)
(*
Epoch[J2000c, J2000c, text] = 
    "J2000c is TT 2000 at 12 noon (J is not julian, should be G)";
Epoch[J2000c, J2000c, daystart] = 1/2;
Epoch[J2000c, J2000c, type] = clock;
Epoch[J2000c, J2000c, JDN] = 2451545;
*)
(*
	GPS(TAI)=TAI-19
	TT(TAI)=TAI+32.184 s
simple to say Epoch[] could define these as functions, but TAI epoch may \
contain solar,lunar,etc which won't be that easy, toseconds possible but new \
epoch would be easier
*)
ZodiacNamesStrings = Map[SymbolName, ZodiacNames];
(*
begin  private
*)
Begin["`Private`"];
(*
helper  funs
*)
(*
>=0 or > 0 ? had previously made use of Sign[0]==0 for correctness for \
reminder and to add number line looping ability: but rather than explain such \
use i revert for simplicity, toward y==0 goes to 0
*)
IncrementToward[y_, x_] := If[(y >= 0 && x > y) || (y <= 0 && x > y), x - 1,
    If[x =!= y, x + 1, x]
    ]
IncrementAway[y_, x_] := If[(y >= 0 && x > y) || (y <= 0 && x > y), x + 1,
    If[x =!= y, x - 1, x + Sign[x]]
    ]
IncrementAwayAbove[y_, x_] := 
  If[(y >= 0 && x > y) || (y <= 0 && x > y), x + 1,
    If[x =!= y, x - 1, x + 1]
    ]
IncrementAwayBelow[y_, x_] := 
  If[(y >= 0 && x > y) || (y <= 0 && x > y), x + 1,
    If[x =!= y, x - 1, x - 1]
    ]
(* to Jn.m , floor's y *)

DayNumberDotForm[
    y_, {Y_, M_, d_, h_, m_, s_}] := (Floor@y + h/24 + m/1440 + s/86400)
(*
	requires latent binding of DaysInMonthjulian for julian
*)
fixdatebegin[dtp_] := Module[{x, dt},
    dt = dtp;
    If[IntegerQ@dt, dt = {dt}];
    If[Head@dt =!= List, Message[Months::"unformatted", dt]; 
      Return[$Failed]];
    x = Length@dt;
    If[x > 6, Message[Months::"unformatted", dt]; Return[$Failed]];
    S = False; If[x == 6, S = Last@dt; dt[[6]] = 0];
    Scan[(If[\[Not] IntegerQ@#, Message[Months::"unformatted", #]; 
            Return[$Failed]]) &, dt];
    If[S =!= False, 
      If[NumericQ@S, dt[[6]] = S, Message[Months::"unformatted", s]; 
        Return[$Failed]]];
    dt
    ]
(*
minor  conversions
*)
julianTogregorian[dt_] := 
  DayNumberTo[DayNumberFrom[dt, julian, julian], gregorian, julian]
gregorianTojulian[dt_] := 
  DayNumberTo[DayNumberFrom[dt, gregorian, julian], julian, julian]
julianTogregorian2[dt_] := calendarChange2[dt, julian, gregorian]
gregorianTojulian2[dt_] := calendarChange2[dt, gregorian, julian]
julianToJDN[dn_?NumericQ] := dn + 1721058
julianFromJDN[dn_?NumericQ] := dn - 1721058
JDNtoMJDN[dn_?NumericQ] := dn - (2400000 + 1/2)
JDNfromMJDN[dn_?NumericQ] := dn + (2400000 + 1/2)
(*
just add or subtract seconds epoch but only on same calendar
*)
gregorianTommSecondsEpoch2 := DayNumberDifference2[
      Epoch[gregorian, gregorian, date],
      Epoch[FromDate, gregorian, date],
      gregorian]24 60^2
(*
calTo is the new , calFrom is other to import from
		if args is TT (clock) it may ignore solar
		if user later supplies a solar also, must run again
*)
YearToCenturyFraction[jdt_?NumericQ, cal_] := 
  jdt (yearDays[cal]/centuryDays[cal])
DaysToCenturyFraction[jdt_?NumericQ, cal_] := jdt/centuryDays[cal]
YearFromCenturyFraction[jdn_?NumericQ, cal_] := 
  jdn (centuryDays[cal]/yearDays[cal])
DaysFromCenturyFraction[jdn_?NumericQ, cal_] := jdn centuryDays[cal]
ToCenturyFraction[jdt_, cal_] := 
  DayNumberFrom[jdt, cal, cal]/centuryDays[cal]
ToCenturyFraction2[jdt_, cal_] := 
  DayNumberFrom2[jdt, cal, cal]/centuryDays[cal]
(*
may vary between 29.272 and 29.833 ? eq. shows parts of sec rise over 800yr
*)
(*
days between conjunction of sun and moons, dark/new moon ? from TT 2000+N==0 \
epoch
*)
synodicEpochDaysTT[m_] = 5.597661 + 29.5305888610 m + (102.026 10^-12)m^2
synodicEpochDaysUT[m_] = 
  5.597661 + 29.5305888610 m + (102.026 10^-12)m^2 - 
    0.000739 - (235 10^-12)m^2
(*
moon days using JDN no calendar shows a counter-example to use of synodic
*)
\!\(synodicFromDays[n_] := 
    3.7601335599440503`*^9\ \((29.530588861\[InvisibleSpace] - 
          0.000023062870593228412\ \@\(1.6395229113969543`*^12 - \((n - \
Epoch[J2000, J2000, JDN])\)\))\)\)
synodicToDays[m_] := synodicEpochDaysUT[m] + Epoch[J2000, J2000, JDN]
moonphase[days_] := 
  FractionalPart@synodicFromDays[days] synodicMonth // MoonPhase
(*
NewEpochFrom
*)
(*
If[Head@Epoch[calFrom,othersymb,#]===Epoch,Continue[]];
	in mm there isn't a good sparse array support per say
	if, ex julian=1, Epoch[len] could be listed given len
	however if >>"file" the data would be illegible
*)
epochlist[] := Module[{i},
    Table[
      DownValues[Epoch][[i]][[1]] /. 
        h_[Epoch[x_, y_, z_, ___]] -> {x, y, z}, {i, 2, 
        Length@DownValues[Epoch]}]]
epochs[] := 
  Module[{tab, i}, tab = epochlist[]; Union[tab[[All, 1]], tab[[All, 2]]]]
types[] := 
  Module[{tab, i}, tab = epochlist[]; Union[tab[[All, 3]], tab[[All, 3]]]]
NewEpochFrom[calFrom_, calTo_, args_List, show_:True, pcals_:{}, ptypes_:{}] :=

    Module[{i, j, typeArg, dnc2, dnd2, tab, tcals1, tcals2, tcals, ttypes, 
      types, cals, calOther, typeOther, x, y, z, epochType, eptT, eptF, eptO, 
      cell, once},
    cals = pcals; types = ptypes; epochType = type;
    If[cals == {},
      tcals1 = Cases[epochlist[], {calFrom, _, _}][[All, 2]];
      cals = Union[tcals1, tcals1]];
    (*see notes below*)
    
    If[types == {}, tab = epochlist[]; ttypes = tab[[All, 3]];
      types = 
        Complement[Union[ttypes, ttypes], {text, epochType, daystart}]];
    If[Head@Epoch[calTo] === Epoch,
      Epoch[calTo, calTo, text] = ToString@calTo];
    If[args == Null, Return[$Failed]];
    calArg = First@args;
    (*Sequence @@ Rest@ may be needed*)
    typeArg = Last@args;
    Print@{cals, types, calArg, typeArg, epochType};
    x = Epoch[calTo, calArg, typeArg];
    If[Head@x === Epoch(*|| Head@Epoch[calFrom, calArg, typeArg] === Epoch*), 
      Return[$Failed]];
    If[\[Not] NumericQ@x, x = DayNumberFrom2[x, calArg, calArg]];
    If[\[Not] NumericQ@x, Return[$Failed]];
    If[Head@Epoch[calTo, calTo, epochType] === Epoch && 
        Head@Epoch[calArg, calArg, epochType] =!= Epoch, 
      Epoch[calTo, calTo, epochType] = Epoch[calArg, calArg, epochType]];
    eptT = Epoch[calTo, calTo, epochType];
    eptF = Epoch[calFrom, calFrom, epochType];
    If[Head@eptT === Epoch || Head@eptF == Epoch, Return[$Failed]];
    dnc2 = DayNumberChange2Update[x, calTo, calFrom, types];
    If[dnc2 === $Failed, Return[$Failed]];
    (*dnd2 is diff calTo, calFrom; dnc2 == (x - diff)*)
    dnd2 = x - dnc2;
    Print@{x, dnc2, "dist: ", dnd2};
    
    (* list calFrom contents (iterate by calFrom, calOther, typeOther) *)
    
    Do[
      calOther = cals[[i]];
      Do[
        typeOther = types[[j]];
        Print@{"trying: ", calOther, typeOther};
        If[calOther == calArg && typeArg == typeOther, Continue[]];
        (* keep existing *)
        
        If[Head@Epoch[calTo, calOther, typeOther] != Epoch, Continue[]];
        (* plus define clock + day no; check last see why l8tr*)
        
        x = Epoch[calFrom, othersymb, typeOther];
        If[Head@x === Epoch, Continue[]];
        If[\[Not] NumericQ@x,
          x = DayNumberFrom2[x, calOther, calOther];
          If[\[Not] NumericQ@x, Return[$Failed]];
          typeOther = CDN];
        Epoch[calTo, calOther, typeOther] = x - dnd2;
        (*need to fill in what can be; why not check diff*)
        
        y = DayNumberChange2Update[x - dnd2, calTo, calOther, types];
        If[y - (x - dnd2) != dnd2, 
          Print@{"conversion check not equal skipped: ", calTo, dnd2, 
              calOther, (x - dnd2) - y}; Clear@Epoch[calTo, calOther, #]; 
          Continue[]];
        
        eptO = Epoch[calOther, calOther, epochType];
        If[Head@eptO == Epoch, Return[$Failed]];
        
        Print@"ADDED";
        If[eptT == eptO, Continue[]];
        
        (*more optional; see note3*)
        Print@"types different";
        If[{clock, solar} == Union[eptT, eptO],
          
          x = First@
              DayNumberTo2[Epoch[calTo, calOther, typeOther], gregorian, 
                gregorian];
          
          y = First@
              DayNumberTo2[Epoch[calFrom, calOther, typeOther], gregorian, 
                gregorian];
          
          If[NumericQ@x && NumericQ@y && x == y, 
            AppendTo[Epoch[calTo, calTo, epochType], eptO]; Continue[]];
          x = Epoch[calTo, calTo, JDN]/(365 + 1/4) // Floor;
          y = Epoch[calFrom, calFrom, JDN]/(365 + 1/4) // Floor;
          
          If[NumericQ@x && NumericQ@y && x == y, 
            AppendTo[Epoch[calTo, calTo, epochType], eptO]; Continue[]]];
        If[{clock, lunar} == Union[eptT, eptO] || {lunar, solar} == 
              Union[eptT, eptO],
          x = Epoch[calTo, calTo, JDN];
          y = Epoch[calFrom, calFrom, JDN];
          If[NumericQ@x && NumericQ@y,
            x = First@DayNumberTo2[x, lunar, lunar];
            y = First@DayNumberTo2[y, lunar, lunar];
            
            If[NumericQ@x && NumericQ@y && x == y, 
              AppendTo[Epoch[calTo, calTo, epochType], eptO]; Continue[]]]];
        (*forget the entry it may be too inaccurate due to dropped seconds*)
 
               Clear@Epoch[calTo, calOther, #];
        Print@"forgot";
        , {j, 1, Length@types}], {i, 1, Length@cals}
      ];
    
    If[show,
      once = False;
      cell = Cell[StringJoin@Map[(
                  If[once, "\n", once = True; ""]
                    <> "Epoch["
                    <> ToString[#[[1]]] <> ","
                    <> ToString[#[[2]]] <> ","
                    <> ToString[#[[3]]] <> "]:="
                    <> ToString[Epoch[Sequence @@ #] // InputForm] <> ";"
                  ) &,
              
              Cases[Table[
                  DownValues[Epoch][[i]][[1]] /. 
                    h_[Epoch[x_, y_, z_, ___]] -> {x, y, z}, {i, 2, 
                    Length@DownValues[Epoch]}], {calTo, _, _}]
              ], "Text"];
      NotebookWrite[EvaluationNotebook[], cell]
      ];
    ]
(*
Day Number 2
*)
(*
see Months.notes.nb section: an important note about any calendar conversion, \
DayStart
see Months.notes.nb section: "Day Number 2"
	LesserYear[calFrom] ie 365
the below method works any linear scale if cal.nb can can return leapdays[] \
and YearFromDays[] (months are optional). works if days are inserted to \
calendarize (can do M inserts or both iff ld[] YFD[] do it).  Head checking \
alows calTranform to click in as calendar w/o defining the whole deal.
*)
DayNumberFrom2[dtp_, calFrom_, calTo_] :=
  
  Module[{y, M, d, h, m, s, dt, Y, ds, f, lesseryear},
    If[calFrom =!= calTo,
      dt = calendarChange2[dtp, calFrom, calTo],
      dt = fixdate[dtp, calTo]];
    If[dt == $Failed, Return[$Failed]];
    ds = Epoch[calTo, calTo, daystart]; If[\[Not] NumericQ@ds, ds = 0];
    {y, M, d, h, m, s} = dt;
    d = d + (h + (m + s/60)/60)/24;
    f = If[M === 0, 0, MonthDays[y, M - 1, calTo]];
    If[Head@f =!= MonthDays, d = d + f];
    d = d - 1; Y = y;
    d = d - ds;
    If[y < 0, Y += 1; d = -(DaysInYear[y, calTo] - d)];
    If[Head@LesserYear[calTo] =!= LesserYear,
      d = 
        LesserYear[calTo] Y + 
          If[y === 0 || y === -1, 0, leapdays[y - Sign@y, calTo]] leapammount[
              calTo] + d];
    f = calTransformFrom[d, calTo];
    If[Head@f =!= calTransformFrom, f, d]
    ]
(*
M=MonthsInYear[y,calTo]-M+1;
*)
(*
1) get right year, leaps and day follow.  2) just call cal functions to fill \
in rest.
if YearFromDays is defined then necessarily leaps leapammount LesserYear \
DaysInYear is numeric
if calTransformTo is used it must make date format indepotent, useable with \
DNF2 (ie, days as moon count), but units and use can vary
*)
DayNumberTo2[dtp_?NumericQ, calFrom_, calTo_] := 
  Module[{y, M, d, h, m, s, f, leaps, days, sign, Y, ave},
    If[calFrom =!= calTo,
      f = DayNumberChange2[dtp, calFrom, calTo],
      f = dtp];
    ds = Epoch[calTo, calTo, daystart]; If[\[Not] NumericQ@ds, ds = 0];
    f = f + ds + 1;
    sign = If[Sign[f] == 0, 1, Sign[f]];
    days = Abs@f;
    (* see note 1 , timescales *)
    y = YearFromDays[f, calTo];
    If[Head@y === YearFromDays,
      d = sign days; y = 0,
      If[sign >= 0,
        leaps = leapdays[y, calTo]leapammount[calTo];
        (* see note 2 *)
        
        If[IsLeapYear[y, calTo], leaps = leaps - leapammount[calTo]];
        If[\[Not] NumbericQ@leaps, leaps = 0];
        d = days - LesserYear[calTo] y - leaps;
        If[Floor@d > DaysInYear[y, calTo], d -= DaysInYear[y, calTo]; 
          y = y + 1]
        ,
        leaps = -leapdays[y, calTo]leapammount[calTo];
        If[IsLeapYear[y, calTo], leaps = leaps - leapammount[calTo]];
        d = days - LesserYear[calTo] (-y - 1) - leaps;
        d = DaysInYear[y, calTo] - d;
        (*see note 3*)
        
        If[d < 1, y = y - 1; d += DaysInYear[y, calTo]];
        ]
      ];
    M = MonthOfYearDay[y, Floor@d, calTo];
    If[Head@M =!= MonthOfYearDay,
      f = If[M === 0, 0, MonthDays[y, M - 1, calTo]];
      If[Head@f =!= MonthDays, d = d - f],
      M = 0
      ];
    f = calTransformTo[d, calTo];
    If[Head@f =!= calTransformTo, Return[f]];
    f = d // FractionalPart; h = f 24;
    f = h // FractionalPart; m = f 60;
    f = m // FractionalPart; s = f 60;
    {d, m, h} = {d, m, h} // Floor;
    {y, M, d, h, m, s}
    ]
calendarChange2[dtp_, calFrom_, calTo_] :=
  Module[{dt, dn, dnc},
    If[calFrom === calTo, Return[dtp]];
    dt = fixdate[dtp, calFrom];
    If[dt == $Failed, Return[$Failed]];
    dn = DayNumberFrom2[dt, calFrom, calFrom];
    If[dn == $Failed, Return[$Failed]];
    dnc = DayNumberChange2[dn, calFrom, calTo];
    If[dnc == $Failed, Return[$Failed]];
    DayNumberTo2[dnc, calTo, calTo]
    ]
(*
note preffer From epoch over To and dn before date
*)
(*
see Months.notes.nb  epochs note 1.  fixme? unlike NewEpoch this likes a \
rigid type list.
*)
(*
if To is +5 ahead  of From, From-5 converts to To, left on timeline
if To is -5 behind of From, From+5 converts to To, right on timeline
*)
(*
if To is +5 ahead of From, From-5
	if To and From are neg,
	-from - 5 , if x is neg
*)
DayNumberChange2[dtp_?NumericQ, calFrom_, calTo_, types_:{CDN, JDN, date}] :=

    Module[{x, y},
    If[calFrom === calTo, Return[dtp]];
    If[Intersection[types, {JDN}] =!= {},
      y = Scan[(
              
              If[Head@Epoch[calFrom, calFrom, #] =!= Epoch && 
                  Head@Epoch[calTo, calTo, #] =!= Epoch,
                
                x = DayNumberDifference2[Epoch[calFrom, calFrom, #], 
                    Epoch[calTo, calTo, #], calTo];
                If[NumericQ@x, Return[dtp - x]]]
              ) &, {JDN}]];
    If[NumericQ@y, Return[y]];
    y = Scan[(
            
            If[Head@Epoch[calFrom, calTo, #] =!= Epoch && 
                Head@Epoch[calTo, calTo, #] =!= Epoch,
              
              x = DayNumberDifference2[Epoch[calFrom, calTo, #], 
                  Epoch[calTo, calTo, #], calTo];
              If[NumericQ@x, Return[dtp - x]]]
            ) &, types];
    If[NumericQ@y, Return[y]];
    (* reverse lookup may have data the other does not yet so check *)
    
    y = Scan[(
            
            If[Head@Epoch[calFrom, calFrom, #] =!= Epoch && 
                Head@Epoch[calFrom, calTo, #] =!= Epoch,
              
              x = DayNumberDifference2[Epoch[calFrom, calFrom, #], 
                  Epoch[calFrom, calTo, #], calFrom];
              If[NumericQ@x, Return[dtp - x]]]
            ) &, types];
    If[NumericQ@y, Return[y]];
    Return[$Failed]
    ]
(*
NewEpochFrom needs this. fill in missing data for From that is obviously \
gleened.  user may have date and need JDN for some equation.  other funs do \
no cross-referencing or data expansion, assume this is unwanted, ie keeping \
epochs in jdn only.  likes rigid type list.
*)
DayNumberChange2Update[dtp_?NumericQ, calFrom_, calTo_, 
    types_:{CDN, JDN, date}] :=
  Module[{x, y, dnd2},
    If[Head@Epoch[calFrom, calFrom, CDN] === Epoch && 
        Head@Epoch[calFrom, calFrom, date] =!= Epoch,
      x = DayNumberFrom2[Epoch[calFrom, calFrom, date], calFrom, calFrom];
      If[x =!= $Failed, x = DayNumberTo2[x, calFrom, calFrom]]];
    If[Head@Epoch[calFrom, calFrom, CDN] =!= Epoch && 
        Head@Epoch[calFrom, calFrom, date] === Epoch,
      x = DayNumberTo2[Epoch[calFrom, calFrom, CDN], calFrom, calFrom];
      If[x =!= $Failed, Epoch[calFrom, calFrom, date] = x]];
    If[Head@Epoch[calFrom, calTo, CDN] === Epoch && 
        Head@Epoch[calFrom, calTo, date] =!= Epoch,
      x = DayNumberFrom2[Epoch[calFrom, calTo, date], calTo, calTo];
      If[x =!= $Failed, x = DayNumberTo2[x, calTo, calTo]]];
    If[Head@Epoch[calFrom, calTo, CDN] =!= Epoch && 
        Head@Epoch[calFrom, calTo, date] === Epoch,
      x = DayNumberTo2[Epoch[calFrom, calTo, CDN], calTo, calTo];
      If[x =!= $Failed, Epoch[calFrom, calTo, date] = x]];
    
    y = DayNumberChange2[dtp, calFrom, calTo, types];
    If[y == $Failed, Return[$Failed]];
    dnd2 = dtp - y;(* delta, dnd2 diff from to, x in dnd2 *)
    
    If[Head@Epoch[calFrom, calFrom, JDN] === Epoch && 
        Head@Epoch[calTo, calTo, JDN] =!= Epoch,
      Epoch[calFrom, calFrom, JDN] = Epoch[calTo, calTo, JDN] - dnd2];
    (* policy if From has JDN already dont repeat it for every entry *)
    (* 
      If[Epoch[calFrom, calTo, JDN] === Epoch same as From *)
    
    If[Head@Epoch[calFrom, calTo, CDN] === Epoch, 
      Epoch[calFrom, calTo, CDN] = Epoch[calFrom, calFrom, CDN] + dnd2];
    If[Head@Epoch[calFrom, calTo, date] === Epoch,
      x = $Failed;
      If[Epoch[calFrom, calTo, CDN] =!= Epoch, 
        x = Epoch[calFrom, calTo, CDN] + dnd2];
      If[Epoch[calFrom, calTo, JDN] =!= Epoch, 
        x = Epoch[calFrom, calTo, JDN]];
      If[x =!= $Failed, 
        Epoch[calFrom, calTo, date] = DayNumberTo2[x, calTo, calTo]];
      ];
    Return[y]
    ]
(*
DayNumberDifference
*)
DayNumberDifference[dt1_, dt2_, cal_] := Module[{dta, dtb},
    If[NumericQ@dt1, dta = dt1, dta = DayNumberFrom[dt1, cal, julian]];
    If[NumericQ@dt2, dtb = dt2, dtb = DayNumberFrom[dt2, cal, julian]];
    If[dta == $Failed || dtb == $Failed, Return[$Failed]];
    If[Sign@dta === Sign@dtb, (dtb - dta), 
      If[dtb > dta, 1, -1](Abs@dta + Abs@dtb)]
    ]
DayNumberDifference2[dt1_, dt2_, cal_] := Module[{dta, dtb, x, sign},
    If[NumericQ@dt1, dta = dt1, dta = DayNumberFrom2[dt1, cal, cal]];
    If[NumericQ@dt2, dtb = dt2, dtb = DayNumberFrom2[dt2, cal, cal]];
    If[dta == $Failed || dtb == $Failed, Return[$Failed]];
    If[Sign@dta === Sign@dtb, (dtb - dta), 
      If[dtb > dta, 1, -1](Abs@dta + Abs@dtb)]
    ]
(*
If[Sign@dta===Sign@dtb,Sign@dta (dtb-dta),If[dtb>dta,1,-1](Abs@dta+Abs@dtb)]
*)
(*
old unsigned
DayNumberDifference2[dt1_,dt2_,cal_]:=Module[{dta,dtb},
    dta=DayNumberFrom2[dt1,cal,cal];
    dtb=DayNumberFrom2[dt2,cal,cal];
    If[Sign@dta===Sign@dtb,Abs@(dta-dtb),Abs@(Abs@dta+Abs@dtb)]
    ]
*)
(*
daystoSunday  daysof ...
*)
(*
# days from Sunday, January 1 2012 (or any year who's 1st day is Sunday)
*)
(*
	this is only suitable if epoch is close for printing pretty date
		these are not rhobust
	will be broken for anomalistic
*)
daystoSunday[dtp_, cal_] :=
  
  Module[{y, M, d, h, m, s, dt, dd, yy, i, Sunday, ds},
    dt = fixdate[dtp, cal];
    If[dt == $Failed, Return[$Failed]];
    ds = Epoch[cal, cal, daystart]; If[\[Not] NumericQ@ds, ds = 0];
    {y, M, d, h, m, s} = dt;
    Sunday = Epoch[SundayYear, cal, date][[1]];
    dd = 0; i = 0;
    yy = IncrementToward[Sunday, y];
    If[y > Sunday && y =!= Sunday && y =!= (Sunday - 1), 
      dd = DaysInYear[Sunday, cal]];
    While[++i < 10^7,
      If[yy == Sunday, Break[]];
      dd += DaysInYear[yy, cal];
      yy = IncrementToward[Sunday, yy];
      ];
    i = YearDay[y, M, d, cal];
    If[y < Sunday, i = DaysInYear[y, cal] - i + 1, --i];
    If[y < Sunday && h < Mod[ds 24, 24], ++i];
    If[y >= Sunday && h < Mod[ds 24, 24], --i];
    dd + i
    ]
(*
		daystart was already counted if > 1 day, but daily hour still matters
*)
(*
d = daystoSunday[year, month, day, gregorian];++d;dow = Mod[d, 7];If[year < \
SundayEpoch && dow > 0, dow = 7 - dow];If[dow == 0, dow = 7];dow
*)
dtsdayofweek[dtp_, cal_] :=
  Module[{y, M, d, h, m, s, dt, dd, i, dow, ds},
    dt = fixdate[dtp, cal];
    If[dt == $Failed, Return[$Failed]];
    {y, M, d, h, m, s} = dt;
    dd = daystoSunday[dt, cal];
    If[y < Epoch[SundayYear, cal, date][[1]], dd = -dd];
    Mod[dd, 7] + 1
    ]
(*
dtsdayofweek[year_,month_,day_,gregorian]:=Mod[If[year<SundayEpoch,7,0]+\
daystoSunday[year,month,day,gregorian],7]+1
*)
dtsdayofweekName[dt_, cal_] := DayNamesStrings[cal][[dtsdayofweek[dt, cal]]]
(*
don't use this
*)
dtsdaysbetween[dtp1_, dtp2_, cal_] := 
  Module[{y, M, d, h, m, s, y2, M2, d2, h2, m2, s2, dt1, dt2, di, df, 
      Sunday},
    Sunday = Epoch[SundayYear, cal, date][[1]];
    dt1 = fixdate[dtp1, cal];
    If[dt1 == $Failed, Return[$Failed]];
    dt2 = fixdate[dtp2, cal];
    If[dt2 == $Failed, Return[$Failed]];
    {y, M, d, h, m, s} = dt1;
    {y2, M2, d2, h2, m2, s2} = dt2;
    di = daystoSunday[dt1, cal];
    df = daystoSunday[dt2, cal];
    If[Sign[y - Sunday] =!= Sign[y2 - Sunday],
      di + df, Abs[di - df]
      ]]
(*
pretty  date
*)
(*
this could be per cal.  no need its not meant to do much
*)
(*
GMT is wrong if your TZ is not for GMT. post ante meridium (meridian, sun at \
noon)
*)
PrettyDate[dtp_List, cal_] :=
  Module[{y, M, d, h, m, s, dt, x, dow, tz},
    dt = fixdate[dtp, cal];
    If[dt == $Failed, Return[$Failed]];
    {y, M, d, h, m, s} = dt;
    tz = TimeZone[];
    dow = dayofweekName[dt, cal];
    dow <> ", " <>
      MonthNamesStrings[cal][[M]] <> " " <>
      
      ToString[d] <> " " <>
      ToString[y] <> ".  " <>
      
      ToString[If[(h - 12) > 0, h - 12, h]] <> ":" <>
      
      If[m < 10, "0", ""] <>
      ToString[m] <> ":" <>
      
      If[s < 10, "0", ""] <>
      ToString[s] <> " " <>
      
      ToString[If[h >= 12, "p.m.", "a.m."]] <> " " <>
      
      ToString[Floor[tz]] <> "\[ThinSpace]" <> "GMT"
    ]
PrettyDate[cal_] := PrettyDate[calendarChange2[Date[], gregorian, cal], cal]
MoonPhase[dtp_?NumericQ, numeric_:False] :=
  
  Module[{x}, x = Mod[dtp, synodicMonth]; 
    If[x > (7synodicMonth/8) || x < (synodicMonth/8), 
      Return[If[numeric, 1, "New Moon"]]];
    If[x >= (synodicMonth/8) && x < (3synodicMonth/8), 
      Return[If[numeric, 2, "Waxing Crecent Moon"]]];
    If[x >= (3 synodicMonth/8) && x < (5synodicMonth/8), 
      Return[If[numeric, 3, "Full Moon"]]];
    If[x >= (5 synodicMonth/8) && x < (7synodicMonth/8), 
      Return[If[numeric, 4, "Wanning Crecent Moon"]]];
    Return[$Failed]]
MoonPhase[dtp_List, cal_, numeric_:False] :=
  
  Module[{y, M, d, h, m, s, dt, x},
    dt = fixdate[dtp, cal];
    If[dt == $Failed, Return[$Failed]];
    dt = calendarChange2[dt, cal, synodic];
    {y, M, d, h, m, s} = dt;
    MoonPhase[d + h/24 + m/60 + s/60^2]
    ]
MoonPhase[cal_] := 
  MoonPhase[calendarChange2[Date[], gregorian, cal], synodic]
(*
Option  Arguement  Defaults  for  functions ,  calendar  or  DayNumberBy
*)
(*
these are for ease of use.  be wary DayNumber funs have diff default than \
Month ones.  a little confusing si vu ple
*)
MonthsInYear[] := MonthsInYear[calendar /. Options[Months]]
MonthsInYear[year_] := MonthsInYear[year, calendar /. Options[Months]]
DayStart[] := DayStart[calendar /. Options[Months]]
DayNames[] := DayNames[calendar /. Options[Months]]
DayNamesStrings[] := DayNamesStrings[calendar /. Options[Months]]
MonthNames[] := MonthNames[calendar /. Options[Months]]
MonthNamesStrings[] := MonthNamesStrings[calendar /. Options[Months]]
fixdate[dtp_] := fixdate[dtp, calendar /. Options[Months]]
fixdate[dtp_, x_] := $Failed
DayTab[{year, ___}] := DayTab[calendar /. Options[Months]]
DayTab[year_] := DayTab[calendar /. Options[Months]]
DayTab[] := DayTab[calendar /. Options[Months]]
leapdays[year_] := leapdays[year, calendar /. Options[Months]]
leapdays[{year_, ___}] := leapdays[year, calendar /. Options[Months]]
YearFromDays[days_] := YearFromDays[days, calendar /. Options[Months]]
YearFromDays[{days_, ___}] := YearFromDays[days, calendar /. Options[Months]]
IsLeapYear[year_] := IsLeapYear[year, calendar /. Options[Months]]
IsLeapYear[{year_, ___}] := IsLeapYear[year, calendar /. Options[Months]]
DaysInMonth[year_, month_] := 
  DaysInMonth[year, month, calendar /. Options[Months]]
DaysInMonth[{year_, month_, ___}] := 
  DaysInMonth[year, month, calendar /. Options[Months]]
MonthTab[year_] := MonthTab[year, calendar /. Options[Months]]
MonthTab[{year_, ___}] := MonthTab[year, calendar /. Options[Months]]
MonthDays[year_, month_] := 
  MonthDays[year, month, calendar /. Options[Months]]
MonthDays[{year_, month_, ___}] := 
  MonthDays[year, month, calendar /. Options[Months]]
DaysInYear[year_] := DaysInYear[year, calendar /. Options[Months]]
DaysInYear[{year_, ___}] := DaysInYear[year, calendar /. Options[Months]]
YearDay[year_, month_, day_] := 
  YearDay[year, month, day, calendar /. Options[Months]]
YearDay[{year_, month_, day_, ___}] := 
  YearDay[year, month, day, calendar /. Options[Months]]
MonthOfYearDay[year_, yearday_] := 
  MonthOfYearDay[year, yearday, calendar /. Options[Months]]
MonthOfYearDay[{year_, yearday_, ___}] := 
  MonthOfYearDay[year, yearday, calendar /. Options[Months]]
dayofweek[dt_] := dayofweek[dt, calendar /. Options[Months]]
dayofweekName[dt_] := dayofweekName[dt, calendar /. Options[Months]]
ToSeconds[dtp_] := ToSeconds[dtp, calendar /. Options[Months]]
FromSeconds[secs_] := FromSeconds[secs, calendar /. Options[Months]]
ToSeconds2[dtp_] := ToSeconds2[dtp, calendar /. Options[Months]]
FromSeconds2[secs_] := FromSeconds2[secs, calendar /. Options[Months]]
ToSecondsmm2[dtp_] := ToSecondsmm2[dtp, calendar /. Options[Months]]
FromSecondsmm2[secs_] := FromSecondsmm2[secs, calendar /. Options[Months]]
daystoSunday[dt_] := daystoSunday[dt, calendar /. Options[Months]]
dtsdayofweek[dt_] := dtsdayofweek[dt, calendar /. Options[Months]]
dtsdayofweekName[dt_] := dtsdayofweekName[dt, calendar /. Options[Months]]
dtsdaysbetween[dt1_, dt2_] := 
  dtsdaysbetween[dt1, dt2, calendar /. Options[Months]]
calTransformFrom[days_] := 
  calTransformFrom[days, calendar /. Options[Months]]
calTransformTo[days_] := calTransformTo[days, calendar /. Options[Months]]
PrettyDate[dtp_List] := PrettyDate[dtp, calendar /. Options[Months]]
PrettyDate[] := PrettyDate[calendar /. Options[Months]]
MoonPhase[] := MoonPhase[synodic]
(*
only default here would be 0, not helpful
*)
Epoch[] := {epochs[], types[]}
(*
julian is default for Day Number things
*)
DayNumberFrom[dtp_, tofrom_] := 
  DayNumberFrom[dtp, tofrom, DayNumberBy /. Options[Months]]
DayNumberTo[dtp_, tofrom_] := 
  DayNumberTo[dtp, tofrom, DayNumberBy /. Options[Months]]
DayNumberFrom[dtp_] := 
  DayNumberFrom[dtp, DayNumberBy /. Options[Months], 
    DayNumberBy /. Options[Months]]
DayNumberTo[dtp_] := 
  DayNumberTo[dtp, DayNumberBy /. Options[Months], 
    DayNumberBy /. Options[Months]]
DayNumberFrom2[dtp_, tofrom_] := 
  DayNumberFrom2[dtp, tofrom, DayNumberBy /. Options[Months]]
DayNumberTo2[dtp_, tofrom_] := 
  DayNumberTo2[dtp, tofrom, DayNumberBy /. Options[Months]]
DayNumberFrom2[dtp_] := 
  DayNumberFrom2[dtp, DayNumberBy /. Options[Months], 
    DayNumberBy /. Options[Months]]
DayNumberTo2[dtp_] := 
  DayNumberTo2[dtp, DayNumberBy /. Options[Months], 
    DayNumberBy /. Options[Months]]
DayNumberChange2[dtp_] := 
  DayNumberChange2[dtp, DayNumberBy /. Options[Months], 
    DayNumberBy /. Options[Months]]
DayNumberChange2[dtp_, tofrom_] := 
  DayNumberChange2[dtp, tofrom, DayNumberBy /. Options[Months]]
DayNumberChange2Update[dtp_] := 
  DayNumberChange2Update[dtp, DayNumberBy /. Options[Months], 
    DayNumberBy /. Options[Months]]
DayNumberChange2Update[dtp_, tofrom_] := 
  DayNumberChange2Update[dtp, tofrom, DayNumberBy /. Options[Months]]
DayNumberDifference[dt1_, dt2_] := 
  DayNumberDifference[dt1, dt2, DayNumberBy /. Options[Months]]
DayNumberDifference2[dt1_, dt2_] := 
  DayNumberDifference[dt1, dt2, DayNumberBy /. Options[Months]]
YearToCenturyFraction[jdt_] := 
  YearToCenturyFraction[jdt, DayNumberBy /. Options[Months]]
DaysToCenturyFraction[jdt_] := 
  DaysToCenturyFraction[jdt, DayNumberBy /. Options[Months]]
YearFromCenturyFraction[jdn_] := 
  YearFromCenturyFraction[jdn, DayNumberBy /. Options[Months]]
DaysFromCenturyFraction[jdn_] := 
  DaysFromCenturyFraction[jdn, DayNumberBy /. Options[Months]]
ToCenturyFraction[jdt_] := 
  ToCenturyFraction[jdt, DayNumberBy /. Options[Months]]
ToCenturyFraction2[jdt_] := 
  ToCenturyFraction[jdt, DayNumberBy /. Options[Months]]
(*
End  Months
*)
End[]
EndPackage[]
Needs["Months`gregorian`"]
Needs["Months`julian`"]
Needs["Months`anomalistic`"]
Needs["Months`synodic`"]
Needs["Months`squared`"]
(*
Needs["Months`lunisolar`"]
*)
