(*
Begin  gregorian
*)
Clear["Months`gregorian`"];
BeginPackage["Months`gregorian`", "Months`Common`MonthsCommon`"]
(*
data
*)
SundayEpoch = 2012;
MonthsInYear[gregorian] = 12;
DayStart[gregorian] = 0;
leapammount[gregorian] = 1;
DayNames[gregorian] = {Sunday, Monday, Tuesday, Wednesday, Thursday, Friday, 
      Saturday};
DayNamesStrings[gregorian] = Map[SymbolName, DayNames[]];
DayTab[gregorian] = {{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, {31, 
        29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31} (*for leap year*)};
MonthDayTab[gregorian] = Module[{Private`d, Private`m},
    {Flatten@
        Table[Table[
            Private`m, {Private`d, 
              DayTab[gregorian][[1, Private`m]]}], {Private`m, 1, 
            MonthsInYear[gregorian]}],
      Flatten@
        Table[Table[
            Private`m, {Private`d, 
              DayTab[gregorian][[2, Private`m]]}], {Private`m, 1, 
            MonthsInYear[gregorian]}]
      }]
GreatestDay[gregorian] = 31; LesserYear[gregorian] = 365; 
GreaterYear[gregorian] = 366;
MonthNames[gregorian] = {January, February, March, April, May, June, July, 
      August, September, October, November, December};
Epoch[gregorian, gregorian, text] = "gregorian";
Epoch[gregorian, gregorian, type] = solar;
Epoch[gregorian, gregorian, daystart] = 0;
Epoch[gregorian, gregorian, CDN] = 0;
Epoch[gregorian, gregorian, JDN] = 1721058 + 1 + 1/2;
Epoch[gregorian, gregorian, date] = {0, 1, 1, DayStart[gregorian], 0, 0};
Epoch[gregorian, julian, date] = {0, 1, 3, DayStart[gregorian], 0, 0};
Epoch[gregorian, julianDayNumber, date] = {4712, 1, 3, 0, 0, 0};
MonthNamesStrings[gregorian] = Map[SymbolName, MonthNames[gregorian]];
(*
utils
*)
Begin["`Private`"];
fixdate[dtp_, gregorian] := 
  Module[{y, M, d, h, m, s, x, dt, S, calgreg, caljul},
      dt = fixdatebegin[dtp];
      If[dt == $Failed, Return[$Failed]];
      calgreg = Epoch[gregorian, gregorian, date];
      While[Length@dt < 6, AppendTo[dt, calgreg[[Length@dt + 1]]]];
      {y, M, d, h, m, s} = dt;
      x = MonthsInYear[gregorian];
      If[M < 1 || M > x, Message[Months::"unformatted", M]; Return[$Failed]];
      x = DaysInMonth[{y, M}, gregorian];
      If[d < 1 || d > x, Message[Months::"unformatted", d]; Return[$Failed]];
      If[Quotient[h, 24] != 0, Message[Months::"unformatted", h]; 
        Return[$Failed]];
      If[Quotient[m, 60] != 0, Message[Months::"unformatted", m]; 
        Return[$Failed]];
      If[Quotient[s, 60] != 0, Message[Months::"unformatted", s]; 
        Return[$Failed]];
      dt
      ] /; ListQ@dtp || dtp \[Element] Integers
(*
gregorian calendar  funs
*)
(*
	0 is a leap year
*)
(*
	if the correct year is know leapdays is easy
	(task is not adding or removing any effecting other, just counting)
*)
MonthsInYear[year_, gregorian] := 12
leapdays[yp_?NumericQ, gregorian] := Module[{y}, y = Floor[Abs@yp];
    If[Sign[yp] === 0, 1, 
        Sign[yp]](Quotient[y, 4] - Quotient[y, 100] + Quotient[y, 400] + 
          If[yp < 0, 0, 1])]
leapdays[{y_, ___}, gregorian] := leapdays[y, gregorian]
IsLeapYear[year_?NumericQ, gregorian] := 
  Module[{y}, 
    y = Floor@
        year; (Mod[y, 4] == 0) && (Mod[y, 100] != 0) || (Mod[y, 400] == 0)]
IsLeapYear[{year_, ___}, gregorian] := IsLeapYear[year, gregorian]
(*
(*see note 4 in months.notes.nb DayNumberTo2*)
*)
YearFromDays[days_?NumericQ, gregorian] := Module[{y, ave},
    ave = (LesserYear[gregorian] + 1/4 - 1/100 + 1/400);
    If[days >= 0,
      Quotient[Floor@days - 1, ave],
      y = Quotient[Floor@Abs@days + LesserYear[gregorian] - 1, ave];
      y = -y; If[y === 0, y = -1];
      y
      ]]
YearFromDays[{days_, ___}, gregorian] := YearFromDays[days, gregorian]
DaysInMonth[year_Integer, month_Integer, gregorian] := 
  DayTab[gregorian][[If[IsLeapYear[year, gregorian], 2, 1], Floor@month]] /; 
    1 <= month <= MonthsInYear[gregorian]
DaysInMonth[{year_, month_, ___}, gregorian] := 
  DaysInMonth[year, month, gregorian]
MonthTab[year_?NumericQ, gregorian] := 
  DayTab[gregorian][[If[IsLeapYear[Floor@year, gregorian], 2, 1]]]
MonthTab[{year_, ___}, gregorian] := MonthTab[year, gregorian]
(*
here allow 0-12 as input because it's not month it's months to add, and \
result is not month days but days to add, ie:
		ToSeconds wants  MonthDays[any,0] == 0
		also want MonthDays[n,12]==365 or 366 for whole year
*)
MonthDays[year_Integer, month_Integer, gregorian] := 
  Plus[Sequence @@ Take[MonthTab[Floor@year, gregorian], Floor@month]] /; 
    0 <= month <= MonthsInYear[gregorian]
MonthDays[{year_, month_, ___}, gregorian] := 
  MonthDays[year, month, gregorian]
DaysInYear[year_?NumericQ, gregorian] := 
  If[IsLeapYear[year, gregorian], 366, 365]
DaysInYear[{year_, ___}, gregorian] := DaysInYear[year, gregorian]
YearDay[year_Integer, month_Integer, day_?NumericQ, gregorian] :=
  
  MonthDays[year, month - 1, gregorian] + day /; 
    1 <= month <= MonthsInYear[gregorian] && 
      1 <= day <= DaysInMonth[year, month, gregorian]
YearDay[{year_, month_, day_, ___}, gregorian] := 
  YearDay[year, month, day, gregorian]
(*
MonthOfYearDay[year_Integer, yearday_?NumericQ, gregorian] := 
  Module[{month, x, i},
      month = Floor[yearday/GreatestDay[gregorian]];
      If[month == 0, Return[1]];
      Do[If[MonthDays[year, month, gregorian] >= yearday, 
          Break[]]; ++month, {i, 1, MonthsInYear[gregorian]}];
      month
      ] /; 1 <= Floor@yearday <= DaysInYear[year, gregorian]
*)
MonthOfYearDay[year_Integer, yearday_?NumericQ, gregorian] := 
  MonthDayTab[gregorian][[If[IsLeapYear[Floor@year, gregorian], 2, 1], 
        yearday]] /; 1 <= Floor@yearday <= DaysInYear[year, gregorian]
MonthOfYearDay[{year_, yearday_, ___}, gregorian] := 
  MonthOfYearDay[year, yearday, gregorian]
dayofweek[dtp_, gregorian] := Module[{dt},
    dt = fixdate[dtp, gregorian]; If[dt == $Failed, Return[$Failed]];
    Mod[Floor@
          DayNumberDifference2[Epoch[SundayYear, gregorian, date], dt, 
            gregorian], 7] + 1]
dayofweekName[dt_, gregorian] := 
  DayNamesStrings[gregorian][[dayofweek[dt, gregorian]]]
ToSeconds[dtp_, gregorian] := DayNumberFrom[dtp, gregorian, julian] 24 60^2
FromSeconds[secs_?NumericQ, gregorian] := 
  DayNumberTo[secs/(24 60^2), julian, gregorian]
ToSeconds2[dtp_, gregorian] := 
  DayNumberFrom2[dtp, gregorian, gregorian] 24 60^2
FromSeconds2[secs_?NumericQ, gregorian] := 
  DayNumberTo2[secs/(24 60^2), gregorian, gregorian]
ToSecondsmm2[dtp_, gregorian] := 
  DayNumberFrom2[dtp, gregorian, gregorian] 24 60^2 + 
    gregorianTommSecondsEpoch2
FromSecondsmm2[secs_?NumericQ, gregorian] := 
  DayNumberTo2[(secs - gregorianTommSecondsEpoch2)/(24 60^2), gregorian, 
    gregorian]
(*
End gregorian
*)
End[]
EndPackage[]
