(*
Begin julian
*)
Clear["Months`julian`"];
BeginPackage["Months`julian`", "Months`Common`MonthsCommon`"]
(*
data
*)
(*
	this set up for using a very popular Julian Day number
	algorithm, it's not historical accurate
*)
JDNMonths = 12;
DayStart[julian] = 12;
leapammount[julian] = 1;
Epoch[julianDayNumber, julianDayNumber, text] = 
    "julianDayNumber julian yr -4712";
Epsolaroch[julianDayNumber, julianDayNumber, type] = solar;
Epoch[julianDayNumber, julianDayNumber, daystart] = 1/2;
Epoch[julianDayNumber, julianDayNumber, CDN] = 0;
Epoch[julianDayNumber, julianDayNumber, JDN] = 0;
Epoch[julianDayNumber, julianDayNumber, date] = {0, 1, 1, 0, 0, 0};
Epoch[julianDayNumber, gregorian, date] = {-4713, 11, 24, DayStart[julian], 0,
       0};
Epoch[julianDayNumber, julian, date] = {-4712, 1, 1, DayStart[julian], 0, 0};
(*
see notes nb
*)
Epoch[julian, julian, text] = "julian";
Epoch[julian, julian, type] = solar;
Epoch[julian, julian, daystart] = 1/2;
Epoch[julian, julian, date] = {0, 1, 1, DayStart[julian], 0, 0};
Epoch[julian, julian, CDN] = 0;
Epoch[julian, julian, JDN] = 1721058;
Epoch[julian, julianDayNumber, date] = {4712, 1, 1, DayStart[julian], 0, 0};
Epoch[julian, gregorian, date] = {-1, 12, 30, DayStart[julian], 0, 0};
(*
note: set this to months JDN thinks there are
*)
MonthsInYear[julian] = JDNMonths;
DayNames[julian] = {domenica, lunedi, martedi, mercoledi, giovedi, venerdi, 
      sabato};
DayNamesStrings[julian] = Map[SymbolName, DayNames[]];
(*
Mercedonius is Intercalaris (inter), go after feb for old leap years, see \
notes nb
*)
MonthNames[julian] = {Ianuarius, Februarius, Martius, Aprilis, Maius, Iunius, 
      Quintilis, Sextilis, September, October, November, December};
MonthNamesStrings[julian] = Map[SymbolName, MonthNames[julian]];
DayTab[julian] = {{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, {31, 29, 
        31, 30, 31, 30, 31, 31, 30, 31, 30, 31}(*leap year*)};
MonthDayTab[julian] = Module[{Private`d, Private`m},
    {Flatten@
        Table[Table[
            Private`m, {Private`d, 
              DayTab[julian][[1, Private`m]]}], {Private`m, 1, 
            MonthsInYear[julian]}],
      Flatten@
        Table[Table[
            Private`m, {Private`d, 
              DayTab[julian][[2, Private`m]]}], {Private`m, 1, 
            MonthsInYear[julian]}]
      }]
GreatestDay[julian] = 31; LesserYear[julian] = 365; 
GreaterYear[julian] = 366;
(*
utils
*)
Begin["`Private`"];
fixdate[dtp_, julian] := 
  Module[{y, M, d, h, m, s, x, dt, S, calgreg, caljul},
      dt = fixdatebegin[dtp];
      If[dt == $Failed, Return[$Failed]];
      caljul = {0, 1, 1, DayStart[julian], 0, 0};
      While[Length@dt < 6, AppendTo[dt, caljul[[Length@dt + 1]]]];
      {y, M, d, h, m, s} = dt;
      x = MonthsInYear[julian];
      If[M < 1 || M > x, Message[Months::"unformatted", M]; Return[$Failed]];
      x = DaysInMonth[{y, M}, julian];
      If[d < 1 || d > x, Message[Months::"unformatted", d]; Return[$Failed]];
      If[Quotient[h, 24] != 0, Message[Months::"unformatted", h]; 
        Return[$Failed]];
      If[Quotient[m, 60] != 0, Message[Months::"unformatted", m]; 
        Return[$Failed]];
      If[Quotient[s, 60] != 0, Message[Months::"unformatted", s]; 
        Return[$Failed]];
      dt
      ] /; ListQ@dtp || dtp \[Element] Integers
(*
julian calendar funs
*)
(*
	(was/is) possible to try old table see notes nb for iterative code
		section: "what is JDN's calendar layout ?"
*)
(*
	note this is how JDN alg. produces it
*)
MonthsInYear[year_, julian] := JDNMonths
leapdays[yp_?NumericQ, julian] := Module[{y}, y = Floor[Abs@yp];
    If[Sign[yp] === 0, 1, Sign[yp]](Quotient[y, 4] + If[yp < 0, 0, 1])]
leapdays[{y_, ___}, julian] := leapdays[y, julian]
(*
(*see note 4 in months.notes.nb DayNumberTo2*)
*)
YearFromDays[days_?NumericQ, julian] := Module[{y, ave},
    ave = (LesserYear[julian] + 1/4);
    If[days >= 0,
      Quotient[Floor@days - 1, ave],
      y = Quotient[Floor@Abs@days + LesserYear[julian] - 1, ave];
      y = -y; If[y === 0, y = -1];
      y
      ]]
YearFromDays[{days_, ___}, julian] := YearFromDays[days, julian]
IsLeapYear[year_?NumericQ, julian] := 
  Module[{y}, y = Floor@year; Mod[y, 4] == 0]
IsLeapYear[{year_, ___}, julian] := IsLeapYear[year, julian]
DaysInMonth[year_Integer, month_Integer, julian] := 
  DayTab[julian][[If[IsLeapYear[year, julian], 2, 1], Floor@month]] /; 
    1 <= month <= MonthsInYear[julian]
DaysInMonth[{year_, month_, ___}, julian] := DaysInMonth[year, month, julian]
MonthTab[year_?NumericQ, julian] := 
  DayTab[julian][[If[IsLeapYear[Floor@year, julian], 2, 1]]]
MonthTab[{year_, ___}, julian] := MonthTab[year, julian]
MonthTab[year_?NumericQ, julian] := 
  Module[{M}, 
    Table[DaysInMonth[Floor@year, M, julian], {M, 1, MonthsInYear[julian]}]]
MonthTab[{year_, ___}, julian] := MonthTab[year, julian]
MonthDays[year_Integer, month_Integer, julian] := 
  Plus[Sequence @@ Take[MonthTab[Floor@year, julian], Floor@month]] /; 
    0 <= month <= MonthsInYear[julian]
MonthDays[{year_, month_, ___}, julian] := MonthDays[year, month, julian]
DaysInYear[year_?NumericQ, julian] := If[IsLeapYear[year, julian], 366, 365]
DaysInYear[{year_, ___}, julian] := DaysInYear[year, julian]
YearDay[year_Integer, month_Integer, day_?NumericQ, julian] :=
  
  MonthDays[year, month - 1, julian] + day /; 
    1 <= month <= MonthsInYear[julian] && 
      1 <= day <= DaysInMonth[year, month, julian]
YearDay[{year_, month_, day_, ___}, julian] := 
  YearDay[year, month, day, julian]
(*
this always works (even with "real julian" above see notes nb about) and yet \
isn't too slow.  tentative months must be on low side
*)
(*
MonthOfYearDay[year_Integer, yearday_?NumericQ, julian] :=
  
  Module[{month, x, i},
      month = Floor[yearday/GreatestDay[julian]];
      If[month == 0, Return[1]];
      Do[If[MonthDays[year, month, julian] >= yearday, Break[]]; ++month, {i, 
          1, MonthsInYear[julian]}];
      month
      ] /; 1 <= Floor@yearday <= DaysInYear[year, julian]
*)
MonthOfYearDay[year_Integer, yearday_?NumericQ, julian] := 
  MonthDayTab[julian][[If[IsLeapYear[Floor@year, julian], 2, 1], yearday]] /; 
    1 <= Floor@yearday <= DaysInYear[year, julian]
MonthOfYearDay[{year_, yearday_, ___}, julian] := 
  MonthOfYearDay[year, yearday, julian]
dayofweek[dtp_, julian] := Module[{dt},
    dt = fixdate[dtp, julian]; If[dt == $Failed, Return[$Failed]];
    Mod[Floor@julianToJDN@DayNumberFrom2[dt, julian, julian], 7] + 1]
dayofweekName[dt_, julian] := 
  DayNamesStrings[julian][[dayofweek[dt, julian]]]
ToSeconds[dtp_, julian] := DayNumberFrom[dtp, julian, julian] 24 60^2
FromSeconds[secs_?NumericQ, julian] := 
  DayNumberTo[secs/(24 60^2), julian, julian]
ToSeconds2[dtp_, julian] := DayNumberFrom2[dtp, julian, julian] 24 60^2
FromSeconds2[secs_?NumericQ, julian] := 
  DayNumberTo2[secs/(24 60^2), julian, julian]
(*
julian day number
*)
(*
see section: lack of tests  julian  , Julian Day Number ,  Julian ,  and \
epochs
*)
DayNumberFrom[dtp_, gregorian, julian] := 
  Module[{dt, A, B, C, D, E, F, JD, y, M, d, h, m, s},
    dt = fixdate[dtp, gregorian];
    If[dt == $Failed, Return[$Failed]];
    {y, M, d, h, m, s} = dt;
    If[M <= 2, --y; M += JDNMonths];
    Y = y; D = d;
    A = Y/100 // Floor;
    B = A/4 // Floor;
    C = 2 - A + B;
    E = (365 + 1/4)(Y + 4716) // Floor;
    F = (306001/10000)(M + 1) // Floor;
    JD = C + D + E + F - (1524 + 1/2);
    JD + (h + (m + s/60)/60)/24
    ]
DayNumberFrom[dtp_, julian, julian] := 
  Module[{dt, A, B, C, D, E, F, JD, y, M, d, h, m, s},
    dt = fixdate[dtp, julian];
    If[dt == $Failed, Return[$Failed]];
    {y, M, d, h, m, s} = dt;
    If[M <= 2, --y; M += JDNMonths];(* why greg months? *)
    Y = y; D = d;
    (*
      A = Y/100 // Floor;
      B = A/4 // Floor;
      C = 2 - A + B;
      *)
    A = 0; B = 0; C = 0;
    E = (365 + 1/4)(Y + 4716) // Floor;
    F = (306001/10000)(M + 1) // Floor;
    JD = C + D + E + F - (1524 + 1/2);
    JD + (h + (m + s/60)/60)/24
    ]
DayNumberTo[jdc_?NumericQ, gregorian, julian] := 
  Module[{dt, Q, Z, W, X, A, B, C, D, E, F, JD, y, M, d, h, m, s},
    Q = jdc + 1/2;
    Z = Q // Floor;
    W = (Z - (1867216 + 1/4))/(36524 + 1/4) // Floor;
    X = W/4 // Floor;
    A = Z + 1 + W - X;
    B = A + 1524;
    C = (B - (1221/10))/(365 + 1/4) // Floor;
    D = (365 + 1/4) C // Floor;
    E = (B - D)/(306001/10000) // Floor;
    F = (306001/10000) E // Floor;
    d = B - D - F + (Q - Z);
    M = If[0 < E - 1 <= JDNMonths, E - 1, E - (JDNMonths + 1)];
    y = If[M <= 2, C - 4715, C - 4716];
    h = FractionalPart[d] 24;
    d = d // Floor;
    m = FractionalPart[h] 60;
    h = h // Floor;
    s = FractionalPart[m] 60;
    m = m // Floor;
    {y, M, d, h, m, s}
    ]
(*
	.5  is because the julian Day begins at noon, Greenwich time
*)
DayNumberTo[jdc_?NumericQ, julian, julian] := 
  Module[{dt, Q, Z, W, X, A, B, C, D, E, F, JD, y, M, d, h, m, s},
    Q = jdc + 1/2;
    Z = Q // Floor;
    (*
      W = (Z - (1867216 + 1/4))/(36524 + 1/4) // Floor;
      X = W/4 // Floor;
      A = Z + 1 + W - X;
      *)
    W = 0; X = 0;
    A = Z;
    B = A + 1524;
    C = (B - 1221/10)/(365 + 1/4) // Floor;
    D = (365 + 1/4) C // Floor;
    E = (B - D)/(306001/10000) // Floor;
    F = (306001/10000)E // Floor;
    d = B - D - F + (Q - Z);
    M = If[0 < E - 1 <= JDNMonths, E - 1, E - (JDNMonths + 1)];
    y = If[M <= 2, C - 4715, C - 4716];
    h = FractionalPart[d] 24;
    d = d // Floor;
    m = FractionalPart[h] 60;
    h = h // Floor;
    s = FractionalPart[m] 60;
    m = m // Floor;
    {y, M, d, h, m, s}
    ]
(*
julian  J  MeanTropicalYear  T
*)
(*
see notes for  julian  J  MeanTropicalYear  T
*)
MeanTropicalYearTTfunc[T_?NumericQ, julian, epoch_] :=
  
  If[epoch =!= Epoch[TT], Return[$Failed],
    365.2421896698 - 6.15359 10^-6 T - 7.29 10^-10 T^2 + 2.64 10^-10 T^3
    ]
(*
End julian
*)
End[]
EndPackage[]
