
(*
:Mathematica Version: 4.0
*)
(*
:Package Version: 1.0
*)
(*
:Copyright: public domain
*)
(*
:Name: timeDeduce`
*)
(*
:Title: timeDeduce
*)
(*
:Author: John Hendrickson (debguy@sourceforge.net)
*)
(*
:History: None
*)
(*
:Keywords: timeDeduce time Date Calendar
*)
(*
:Requirements: None
*)
(*
:Sources: all code is origional by author
*)
(*
:Summary: see also timeDeduce.notes.nb

	quick ex:
		 input: GMT,lon,tz,arieslon
		output: input + SHA,Ra,LHA ; in dn,tm or ang

	deduces what can be known without simulation
	(uses equality based equations of time)

	NOTE: theoretically done but hardly tested

	by: John Hendrickson (debguy@sourceforge.net), 2013, public domain
		credit to many authors of math in general
*)
(*
setup
*)
(*
(* suggested , see the mm book *)
If[False,
  (* On[MemoryConserve] *)(* may be causing crashes *)
  (* 
    turn off has similar spelling messages *)
  Off[General::"spell"];
  Unprotect[In, Out];
  Clear[In, Out];
  Protect[In, Out];
  $HistoryLength = 5;
  MemoryInUse[];
  ]
*)
(*
Needs["Miscellaneous`Geodesy`"]
*)
Needs["Months`Months`"]
BeginPackage[
    "timeDeduce`", {"Months`Months`", "Months`Common`MonthsCommon`"}];
(*
timeDeduce
*)
(*
setup
*)
Clear[timeDeduce, timeConvert, timeTimeRules, timeAngleRules, timeShow, 
    timeTest4, timeTest2, timeTestEq, timeTest3, timeTest3b, vars, varsT, 
    varsHA, varsRA, tm, dn, ang];
timeDeduce::usage := 
    "timeDeduce[cal:gregorian,pass:2]  shows time in formats known given data \
(ex. GMT, Ra).  Directions: use timeReset[cal] then set which of longitudes, \
hour angles, times you know.  Then use timeDeduce to fill in deduced. both \
day number and time or longitude are output.  show outputs a Input cell.  see \
also: DayNumberChange2 ToDegrees Convert.\n\nnotes: T can be set tm or dn, L \
and HA can be set ang or dn.  For beginners: all should  refer to same day so \
output is not confusing. tm are usual date format for cal, dn are day numbers \
from day 0 of cal not JDN longitude or hour ang [-180,180] degrees. sign of \
tm dn is significant.  ex of setting times (only need be set): \
GMT[tm]={x,...}; GHAstar[dn]=.; lon[ang]=x;  star can be sun.  see also: \
timeDeduce.notes.nb";
  timeShow::usage := "Use at any time to see current values.";
  timeReset::usage := "Start over, reset all time,ang to Null.";
  timeShow::usage := "Use timeShow at any time to see what is set";
  timeConvert::usage := "timeDeduce uses, but can be used at any time";
timeConvert::"unformatted" = "`1`";
timeDeduce::"errata" = "RA[ang]<->SHA[dn], only HA used for dn";
timeAngleRules::"dayless" = "`1` is from two longitudes";
timeTest4::"extrapolated" = "`1` taken from diff. days";
timeTest3::"extrapolated" = "`1` taken from diff. days";
(*
Needs["Miscellaneous`Units`"]
*)
Options[timeDeduce] = {daylessOk -> True, timePasses -> 2, ShowCell -> False};
  
Clear[
    vars, varsT, varsST, varsHA, varsRA, tm, dn, ang,
    midday, dayless,
    localtime, timezone, lon, arieslon,
    UTC, GMT, LMT, LAT, EOT,
    LST, LMST, AST, GST, AMST, GMST,
    GHAaries, GHAstar,
    LHAaries, LHAstar, LHAsun, SHAstar, SHAlm, SHAgm,
    RAstar, RAaries, RAlm, RAgm
    ];
(*
functions
*)
Begin["`Private`"];
timeReset[cal_:gregorian] := Module[{},
      (*Clear[lat, gdate, jdn, gdn, star, height];*)
      Clear[
        vars, varsT, varsST, varsHA, varsRA, tm, dn, ang,
        midday, dayless,
        localtime, timezone, lon, arieslon,
        UTC, GMT, LMT, LAT, EOT,
        LST, LMST, AST, GST, AMST, GMST,
        GHAaries, GHAstar,
        LHAaries, LHAstar, LHAsun, SHAstar, SHAlm, SHAgm,
        RAstar, RAaries, RAlm, RAgm
        ];
      (*vars = Sort@{lat, gdate, jdn, gdn, star, height};*)
      
      varsT = Sort@{GMT, LAT, LMT, localtime, timezone, UTC, EOT};
      varsST = Sort@{LST, LMST, AST, AMST, GMST, GST};
      varsHA = 
        Sort@{GHAaries, GHAstar, LHAaries, LHAstar, LHAsun, SHAstar, SHAlm, 
            SHAgm};
      varsRA = Sort@{RAstar, RAaries, RAlm, RAgm};
      varsL = Sort@{arieslon, lon};
      varsStr = Map[ToString, vars];
      varsTStr = Map[ToString, varsT];
      varsSTStr = Map[ToString, varsST];
      varsHAStr = Map[ToString, varsHA];
      varsRAStr = Map[ToString, varsRA];
      varsLStr = Map[ToString, varsL];
      Scan[(#[tm] = Null; #[dn] = Null) &, varsT];
      Scan[(#[tm] = Null; #[dn] = Null) &, varsST];
      Scan[(#[ang] = Null; #[dn] = Null) &, varsHA];
      Scan[(#[ang] = Null; #[dn] = Null) &, varsRA];
      Scan[(#[ang] = Null; #[dn] = Null) &, varsL];
      If[Head@Epoch[cal, cal, daystart] === Epoch, midday[dn] = 1/2,
        midday[dn] = 1/2 - Epoch[cal, cal, daystart]];
      (*midday is 0 for julian if attempted*)
      
      dayless = daylessOk /. Options[timeDeduce];
      Null
      ];
timeConvert[cal_:gregorian] := Module[{x},
      Scan[(
            If[#[dn] === Null && #[ang] =!= Null,
              #[dn] = (-(#[ang] - 180)/15)/24];
            If[#[ang] === Null && #[dn] =!= Null,
              #[ang] = (1 - #[dn]) 15 24 - 180];
            If[Abs@#[dn] >= 1 || Abs@#[ang] >= 180,
              Message[timeConvert::"unformatted", {#[dn], #[ang]}]; 
              Return[$Failed]];
            ) &, varsL];
      Scan[(
            If[#[dn] === Null && #[ang] =!= Null,
              #[dn] = #[ang]/360;
              #[dn] = ((IntegerPart@#[dn] + 1 - FractionalPart@#[dn])360/15)/
                  24];
            If[#[ang] === Null && #[dn] =!= Null,
              #[ang] = FractionalPart@#[dn] 24 15;
              #[ang] = IntegerPart@#[dn] + 360 - #[ang];
              ];
            ) &, varsRA];
      Scan[(
            If[#[dn] === Null && #[ang] =!= Null,
              #[dn] = (#[ang]/15)/24];
            If[#[ang] === Null && #[dn] =!= Null,
                #[ang] = #[dn]] 15 24;
            ) &, varsHA];
      Scan[(
            If[#[dn] === Null && #[tm] =!= Null,
              #[dn] = DayNumberFrom2[#[tm], cal, cal]];
            If[#[tm] === Null && #[dn] =!= Null,
              #[tm] = DayNumberTo2[#[dn], cal, cal]];
            ) &, varsT];
      Scan[(
            If[#[dn] === Null && #[tm] =!= Null,
              x = DayNumberFrom2[#[tm], cal, cal];
              #[dn] = IntegerPart@x + 1 - FractionalPart@x];
            If[#[tm] === Null && #[dn] =!= Null,
              x = #[dn]; x = IntegerPart@x + 1 - FractionalPart@x;
              #[tm] = DayNumberTo2[x, cal, cal]];
            ) &, varsST];
      Null
      ];
(*
note /; would cause mm to inf recurse and i doubt Solve could complete all \
tasks.

If[LMT[dn]===Null&&GMT[dn]=!=Null&&lon[dn]=!=Null,
    LMT[dn]:=GMT[dn]-lon[dn]+midday;once=True];
If[GMT[dn]===Null&&LMT[dn]=!=Null&&lon[dn]=!=Null,
    GMT[dn]:=LMT[dn]+lon[dn]-midday;once=True];
If[lon[dn]===Null&&LMT[dn]=!=Null&&GMT[dn]=!=Null,
    lon[dn]:=GMT[dn]-LMT[dn]+midday;once=True];
*)
(*
there is a dependancy tree of prefferred order (tm then ang) and also of not \
extrapolating lon.  but not enough to bother using sorting.  pass 2 "allows" \
lon equated from diff days (usu. ok)
*)
(*
using dn many share same formula though dn takes "getting used to".  i don't \
think anything is "easier" all said and done (ie, if days are separated and \
added back later: why?), also dn works +- days
*)
(*
lon/midday or lon/lon (t3 t4) have no sign in arithmetic and do not belong to \
either t1,t2 so: un-generically adjust sign t3 t4
	ex: if GMT is neg, then lon, midday are when added
*)
timeTest4[{t1_, t2_, t3_, t4_}, pass_] := Module[{once, sign}, once = False;
    If[t4[dn] == Null, Return[once]];
    If[t1[dn] === Null && t2[dn] =!= Null && t3[dn] =!= Null,
      sign = If[t2[dn] < 0, -1, 1];
      t1[dn] = t2[dn] - sign t3[dn] + sign t4[dn]; once = True];
    If[t2[dn] === Null && t1[dn] =!= Null && t3[dn] =!= Null,
      sign = If[t1[dn] < 0, -1, 1];
      t2[dn] = t1[dn] + sign t3[dn] - sign t4[dn]; once = True];
    If[Floor@t2[dn] === Floor@t1[dn] || (pass === 2 && dayless),
      If[t3[dn] === Null && t1[dn] =!= Null && t2[dn] =!= Null,
        sign = If[FractionalPart@t2[dn] - FractionalPart@t1[dn] < 0, -1, 1];
        t3[dn] = FractionalPart@t2[dn] - FractionalPart@t1[dn] + sign t4[dn];
        If[Floor@t2[dn] =!= Floor@t1[dn], 
          Message[timeTest4::"extrapolated", t4]];
        once = True];
      If[t4[dn] === Null && t2[dn] =!= Null && t3[dn] =!= Null,
        sign = If[FractionalPart@t1[dn] - FractionalPart@t2[dn] < 0, -1, 1];
        t4[dn] = sign t3[dn] - FractionalPart@t2[dn] + FractionalPart@t1[dn];
        If[Floor@t2[dn] =!= Floor@t1[dn], 
          Message[timeTest4::"extrapolated", t4]];
        once = True];
      ];
    once]
timeTest2[{t1_, t2_}] := Module[{once}, once = False;
    If[t1[dn] === Null && t2[dn] =!= Null,
      t1[dn] = t2[dn]; once = True];
    If[t2[dn] === Null && t1[dn] =!= Null,
      t2[dn] = t1[dn]; once = True];
    once]
timeTest3[{t1_, t2_, t3_}, pass_] := Module[{once, sign}, once = False;
    If[t1[dn] === Null && t2[dn] =!= Null && t3[dn] =!= Null,
      sign = If[t2[dn] < 0, -1, 1];
      t1[dn] = t2[dn] + sign t3[dn]; once = True];
    If[t2[dn] === Null && t1[dn] =!= Null && t3[dn] =!= Null,
      sign = If[t1[dn] < 0, -1, 1];
      t2[dn] = t1[dn] - sign t3[dn]; once = True];
    If[t3[dn] === Null && t1[dn] =!= Null && t2[dn] =!= Null,
      If[Floor@t2[dn] === Floor@t1[dn] || (pass === 2 && dayless),
        t3[dn] = FractionalPart@t1[dn] - FractionalPart@t2[dn];
        If[Floor@t2[dn] =!= Floor@t1[dn], 
          Message[timeTest3::"extrapolated", t3]];
        ];
      once = True];
    once]
timeTest3b[{t1_, t2_, t3_}] := Module[{once}, once = False;
    If[t1[dn] === Null && t2[dn] =!= Null && t3[dn] =!= Null,
      t1[dn] = t2[dn] + t3[dn]; once = True];
    If[t2[dn] === Null && t1[dn] =!= Null && t3[dn] =!= Null,
      t2[dn] = t1[dn] - t3[dn]; once = True];
    If[t3[dn] === Null && t1[dn] =!= Null && t2[dn] =!= Null,
      t3[dn] = t1[dn] - t2[dn];
      once = True];
    once]
timeTestEq[{t1_, t2_}] := Module[{once}, once = False;
    If[t1[dn] === Null && t2[dn] =!= Null,
      t1[dn] = t2[dn]; once = True];
    If[t2[dn] === Null && t1[dn] =!= Null,
      t2[dn] = t1[dn]; once = True];
    once]
timeTimeRules[pass_, cal_:gregorian] :=
  Module[{once}, once = False;
    
    If[timezone[dn] == Null && lon[dn] != Null,
      timezone[dn] = 
        IntegerPart@(-((Abs@lon[dn] - Abs@midday[dn]) 24 - 1/2))];
    If[localtime[dn] == Null && timezone[dn] != Null && GMT[dn] != Null, 
      localtime[dn] = GMT[dn] + timezone[dn] 24];
    If[GMT[dn] == Null && timezone[dn] != Null && localtime[dn] != Null, 
      GMT[dn] = localtime[dn] - timezone[dn] 24];
    
    Scan[(once = Or[once, timeTestEq[#]]) &, {{UTC, GMT}}];
    
    Scan[(once = Or[once, timeTest4[#, pass]]) &, {{LMT, GMT, lon, 
          midday}, {AST, GST, arieslon, midday}, {AMST, GMST, arieslon, 
          midday}, {LST, GST, lon, midday}, {LMST, GMST, lon, midday}, {LST, 
          AST, lon, arieslon}}];
    
    (* remember ST[dn] -> T[dn] on input *)
    
    Scan[(once = Or[once, timeTest3[#, pass]]) &, {{GMST, GMT, midday}}];
    Scan[(once = Or[once, timeTest3b[#]]) &, {{GST, EOT, GMST}}];
    
    once
    ]
(*
notes:
	EOT == GHAsun - GMHAsun
	ST(0) preceeds T(0), which is earlier arbitrary
*)
(*
	is there a way to get ST to T or HA ?  eot?
	? was sin cos better idea (too late, no days on either)
*)
timeAngleRules[ponce_, pass_, cal_:gregorian] := 
  Module[{once, sign, test}, once = False;
    
    (*remember RA[ang] -> SHA[dn] on input. RA does not always mean to aries*)

        Scan[(once = Or[once, timeTestEq[#]]) &, {{RAstar, SHAstar}, {RAlm, 
          SHAlm}, {RAgm, SHAgm}, {RAaries, LHAaries}}];
    
    If[LHAsun[dn] =!= Null && LHAstar[dn] === Null, LHAstar[dn] = LHAsun[dn]; 
      once = True];
    If[LHAsun === Null,
      sign = If[GST[dn] < 0, -1, 1];
      If[GHAstar[dn] === GST[dn] - sign midday[dn], LHAsun = LHAstar; 
        once = True];
      sign = If[LHAstar[dn] < 0, -1, 1];
      If[LHAstar[dn] === LST[dn] - sign midday[dn], LHAsun = LHAstar; 
        once = True];
      If[SHAlm[dn] === SHAstar[dn], LHAsun = LHAstar; once = True];
      ];
    
    (* LAT preceeds LHAsun , choice is arbitrary *)
    
    Scan[(once = Or[once, timeTest3[#, pass]]) &, {{LAT, LHAsun, midday}}];
    Scan[(once = Or[once, timeTest3b[#]]) &, {{GHAstar, GHAaries, SHAstar}}];
    Scan[(once = Or[once, timeTest4[#, pass]]) &, {{LHAstar, GHAstar, lon, 
          midday}}];
    
    test[{x1_, x2_}] := If[x2[dn] === Null && x1[dn] =!= Null,
        x2[dn] = IntegerPart@x1[dn] + 1 - FractionalPart@x1[dn];
        once = True];
    Scan[(
          (* day should not change *)
          test[{#[[1]], #[[2]]}];
          test[{#[[2]], #[[1]]}];
          sign = If[#[[1]][dn] < 0, -1, 1];
          
          If[#[[3]][dn] === Null && #[[1]][dn] =!= Null && #[[4]][dn] =!= 
                Null,
            #[[3]][dn] = sign #[[4]][dn] + FractionalPart@#[[1]][dn];
            If[#[[3]][dn] > 1, #[[3]][dn] = #[[3]][dn] - 1];
            once = True];
          
          If[#[[4]][dn] === Null && #[[1]][dn] =!= Null && #[[3]][dn] =!= 
                Null,
            #[[4]][dn] = sign #[[3]][dn] - FractionalPart@#[[1]][dn];
            If[#[[4]][dn] < 1, #[[4]][dn] = 1 + #[[4]][dn]];
            once = True];
          (* day may be deduced later, so do last of (first, any) pass *)
    
                If[dayless && ponce === False,
            If[once == False,
                
                If[#[[1]][dn] === Null && #[[3]][dn] =!= Null && #[[4]][dn] =!=
                       Null,
                  #[[1]][dn] = Abs@#[[3]][dn] - Abs@#[[4]][dn];
                  If[#[[1]][dn] < 1, #[[1]][dn] = 1 + #[[1]][dn]];
                  Message[timeAngleRules::"dayless", {#[[1]], #[[2]]}];
                  once = True]
                ];
            ];
          (* poll again *)
          test[{#[[1]], #[[2]]}];
          ) &, {{LHAaries, SHAlm, arieslon, lon}, {GHAaries, SHAgm, arieslon, 
          midday}}];
    once
    ]
timeDeduce[cal_:gregorian] :=
  Module[{i, j, once, pass, passes},
    passes = timePasses /. Options[timeDeduce];
    Do[
      If[pass === 2 && lon[dn] =!= Null && arieslon[dn] =!= Null && 
          midday =!= Null, Continue[]];
      i = timeConvert[cal];
      If[i == $Failed, Return[$Failed]];
      (* times before angles assuming T is accurate and has dn *)
      
      once = True;
      While[once, once = False;
        While[timeTimeRules[pass, cal], once = True];
        once = Or[once, timeAngleRules[once, pass, cal]]
        ];
      i = timeConvert[cal];
      , {pass, 1, passes}];
    ]
timeShow[cal_:gregorian] :=
  
  Module[{i, j, cell, once, t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, 
      tab},
    tab = Map[(
            Table[{
                #[[1]][[i]] <> "[" <> ToString@#[[3]] <> "]",
                InputForm@(#[[2]][[i]][#[[3]]])
                }, {i, 1, Length@#[[1]]}]
            ) &, {
          {varsLStr, varsL, dn}, {varsHAStr, varsHA, dn}, {varsRAStr, varsRA, 
            dn}, {varsTStr, varsT, dn}, {varsSTStr, varsST, dn}, {varsLStr, 
            varsL, ang}, {varsHAStr, varsHA, ang}, {varsRAStr, varsRA, 
            ang}, {varsTStr, varsT, tm}, {varsSTStr, varsST, tm}
          }];
    tab = Partition[Flatten@tab, 2];
    tab = Prepend[tab, {"calendar", cal}];
    
    If[ShowCell /. Options[timeDeduce],
      once = False;
      cell = Cell[
          
          StringJoin@
            Table[{If[once, "\n", once = True; ""] <> ToString@tab[[i, 1]] <> 
                  ":=" <> ToString@InputForm@tab[[i, 2]] <> ";"}
              , {i, 1, Length@tab}],
          "Text"];
      NotebookWrite[EvaluationNotebook[], cell]
      ,
      tab]
    ]
(*
end package
*)
End[];
EndPackage[];
(*
On[General::"spell"];
*)
