(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     75085,       1867]*)
(*NotebookOutlinePosition[     75753,       1891]*)
(*  CellTagsIndexPosition[     75709,       1887]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Package Info", "Subsubsection"],

Cell[":Name: Months`", "Text"],

Cell[":Context: Months`Months`", "Text"],

Cell["\<\
:Title: gregorian and julian Months , a few Astronomical \
conversions\
\>", "Text"],

Cell["\<\
:Summary:

\tNeeds[Months`Months`]

\ta few extra astronomical / calendar / epoch features
\t
\tsupports multiple calendars
\t\t(julian, gregorian, synodic, anomalistic)
\t\tsupports JDN conveniently
\t\tNEW: supports day number from 0 of any calendar
\t
\tconversion all-way between supported calendars

\tsupports multiple epochs and epoch generation
\t\tconvert between some epochs without calendar support
\t\t\tie, J2000.0, JDN
\t\tcreate new epoch with a datum + other epoch
\t\tcan work with TAI/clock epochs
\t\t\tbut cannot import from solar only

\tnote defaults: calendar->gregorian , DayNumber->julian

\thas a few other convenience functions

\tlong names used (use Alt-K should bring up a list as you type)
\treadable code, no crypic calendar formulas at all.

\tIt should be first noted JPL doesn't use \"a calendar\" for time
\t\t(their Horizons product uses simulation maybe?)
\tHOWEVER due to heavy use in (the past of) astronomy,
\tone may (often) be drawn to need date conversion.

\t(if made to be right within one day by adjusting orig
\t formula: it's a new calendar wrong to stored dates)

\tSEE: Months.notes.nb for warnings, discusssions, tests
\t
\ttodo: simple SHA GHA etc time conversion / print
\t\t  synodic is NOT accurate to moon, uses average\
\>", "Text"],

Cell[":Author: John D. Hendrickson", "Text"],

Cell[":Mathematica Version 4.0", "Text"],

Cell[":Package Version: 1.0.4", "Text"],

Cell["\<\
:History:
\t1.0 had a small oversight but that effeced date a year after made.
\t1.0.2 improved packaging
\t1.0.4 above
\tsee Warnings\
\>", "Text"],

Cell[":Keywords: Months, Month, Date, Day, Week", "Text"],

Cell[":Requirements: None.", "Text"],

Cell[":Copyright: None.", "Text"],

Cell[TextData[{
  ":Warnings:\n\n\tMost of Months has no thorough checking, but has some \
miles on it.\n\tBut consider NewEpochFrom untested and hardly used.\n\t\n\t\
see Months.notes.nb for general calendar warnings\n\t(historic and ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " calendar swapping,\n\t Day Number does not swap, general stuff - \
awareness)\n\n\tThe `calendar` package assumes that at {1752,9,2}\n\t\
mathematica changes calendars (you may not know!)\n\tMonths` stays on same \
calendar."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["outer  var  decl or init", "Subsubsection"],

Cell["see notes nb about Calendar and adding a calendar", "Text"],

Cell["\<\
Clear[\"Months`Months`\",\"Months`anomalistic`\",\"Months`julian`\",\
\"Months`gregorian`\",\"Months`Common`MonthsCommon`\"];\
\>", "Text",
  InitializationCell->True],

Cell[BoxData[
    \(\(Clear["\<Months`Months`\>"];\)\)], "Input",
  InitializationCell->True],

Cell["\tMonths common", "Text"],

Cell[BoxData[
    \(\(BeginPackage["\<Months`Months`\>", \
"\<Months`Common`MonthsCommon`\>"];\)\)], "Input",
  InitializationCell->True],

Cell["lunisolarOrder", "Text"],

Cell[BoxData[
    \(\(Options[Months] = {calendar \[Rule] gregorian, 
          DayNumberBy \[Rule] julian};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Months::usage = "\<gregorian and julian Months , a few Astronomical \
conversions.  if you load calendar` after a few help items will change in \
default context search path.\>";\)\), "\[IndentingNewLine]", 
    \(\(Months::"\<unformatted\>" = "\<`1`\>";\)\), "\[IndentingNewLine]", 
    \(\(MonthsGenericText := "\<used in Months`, see Months.m\>";\)\)}], \
"Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(CDN::usage = "\<calendar day number\>";\)\), "\[IndentingNewLine]", 
    \(\(DayNumberChange2::usage = "\<[dn, calFrom, calTo], convert day number \
from one calendar to another\>";\)\), "\[IndentingNewLine]", 
    \(\(DayNumberDifference::usage = "\<[date1|dn, date2|dn, cal], positive \
when date2 is later than date1.  note date is always converted as from cal to \
julian and diff is always julian days.  arg may be dn (elsewhere a plain \
number might be fixed to be date for year, not here). See also DND2\>";\)\), \
"\[IndentingNewLine]", 
    \(\(DayNumberDifference2::usage = "\<[date1|dn, date2|dn, cal], positive \
when date2 is later than date1.  diff is in days on same calendar dates are \
in (use cal to specify which cal dt are).  see also DND \>";\)\), "\
\[IndentingNewLine]", 
    \(\(DayStart::usage = "\<beginning of day for given calendar (julian \
starts day at 12 pm).  please use Epoch[cal,cal,daystart]\>";\)\), "\
\[IndentingNewLine]", 
    \(\(DayNumber::usage = "\<To count of days since epoch -4712 given date \
or day count from a date.  remember julian day starts at 12 pm and is read as \
12 not as 0.  DN goes from and to julian or gregorian but always uses julian \
day count (convert if you need otherwise)\>";\)\), "\[IndentingNewLine]", 
    \(\(DayNumber2::usage = "\<DayNumber2 works cal to cal (any supported \
calendar) using the calendar day 0 epochs, while DayNumber always works from \
JDN epoch.  it has no classic \"Y<400 or no non-negative julian\" problems \
(see Months.notes.nb)\>";\)\), "\n", 
    \(\(DayNumberChange2::usage = "\<[dn,from cal,to cal] change day number \
to other cal, like calendarChange2 but in dn\>";\)\), "\[IndentingNewLine]", 
    \(\(DayNumberChange2Update::usage = "\<like DayNumberChange2 except From \
will be updated with gleened info if possible, NewEpochFrom needs \
this\>";\)\), "\[IndentingNewLine]", 
    \(\(DayNumberFrom::usage = "\<[date, From cal , to cal] , count of days \
relative to julian epoch\>";\)\), "\n", 
    \(\(DayNumberFrom2::usage = "\<[date, From cal , to cal] , 2 is a new \
rewrite of old other.  count of days relative to epoch of calendar\>";\)\), "\
\[IndentingNewLine]", 
    \(\(DayNumberFromCenturyFraction::usage = "\<see \
DayNumberToCenturyFraction, ToCenturyFraction\>";\)\), "\n", 
    \(\(DayNumberTo::usage = "\<[date, To cal , from number] , count of days \
relative to julian dn epoch\>";\)\), "\[IndentingNewLine]", 
    \(\(DayNumberTo2::usage = "\<[date, To cal , from number] , 2 is a new \
rewrite of old other.  count of days relative to epoch of calendar\>";\)\), "\
\[IndentingNewLine]", 
    \(\(DayNumberToCenturyFraction::usage = "\<julian date expressed as \
century fraction is used as 'T' in some popular astronomy equations, ie \
J2000.0  see: julianTojulianCenturyFraction\>";\)\), "\[IndentingNewLine]", 
    \(\(DaysFromCenturyFraction::usage = 
        MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(DaysToCenturyFraction::usage = 
        MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(DayNumberDotForm::usage = "\<Floor@y + any date == Jn.m\>";\)\), "\
\[IndentingNewLine]", 
    \(\(Epoch::usage = "\<[this,this|other,type] store or get data. the \
offset, a time from which a set of dates originate. note it matters if data \
is solar (solar) or clock.\>";\)\), "\[IndentingNewLine]", 
    \(\(JDN::usage = "\<julian day number\>";\)\), "\[IndentingNewLine]", 
    \(\(JDNMonths::usage = "\<is what julian day count algorithm thinks \
(12)\>";\)\), "\[IndentingNewLine]", 
    \(\(MJDN::usage = "\<modified julian day number\>";\)\), "\
\[IndentingNewLine]", 
    \(\(MoonPhase::usage = "\<[date,cal,False] text for moon phase or \
cardinal if True\>";\)\), "\[IndentingNewLine]", 
    \(\(NewEpochFrom::usage = "\<[From,To,{args},show,cals|{},types|{}] try \
to fill in the skeleton of new Epoch[To] using Epoch[From].  NOTE: not well \
tested, not used much yet.  First assign Epoch[new,x] one date you know that \
has lookup in Epoch[From] (also daystart,CDN,JDN if you can).  Say you know \
gregorian JDN of new To, let args=={gregorian,JDN} to imply Epoch[From and \
To,gregorian,JDN] exist.  Having that the rest can be attempted (depending on \
avail. conversion and data).  show gives Output cell which can be evaluated.\n\
\ntodo: only one {args} can be given per attempt but running 2x same To it is \
not destructive (maybe running with another arg imports more).\n\ntodo: will \
TT import to solar? no unless same year.  if for some reason both solar and \
clock should import (if coincidnt) the only way will be to make two new \
epochs; one from each type, and merge.  see Months.notes.nb about solar v. \
clock calendaring\>";\)\), "\[IndentingNewLine]", 
    \(\(PrettyDate::usage = "\<pretty formatted date, no options \
avail\>";\)\), "\[IndentingNewLine]", 
    \(\(SecondsEpoch::usage = "\<add these to change seconds epoch on same \
calendar\>";\)\), "\[IndentingNewLine]", 
    \(\(ToCenturyFraction::usage = "\<convert date to century fraction (this \
is T for MeanTropicalYearTTfunc but note it does not change epoch, so julian \
yr -2000 is 0 (TT 2000) and gregorian is in wrong epoch to count days in, i \
beleive)\>";\)\), "\n", 
    \(\(ToCenturyFraction2::usage = "\<uses DayNumberFrom2 otherwise same \
(epoch of cal assumption)\>";\)\), "\[IndentingNewLine]", 
    \(\(YearFromCenturyFraction::usage = 
        MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(YearToCenturyFraction::usage = 
        MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(ZodiacNames::usage = MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(ZodiacNamesStrings::usage = 
        MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(anomalistic::usage = "\<is package Months`anomalistic` adding dates \
by average anomalistic year (begins jan 3 at noon).  it is provided to \
exhibit dates without leap years (also w/o monthless), each year is exactly \
an anomalistic year.\>";\)\), "\[IndentingNewLine]", 
    \(\(calendarChange2::usage = "\<[dt, calFrom, calTo], convert date from \
one calendar to another\>";\)\), "\[IndentingNewLine]", 
    \(\(calTransformFrom::usage = "\<\>";\)\), "\[IndentingNewLine]", 
    \(\(calTransformTo::usage = "\<\>";\)\), "\[IndentingNewLine]", 
    \(\(dayofweekjulian::usage = "\<note julian day starts at 12\>";\)\), "\
\[IndentingNewLine]", 
    \(\(gregorian::usage = "\<is package in Months`gregorian adding that \
feature set\>";\)\), "\[IndentingNewLine]", 
    \(\(gregorianTojulian::usage = "\<uses DayNumber not Daynumber2.  see \
calendarChange2\>";\)\), "\[IndentingNewLine]", 
    \(\(gregorianTojulian2::usage = 
        MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(julian::usage = "\<is a package in Months`julian` adding that feature \
set\>";\)\), "\[IndentingNewLine]", 
    \(\(julianDayNumber::usage = "\<days passed on new julian calendar since \
-4712, see Epochs et al\>";\)\), "\[IndentingNewLine]", 
    \(\(julianTogregorian::usage = "\<uses DayNumber not Daynumber2.  see \
calendarChange2\>";\)\), "\[IndentingNewLine]", 
    \(\(julianTogregorian2::usage = 
        MonthsGenericText;\)\[IndentingNewLine] (*\(lunisolar::usage = "\<was \
not finished to save time, not worth it, see notes\>";\)*) \), "\
\[IndentingNewLine]", 
    \(\(moonphase::usage = "\<[jdn] uses synodic EpochDaysUT\>";\)\), "\
\[IndentingNewLine]", 
    \(\(squared::usage = "\<example calendar package, a counter example to \
synodicToDays and synodic\>";\)\), "\[IndentingNewLine]", 
    \(\(synodic::usage = "\<synodic is a supported cal that uses ave mean \
synodic month. it isnt' too accurate per lunation but is fine for showing \
moon phase over many years\>";\)\), "\[IndentingNewLine]", 
    \(\(synodicEpochDaysTT::usage = "\<see synodicEpochDaysUT\>";\)\), "\
\[IndentingNewLine]", 
    \(synodicEpochDaysUT::usage = \(\*"\""can\ be\ used\ to\ make\ a\ new\ \
synodic\ calendar\ \((new\ timescale\ and\ epoch)\) . \ \ current\ is\ \
DND2[\((sedUT[0] - 1)\), gregorian, gregorian]\), \ \(is\ UT2000 . \ 
          with\ that\ use\ calendarChange2\ to\ edit\ epochs\ for\ ea . \ 
          cal . \*"\""\[IndentingNewLine]\(synodicFromDays::usage = "\<[jdn] \
given jdn return lunations, uses synodicEpochDaysUT (inaccurate \
eq)\>"\);\)\), "\[IndentingNewLine]", 
    \(\(synodicToDays::usage = "\<[lun] given lunations return days, uses \
synodicEpochDaysUT (inaccurate eq).  shows a counter example to use of \
synodic and squared\>";\)\), "\[IndentingNewLine]", 
    \(\(synodic::usage = "\<is a package in Months`synodic` adding synodic \
caledar\>";\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(DayTab::usage = "\<list of month lengths MonthTabs needs to yeild one \
for given year, shape may differ among calendars\>";\)\), "\n", 
    \(\(DaysInMonth::usage = "\<Days in month for given year, 0 \
allowed.\>";\)\), "\n", 
    \(\(DaysInYear::usage = "\<Number of Days in given Year.\>";\)\), "\n", 
    \(\(FromSeconds::usage = "\<uses julianDayNumber epoch\>";\)\), "\
\[IndentingNewLine]", 
    \(\(FromSeconds2::usage = "\<uses calendar epoch\>";\)\), "\
\[IndentingNewLine]", 
    \(\(FromSecondsmm2::usage = "\<uses mathematica 1900 epoch, from \
gregorian only\>";\)\), "\[IndentingNewLine]", 
    \(\(FromSecondsanomalistic::usage = "\<uses y2k MeanTropicalYear[] with \
no further correction\>";\)\), "\[IndentingNewLine]", 
    \(\(GreatestDay::usage = MonthsGenericText;\)\), "\n", 
    \(\(IsLeapYear::usage = "\<True,False given year.\>";\)\), "\n", 
    \(\(LesserYear::usage = MonthsGenericText;\)\), "\[IndentingNewLine]", 
    \(\(MonthDays::usage = "\<Sum of days in first n months for given year \
inclusive, 0 allowed.\>";\)\), "\n", 
    \(\(MonthNames::usage = "\<Names of month, as Symbols.\>";\)\), "\
\[IndentingNewLine]", 
    \(\(MonthNamesjulian::usage = "\<Intercalarus (Mercedonius) are never in \
julian Day Number calendar. see julian`DayTab for possible older layout\>";\)\
\), "\n", 
    \(\(MonthNamesStrings::usage = "\<Names of month, as Strings.\>";\)\), "\n\
", 
    \(\(MonthTab::usage = "\<List of days in each month for given year.\>";\)\
\), "\[IndentingNewLine]", 
    \(\(MonthOfYear::usage = "\<Contains the names of the months of the year.\
\>";\)\), "\n", 
    \(\(MonthOfYearDay::usage = "\<returns Month given year and \
yearday.\>";\)\), "\n", 
    \(\(MonthsInYear::usage = "\<usually 12, see synodic\>";\)\), "\
\[IndentingNewLine]", 
    \(\(ToSeconds::usage = "\<uses julianDayNumber epoch\>";\)\), "\
\[IndentingNewLine]", 
    \(\(ToSeconds2::usage = "\<uses calendar epoch\>";\)\), "\
\[IndentingNewLine]", 
    \(\(ToSecondsmm2::usage = "\<uses mathematica 1900 epoch, from gregorian \
only\>";\)\), "\[IndentingNewLine]", 
    \(\(ToSecondsanomalistic::usage = "\<uses y2k MeanTropicalYear[] with no \
further correction\>";\)\), "\[IndentingNewLine]", 
    \(\(YearDay::usage = "\<Yearday given year, month and day of month.\>";\)\
\), "\[IndentingNewLine]", 
    \(\(YearFromDays::usage = "\<return signed lesseryear * Year given signed \
days (to find 365*Y it exludes all implied leaps, and knows -y time is in \
reverse).  see also: leapdays (the inverse per say)\>";\)\), "\
\[IndentingNewLine]", 
    \(\(dayofweek::usage = "\<Day of Week given date, numeric, given by \
julian Day Number conversion, the first cardinal day of the week is \
Sunday\>";\)\), "\[IndentingNewLine]", 
    \(\(dayofweekName::usage = "\<day of week string\>";\)\), "\
\[IndentingNewLine]", 
    \(\(daystoSunday::usage = "\<experimental and iterative, use something \
else.  Days to Sunday, Jan 1 2012 ; used by dayofweek.\>";\)\), "\
\[IndentingNewLine]", 
    \(\(dtsdayofweek::usage = "\<experimental and iterative, use something \
else.  Day of Week given date, numeric, uses daystoSunday\>";\)\), "\
\[IndentingNewLine]", 
    \(\(dtsdayofweekName::usage = "\<experimental and iterative, use \
something else.  Name of Day of Week given date (text), uses daystoSunday\>";\
\)\), "\[IndentingNewLine]", 
    \(\(dtsdaysbetween::usage = "\<experimental and iterative, use something \
else.  Days between two dates (from noon to noon), use DaysBetween, uses \
daystoSunday\>";\)\), "\[IndentingNewLine]", 
    \(\(epochlist::usage = "\<List of all epoch  \
Epoch[cal,cal,types,data]\>";\)\), "\[IndentingNewLine]", 
    \(\(epochs::usage = "\<List of all epoch triplets \
Epoch[cal,cal,types]\>";\)\), "\[IndentingNewLine]", 
    \(\(fixdate::usage = "\<a minor detail to preen {y,M,d,h,m,s}.  it checks \
calendar.\>";\)\), "\[IndentingNewLine]", 
    \(\(fixdatebegin::usage = "\<a minor detail of fixdate\>";\)\), "\
\[IndentingNewLine]", 
    \(\(leapdays::usage = "\<given year a count of all leap days, inclusive.  \
with right year leap days is known.  with days and leap formula right year \
can be known\>";\)\), "\[IndentingNewLine]", 
    \(\(types::usage = "\<List of all epoch data types \
Epoch[cal,cal,types,data]\>";\)\)}], "Input",
  InitializationCell->True],

Cell["\<\
(* Miscellaneous`Calendar` defines Day name messages *)
(* MonthUsageText:=\"is a month name\"; too many messages we all know \
*)\
\>", "Text"],

Cell[BoxData[{
    \(\(DayNames::usage = "\<Names of days, as Symbols (context is \
Miscellaneous`Calendar`).\>";\)\), "\n", 
    \(\(DayNamesStrings::usage = MonthsGenericText;\)\), "\n", 
    \(\(ZodiacNames = {Aries, Taurus, Gemini, Cancer, Leo, Virgo, Libra, 
          Scorpio, Sagittarius, Capricorn, Aquarius, Pisces};\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(julianMonths = 13;\)\(\ \)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Clear[gregorianTommSecondsEpoch2]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(gregorianTommSecondsEpoch2;\)\)], "Input",
  InitializationCell->True],

Cell["\<\
these typical values are for recent epochs, ie, TT 2000, i imagine \
and are not up to date\
\>", "Text"],

Cell[BoxData[
    \(\(MeanTropicalYearTTfunc::usage = "\<[ jcf, julian, Epoch[TT,TT] ]  \
results show diffreneces due to precession \"for distant past\".  est length \
of yr in days given julian century fraction from TT 2000, see Months.notes.nb \
example.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(IncrementToward::usage = "\<For [y,x] return n closer to y than \
x.\>";\)\), "\[IndentingNewLine]", 
    \(\(IncrementAway::usage = "\<For [y,x] return n further from y than x.\n\
\t(from side of y or if zero to sign of x).\>";\)\), "\[IndentingNewLine]", 
    \(\(IncrementAwayAbove::usage = "\<For [y,x] return n further above y \
than x.\>";\)\), "\[IndentingNewLine]", 
    \(\(IncrementAwayBelow::usage = "\<For [y,x] return n further under y \
than x.\>";\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\({JDN, TT, TAI, TGC, SundayYear, MTY, 
        synodicMonth,  (*\(lunisolarMonth\)\(,\)*) julianToJDN, 
        julianFromJDN, JDNtoMJDN, JDNfromMJDN, date, type, solar, seasonal, 
        clock, lunar};\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["data  init", "Subsubsection"],

Cell[BoxData[
    \(\(DayStartJulian = 12;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(MeanSiderealDay = 23.9344699;\) (*apparent\ to\ stars*) \), "\n", 
    \(\(MeanTropicalYear = 365.242190402;\) (*current, \ 
      loose\  - 1.9  s/yr*) \n (*MSY\ at\ epoch\ J2000  .0 = January\ 1, 
      2000, 12 : \(00 : 00\ TT\)*) \), "\n", 
    \(\(MeanSolarYear = 365.256363004;\) (*apparent\ to\ sun*) \), "\n", 
    \(\(MeanAnomalisticYear = 365.259636;\) (*earth\ sun\ orbit*) \), "\n", 
    \(\(VernalEquinoxYear = 365.2424;\) (*see\ mty*) \), "\n", 
    \(\(synodicMonth = 29.530588853;\) (*apparent*) \)}], "Input",
  InitializationCell->True],

Cell["\<\
meanLunarYeriod=27.321582;
synodicMonthFC=(synodicMonthReal \
synodicMonth)/(synodicMonth-synodicMonthReal);\
\>", "Text",
  InitializationCell->True],

Cell[BoxData[{
    \(\(centuryDays[gregorian] = 146463/4;\)\), "\[IndentingNewLine]", 
    \(\(yearDays[gregorian] = 
        centuryDays[gregorian]/100;\)\), "\[IndentingNewLine]", 
    \(\(centuryDays[julian] = 36525;\)\), "\[IndentingNewLine]", 
    \(\(yearDays[julian] = 
        centuryDays[julian]/100;\)\), "\[IndentingNewLine]", 
    \(\(centuryDays[anomalistic] = 
        MeanAnomalisticYear\ 100;\)\), "\[IndentingNewLine]", 
    \(\(yearDays[anomalistic] = 
        MeanAnomalisticYear;\)\[IndentingNewLine]\), "\[IndentingNewLine]", 
    \(\(centuryDays[anomalistic] = 
        MeanAnomalisticYear\ 100;\)\), "\[IndentingNewLine]", 
    \(\(yearDays[anomalistic] = MeanAnomalisticYear;\)\)}], "Input",
  InitializationCell->True],

Cell["\<\
epochs vary if days flow through or from 0 and daystart is not \
known from one or a few epochs - it really must be said, thus daystart is \
doubly necessary\
\>", "Text"],

Cell["assume GMT or usual for epoch (could store else and check)", "Text"],

Cell["\thow do you count seconds from 1900 on solar? eraser used?", "Text"],

Cell[BoxData[{
    \(\(Epoch[FromDate, FromDate, 
          text] = "\<FromDate ToDate, mm\>";\)\), "\[IndentingNewLine]", 
    \(\(Epoch[FromDate, FromDate, type] = solar;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[FromDate, FromDate, JDN] = 
        2415020 + 1/2;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[FromDate, gregorian, date] = {1900, 1, 1, 0, 0, 0};\)\), "\n", 
    \(\(Epoch[FromDate, julian, date] = {1899, 12, 20, 0, 0, 
          0};\)\)}], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Epoch[SundayYear, SundayYear, 
          text] = "\<a year who's first day is Sunday NOT AN EPOCH\>";\)\), "\
\[IndentingNewLine]", 
    \(\(Epoch[SundayYear, SundayYear, type] = 
        solar;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[SundayYear, gregorian, date] = {2012, 1, 1, 0, 0, 
          0};\)\), "\n", 
    \(\(Epoch[SundayYear, julian, date] = {\(-4712\), 1, 1, DayStartJulian, 
          0, 0};\)\)}], "Input",
  InitializationCell->True],

Cell["-0.000739 is ~ 63.84 sec TT", "Text",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Epoch[TT, TT, 
          text] = "\<TT day count 0\>";\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TT, TT, daystart] = 1/2;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TT, TT, type] = clock;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TT, TT, date] = {2000, 1, 1, DayStartJulian, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(Epoch[TT, gregorian, date] = {2000, 1, 1, DayStartJulian - 1, 58, 
        56.1503966152668`}; \ 
    Epoch[TT, julian, date] = {1999, 12, 19, DayStartJulian - 1, 58, 
        56.1503966152668`}\ ;\)}], "Input",
  InitializationCell->True],

Cell["\<\
see Months.notes.nb  about  epochs
\tTAI TT GPS all tick same rate, all clock. but don't use \"clock\" for TGC \
rename it.\
\>", "Text"],

Cell[BoxData[{
    \(\(Epoch[TAI, TAI, text] = "\<TAI 1977\>";\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TAI, daystart] = 0;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TAI, type] = clock;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TAI, date] = {1977, 1, 1, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TAI, JDN] = 2443144.5003725;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TT, JDN] = 
        Epoch[TAI, TAI, JDN] + 
          32.184/\((24\ 60^2)\);\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, GPS, JDN] = 
        Epoch[TAI, TAI, JDN] - 
          19/\((24\ 60^2)\);\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TT, MJDN] = 
        JDNtoMJDN[Epoch[TAI, TAI, JDN]];\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TT, date] = {1977, 1, 1, 0, 0, 
          32.184};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, TGC, date] = {1977, 1, 1, 0, 0, 
          32.184};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, gregorian, date] = {1977, 1, 1, 0, 0, 
          32.18401372432709`};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[TAI, julian, date] = {1976, 12, 19, 0, 0, 
          32.18401372432709`};\)\)}], "Input",
  InitializationCell->True],

Cell["\tsolar, specified by JDN, not a running clock", "Text"],

Cell[BoxData[{
    \(\(Epoch[J2000, J2000, 
          text] = "\<J2000 is gregorian 2000 at 12 noon (J is not julian, \
should be G)\>";\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000, J2000, daystart] = 1/2;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000, J2000, type] = solar;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000, J2000, JDN] = 2451545;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000, gregorian, date] = {2000, 1, 1, 12, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000, julian, date] = {1999, 12, 19, 12, 0, 0};\)\)}], "Input",\

  InitializationCell->True],

Cell["\<\
\twhat if one needed epoch that assumes any clock can be imported \
or offset from
\t0 of, but not any solar ?\
\>", "Text"],

Cell[BoxData[{
    \(\(Epoch[J2000c, J2000c, 
          text] = "\<J2000c is TT 2000 at 12 noon (J is not julian, should be \
G)\>";\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000c, J2000c, daystart] = 1/2;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000c, J2000c, type] = clock;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[J2000c, J2000c, JDN] = 2451545;\)\)}], "Input"],

Cell["\<\
\tGPS(TAI)=TAI-19
\tTT(TAI)=TAI+32.184 s
simple to say Epoch[] could define these as functions, but TAI epoch may \
contain solar,lunar,etc which won't be that easy, toseconds possible but new \
epoch would be easier\
\>", "Text"],

Cell[BoxData[
    \(\(ZodiacNamesStrings = Map[SymbolName, ZodiacNames];\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["begin  private", "Subsubsection"],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"];\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["helper  funs", "Subsubsection"],

Cell["\<\
>=0 or > 0 ? had previously made use of Sign[0]==0 for correctness \
for reminder and to add number line looping ability: but rather than explain \
such use i revert for simplicity, toward y==0 goes to 0\
\>", "Text"],

Cell[BoxData[
    \(IncrementToward[y_, x_] := 
      If[\((y >= 0 && x > y)\) || \((y <= 0 && x > y)\), 
        x - 1, \[IndentingNewLine]If[x =!= y, x + 1, 
          x]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(IncrementAway[y_, x_] := 
      If[\((y >= 0 && x > y)\) || \((y <= 0 && x > y)\), 
        x + 1, \[IndentingNewLine]If[x =!= y, x - 1, 
          x + Sign[x]]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(IncrementAwayAbove[y_, x_] := 
      If[\((y >= 0 && x > y)\) || \((y <= 0 && x > y)\), 
        x + 1, \[IndentingNewLine]If[x =!= y, x - 1, 
          x + 1]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(IncrementAwayBelow[y_, x_] := 
      If[\((y >= 0 && x > y)\) || \((y <= 0 && x > y)\), 
        x + 1, \[IndentingNewLine]If[x =!= y, x - 1, 
          x - 1]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ to\ Jn . m\ , \ 
      floor' s\ y\ *) \)\(\[IndentingNewLine]\)\(DayNumberDotForm[
        y_, {Y_, M_, d_, h_, m_, s_}] := \((Floor@y + h/24 + m/1440 + 
          s/86400)\)\)\)\)], "Input",
  InitializationCell->True],

Cell["\trequires latent binding of DaysInMonthjulian for julian", "Text"],

Cell[BoxData[
    \(fixdatebegin[dtp_] := 
      Module[{x, dt}, \[IndentingNewLine]dt = dtp; \[IndentingNewLine]If[
          IntegerQ@dt, dt = {dt}]; \[IndentingNewLine]If[Head@dt =!= List, 
          Message[Months::"\<unformatted\>", dt]; 
          Return[$Failed]]; \[IndentingNewLine]x = 
          Length@dt; \[IndentingNewLine]If[x > 6, 
          Message[Months::"\<unformatted\>", dt]; 
          Return[$Failed]]; \[IndentingNewLine]S = False; 
        If[x \[Equal] 6, S = Last@dt; 
          dt[\([6]\)] = 
            0]; \[IndentingNewLine]Scan[\((If[\[Not] IntegerQ@#, 
                Message[Months::"\<unformatted\>", #]; Return[$Failed]])\) &, 
          dt]; \[IndentingNewLine]If[S =!= False, 
          If[NumericQ@S, dt[\([6]\)] = S, 
            Message[Months::"\<unformatted\>", s]; 
            Return[$Failed]]]; \[IndentingNewLine]dt\[IndentingNewLine]]\)], \
"Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["minor  conversions", "Subsubsection"],

Cell[BoxData[
    \(julianTogregorian[dt_] := 
      DayNumberTo[DayNumberFrom[dt, julian, julian], gregorian, 
        julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(gregorianTojulian[dt_] := 
      DayNumberTo[DayNumberFrom[dt, gregorian, julian], julian, 
        julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(julianTogregorian2[dt_] := 
      calendarChange2[dt, julian, gregorian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(gregorianTojulian2[dt_] := 
      calendarChange2[dt, gregorian, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(julianToJDN[dn_?NumericQ] := dn + 1721058\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(julianFromJDN[dn_?NumericQ] := dn - 1721058\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(JDNtoMJDN[dn_?NumericQ] := dn - \((2400000 + 1/2)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(JDNfromMJDN[dn_?NumericQ] := dn + \((2400000 + 1/2)\)\)], "Input",
  InitializationCell->True],

Cell["just add or subtract seconds epoch but only on same calendar", "Text"],

Cell[BoxData[
    \(gregorianTommSecondsEpoch2 := 
      DayNumberDifference2[\[IndentingNewLine]Epoch[gregorian, gregorian, 
            date], \[IndentingNewLine]Epoch[FromDate, gregorian, 
            date], \[IndentingNewLine]gregorian] 24\ 60^2\)], "Input",
  InitializationCell->True],

Cell["\<\
calTo is the new , calFrom is other to import from
\t\tif args is TT (clock) it may ignore solar
\t\tif user later supplies a solar also, must run again\
\>", "Text"],

Cell[BoxData[
    \(YearToCenturyFraction[jdt_?NumericQ, cal_] := 
      jdt\ \((yearDays[cal]/centuryDays[cal])\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysToCenturyFraction[jdt_?NumericQ, cal_] := 
      jdt/centuryDays[cal]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearFromCenturyFraction[jdn_?NumericQ, cal_] := 
      jdn\ \((centuryDays[cal]/yearDays[cal])\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysFromCenturyFraction[jdn_?NumericQ, cal_] := 
      jdn\ centuryDays[cal]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToCenturyFraction[jdt_, cal_] := 
      DayNumberFrom[jdt, cal, cal]/centuryDays[cal]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToCenturyFraction2[jdt_, cal_] := 
      DayNumberFrom2[jdt, cal, cal]/centuryDays[cal]\)], "Input",
  InitializationCell->True],

Cell["\<\
may vary between 29.272 and 29.833 ? eq. shows parts of sec rise \
over 800yr\
\>", "Text"],

Cell["\<\
days between conjunction of sun and moons, dark/new moon ? from TT \
2000+N\[Equal]0 epoch\
\>", "Text"],

Cell[BoxData[
    \(synodicEpochDaysTT[m_] = 
      5.597661 + 29.5305888610\ m + \((102.026\ 10^\(-12\))\) m^2\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(synodicEpochDaysUT[m_] = 
      5.597661 + 29.5305888610\ m + \((102.026\ 10^\(-12\))\) m^2 - 
        0.000739 - \((235\ 10^\(-12\))\) m^2\)], "Input",
  InitializationCell->True],

Cell["\<\
moon days using JDN no calendar shows a counter-example to use of \
synodic\
\>", "Text"],

Cell[BoxData[
    \(synodicFromDays[n_] := 
      3.7601335599440503`*^9\ \((29.530588861\[InvisibleSpace] - 
            0.000023062870593228412\ \@\(1.6395229113969543`*^12 - \((n - \
Epoch[J2000, J2000, JDN])\)\))\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(synodicToDays[m_] := 
      synodicEpochDaysUT[m] + Epoch[J2000, J2000, JDN]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(moonphase[days_] := 
      FractionalPart@synodicFromDays[days]\ synodicMonth // 
        MoonPhase\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["NewEpochFrom", "Subsubsection"],

Cell["\<\
If[Head@Epoch[calFrom,othersymb,#]===Epoch,Continue[]];
\tin mm there isn't a good sparse array support per say
\tif, ex julian=1, Epoch[len] could be listed given len
\thowever if >>\"file\" the data would be illegible\
\>", "Text"],

Cell[BoxData[
    \(epochlist[] := 
      Module[{i}, \[IndentingNewLine]Table[\(\(DownValues[
                  Epoch]\)[\([i]\)]\)[\([1]\)] /. 
            h_[Epoch[x_, y_, z_, ___]] \[Rule] {x, y, z}, {i, 2, 
            Length@DownValues[Epoch]}]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(epochs[] := 
      Module[{tab, i}, tab = epochlist[]; 
        Union[tab[\([All, 1]\)], tab[\([All, 2]\)]]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(types[] := 
      Module[{tab, i}, tab = epochlist[]; 
        Union[tab[\([All, 3]\)], tab[\([All, 3]\)]]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(NewEpochFrom[calFrom_, calTo_, args_List, show_:  True, pcals_: {}, 
        ptypes_: {}] := \[IndentingNewLine]Module[{i, j, typeArg, dnc2, dnd2, 
          tab, tcals1, tcals2, tcals, ttypes, types, cals, calOther, 
          typeOther, x, y, z, epochType, eptT, eptF, eptO, cell, 
          once}, \[IndentingNewLine]cals = pcals; types = ptypes; 
        epochType = type; \[IndentingNewLine]If[
          cals \[Equal] {}, \[IndentingNewLine]tcals1 = \(Cases[
                epochlist[], {calFrom, _, _}]\)[\([All, 
                2]\)]; \[IndentingNewLine]cals = 
            Union[tcals1, 
              tcals1]]; \[IndentingNewLine] (*see\ notes\ below\
*) \[IndentingNewLine]If[types \[Equal] {}, tab = epochlist[]; 
          ttypes = tab[\([All, 3]\)]; \[IndentingNewLine]types = 
            Complement[
              Union[ttypes, ttypes], {text, epochType, 
                daystart}]]; \[IndentingNewLine]If[
          Head@Epoch[calTo] === 
            Epoch, \[IndentingNewLine]Epoch[calTo, calTo, text] = 
            ToString@calTo]; \[IndentingNewLine]If[args \[Equal] Null, 
          Return[$Failed]]; \[IndentingNewLine]calArg = 
          First@args; \[IndentingNewLine] (*Sequence @@ 
              Rest@\ may\ be\ needed*) \[IndentingNewLine]typeArg = 
          Last@args; \[IndentingNewLine]Print@{cals, types, calArg, typeArg, 
            epochType}; \[IndentingNewLine]x = 
          Epoch[calTo, calArg, typeArg]; \[IndentingNewLine]If[
          Head@x === Epoch (*\(||\)\(Head@Epoch[calFrom, calArg, typeArg] === 
                Epoch\)*) , 
          Return[$Failed]]; \[IndentingNewLine]If[\[Not] NumericQ@x, 
          x = DayNumberFrom2[x, calArg, 
              calArg]]; \[IndentingNewLine]If[\[Not] NumericQ@x, 
          Return[$Failed]]; \[IndentingNewLine]If[
          Head@Epoch[calTo, calTo, epochType] === Epoch && 
            Head@Epoch[calArg, calArg, epochType] =!= Epoch, 
          Epoch[calTo, calTo, epochType] = 
            Epoch[calArg, calArg, epochType]]; \[IndentingNewLine]eptT = 
          Epoch[calTo, calTo, epochType]; \[IndentingNewLine]eptF = 
          Epoch[calFrom, calFrom, epochType]; \[IndentingNewLine]If[
          Head@eptT === Epoch || Head@eptF \[Equal] Epoch, 
          Return[$Failed]]; \[IndentingNewLine]dnc2 = 
          DayNumberChange2Update[x, calTo, calFrom, 
            types]; \[IndentingNewLine]If[dnc2 === $Failed, 
          Return[$Failed]]; \[IndentingNewLine] (*dnd2\ is\ diff\ calTo, 
          calFrom; \ dnc2 == \((x - diff)\)*) \[IndentingNewLine]dnd2 = 
          x - dnc2; \[IndentingNewLine]Print@{x, dnc2, "\<dist: \>", 
            dnd2}; \[IndentingNewLine]\[IndentingNewLine] (*\ 
          list\ calFrom\ contents\ \((iterate\ by\ calFrom, calOther, 
              typeOther)\)\ \
*) \[IndentingNewLine]Do[\[IndentingNewLine]calOther = 
            cals[\([i]\)]; \[IndentingNewLine]Do[\[IndentingNewLine]typeOther \
= types[\([j]\)]; \[IndentingNewLine]Print@{"\<trying: \>", calOther, 
                typeOther}; \[IndentingNewLine]If[
              calOther \[Equal] calArg && typeArg == typeOther, 
              Continue[]]; \[IndentingNewLine] (*\ 
              keep\ existing\ *) \[IndentingNewLine]If[
              Head@Epoch[calTo, calOther, typeOther] \[NotEqual] Epoch, 
              Continue[]]; \[IndentingNewLine] (*\ 
              plus\ define\ clock + day\ no; \ 
              check\ last\ see\ why\ l8tr*) \[IndentingNewLine]x = 
              Epoch[calFrom, othersymb, typeOther]; \[IndentingNewLine]If[
              Head@x === Epoch, 
              Continue[]]; \[IndentingNewLine]If[\[Not] NumericQ@
                  x, \[IndentingNewLine]x = 
                DayNumberFrom2[x, calOther, 
                  calOther]; \[IndentingNewLine]If[\[Not] NumericQ@x, 
                Return[$Failed]]; \[IndentingNewLine]typeOther = 
                CDN]; \[IndentingNewLine]Epoch[calTo, calOther, typeOther] = 
              x - dnd2; \[IndentingNewLine] (*need\ to\ fill\ in\ what\ can\ \
be; \ why\ not\ check\ diff*) \[IndentingNewLine]y = 
              DayNumberChange2Update[x - dnd2, calTo, calOther, 
                types]; \[IndentingNewLine]If[
              y - \((x - dnd2)\) \[NotEqual] dnd2, 
              Print@{"\<conversion check not equal skipped: \>", calTo, dnd2, 
                  calOther, \((x - dnd2)\) - y}; 
              Clear@Epoch[calTo, calOther, #]; 
              Continue[]]; \[IndentingNewLine]\[IndentingNewLine]eptO = 
              Epoch[calOther, calOther, epochType]; \[IndentingNewLine]If[
              Head@eptO \[Equal] Epoch, 
              Return[$Failed]]; \[IndentingNewLine]\[IndentingNewLine]Print@"\
\<ADDED\>"; \[IndentingNewLine]If[eptT == eptO, 
              
              Continue[]]; \[IndentingNewLine]\[IndentingNewLine] (*more\ \
optional; \ 
              see\ note3*) \[IndentingNewLine]Print@"\<types different\>"; \
\[IndentingNewLine]If[{clock, solar} == 
                Union[eptT, eptO], \[IndentingNewLine]x = 
                First@DayNumberTo2[Epoch[calTo, calOther, typeOther], 
                    gregorian, gregorian]; \[IndentingNewLine]y = 
                First@DayNumberTo2[Epoch[calFrom, calOther, typeOther], 
                    gregorian, gregorian]; \[IndentingNewLine]If[
                NumericQ@x && NumericQ@y && x \[Equal] y, 
                AppendTo[Epoch[calTo, calTo, epochType], eptO]; 
                Continue[]]; \[IndentingNewLine]x = 
                Epoch[calTo, calTo, JDN]/\((365 + 1/4)\) // 
                  Floor; \[IndentingNewLine]y = 
                Epoch[calFrom, calFrom, JDN]/\((365 + 1/4)\) // 
                  Floor; \[IndentingNewLine]If[
                NumericQ@x && NumericQ@y && x \[Equal] y, 
                AppendTo[Epoch[calTo, calTo, epochType], eptO]; 
                Continue[]]]; \[IndentingNewLine]If[{clock, lunar} == 
                  Union[eptT, eptO] || {lunar, solar} == 
                  Union[eptT, eptO], \[IndentingNewLine]x = 
                Epoch[calTo, calTo, JDN]; \[IndentingNewLine]y = 
                Epoch[calFrom, calFrom, JDN]; \[IndentingNewLine]If[
                NumericQ@x && NumericQ@y, \[IndentingNewLine]x = 
                  First@DayNumberTo2[x, lunar, lunar]; \[IndentingNewLine]y = 
                  First@DayNumberTo2[y, lunar, lunar]; \[IndentingNewLine]If[
                  NumericQ@x && NumericQ@y && x \[Equal] y, 
                  AppendTo[Epoch[calTo, calTo, epochType], eptO]; 
                  Continue[]]]]; \[IndentingNewLine] (*forget\ the\ entry\ it\
\ may\ be\ too\ inaccurate\ due\ to\ dropped\ \
seconds*) \[IndentingNewLine]Clear@
              Epoch[calTo, 
                calOther, #]; \[IndentingNewLine]Print@"\<forgot\>";\
\[IndentingNewLine], {j, 1, Length@types}], {i, 1, 
            Length@cals}\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]If[
          show, \[IndentingNewLine]once = False; \[IndentingNewLine]cell = 
            Cell[StringJoin@
                Map[\((\[IndentingNewLine]If[once, "\<\n\>", 
                          once = 
                            True; "\<\>"]\[IndentingNewLine] <> "\<Epoch[\>"\
\[IndentingNewLine] <> ToString[#[\([1]\)]] <> "\<,\>"\[IndentingNewLine] <> 
                        ToString[#[\([2]\)]] <> "\<,\>"\[IndentingNewLine] <> 
                        ToString[#[\([3]\)]] <> "\<]:=\>"\[IndentingNewLine] <> 
                        ToString[
                          Epoch[Sequence @@ #] // 
                            InputForm] <> "\<;\>"\[IndentingNewLine])\) &, \
\[IndentingNewLine]Cases[
                    Table[\(\(DownValues[Epoch]\)[\([i]\)]\)[\([1]\)] /. 
                        h_[Epoch[x_, y_, z_, ___]] \[Rule] {x, y, z}, {i, 2, 
                        Length@
                          DownValues[
                            Epoch]}], {calTo, _, _}]\[IndentingNewLine]], \
"\<Text\>"]; \[IndentingNewLine]NotebookWrite[EvaluationNotebook[], 
            cell]\[IndentingNewLine]];\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Day Number 2", "Subsubsection"],

Cell["\<\
see Months.notes.nb section: an important note about any calendar \
conversion, DayStart
see Months.notes.nb section: \"Day Number 2\"
\tLesserYear[calFrom] ie 365
the below method works any linear scale if cal.nb can can return leapdays[] \
and YearFromDays[] (months are optional). works if days are inserted to \
calendarize (can do M inserts or both iff ld[] YFD[] do it).  Head checking \
alows calTranform to click in as calendar w/o defining the whole deal.\
\>", \
"Text"],

Cell[BoxData[
    \(DayNumberFrom2[dtp_, calFrom_, 
        calTo_] := \[IndentingNewLine]Module[{y, M, d, h, m, s, dt, Y, ds, f, 
          lesseryear}, \[IndentingNewLine]If[
          calFrom =!= calTo, \[IndentingNewLine]dt = 
            calendarChange2[dtp, calFrom, calTo], \[IndentingNewLine]dt = 
            fixdate[dtp, calTo]]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]ds = 
          Epoch[calTo, calTo, daystart]; 
        If[\[Not] NumericQ@ds, 
          ds = 0]; \[IndentingNewLine]{y, M, d, h, m, s} = 
          dt; \[IndentingNewLine]d = 
          d + \((h + \((m + s/60)\)/60)\)/24; \[IndentingNewLine]f = 
          If[M === 0, 0, MonthDays[y, M - 1, calTo]]; \[IndentingNewLine]If[
          Head@f =!= MonthDays, d = d + f]; \[IndentingNewLine]d = d - 1; 
        Y = y; \[IndentingNewLine]d = d - ds; \[IndentingNewLine]If[y < 0, 
          Y += 1; d = \(-\((DaysInYear[y, calTo] - 
                  d)\)\)]; \[IndentingNewLine]If[
          Head@LesserYear[calTo] =!= LesserYear, \[IndentingNewLine]d = 
            LesserYear[calTo]\ Y\  + 
              If[y === 0 || y === \(-1\), 0, 
                  leapdays[y - Sign@y, calTo]]\ leapammount[calTo] + 
              d]; \[IndentingNewLine]f = 
          calTransformFrom[d, calTo]; \[IndentingNewLine]If[
          Head@f =!= calTransformFrom, f, d]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["M=MonthsInYear[y,calTo]-M+1;", "Text"],

Cell["\<\
1) get right year, leaps and day follow.  2) just call cal \
functions to fill in rest.
if YearFromDays is defined then necessarily leaps leapammount LesserYear \
DaysInYear is numeric
if calTransformTo is used it must make date format indepotent, useable with \
DNF2 (ie, days as moon count), but units and use can vary\
\>", "Text"],

Cell[BoxData[
    \(DayNumberTo2[dtp_?NumericQ, calFrom_, calTo_] := 
      Module[{y, M, d, h, m, s, f, leaps, days, sign, Y, 
          ave}, \[IndentingNewLine]If[
          calFrom =!= calTo, \[IndentingNewLine]f = 
            DayNumberChange2[dtp, calFrom, calTo], \[IndentingNewLine]f = 
            dtp]; \[IndentingNewLine]ds = Epoch[calTo, calTo, daystart]; 
        If[\[Not] NumericQ@ds, ds = 0]; \[IndentingNewLine]f = 
          f + ds + 1; \[IndentingNewLine]sign = 
          If[Sign[f] \[Equal] 0, 1, Sign[f]]; \[IndentingNewLine]days = 
          Abs@f; \[IndentingNewLine] (*\ see\ note\ 1\ , \ 
          timescales\ *) \[IndentingNewLine]y = 
          YearFromDays[f, calTo]; \[IndentingNewLine]If[
          Head@y === YearFromDays, \[IndentingNewLine]d = sign\ days; 
          y = 0, \[IndentingNewLine]If[
            sign >= 0, \[IndentingNewLine]leaps = 
              leapdays[y, calTo] leapammount[calTo]; \[IndentingNewLine] (*\ 
              see\ note\ 2\ *) \[IndentingNewLine]If[IsLeapYear[y, calTo], 
              leaps = leaps - 
                  leapammount[calTo]]; \[IndentingNewLine]If[\[Not] NumbericQ@
                  leaps, leaps = 0]; \[IndentingNewLine]d = 
              days - LesserYear[calTo]\ y - leaps; \[IndentingNewLine]If[
              Floor@d > DaysInYear[y, calTo], d -= DaysInYear[y, calTo]; 
              y = y + 1]\[IndentingNewLine], \[IndentingNewLine]leaps = \
\(-leapdays[y, calTo]\) leapammount[calTo]; \[IndentingNewLine]If[
              IsLeapYear[y, calTo], 
              leaps = leaps - leapammount[calTo]]; \[IndentingNewLine]d = 
              days - LesserYear[calTo]\ \((\(-y\) - 1)\) - 
                leaps; \[IndentingNewLine]d = 
              DaysInYear[y, calTo] - 
                d; \[IndentingNewLine] (*see\ note\ 3*) \[IndentingNewLine]If[
              d < 1, y = y - 1; \ 
              d += DaysInYear[y, 
                  calTo]];\[IndentingNewLine]]\[IndentingNewLine]]; \
\[IndentingNewLine]M = 
          MonthOfYearDay[y, Floor@d, calTo]; \[IndentingNewLine]If[
          Head@M =!= MonthOfYearDay, \[IndentingNewLine]f = 
            If[M === 0, 0, MonthDays[y, M - 1, calTo]]; \[IndentingNewLine]If[
            Head@f =!= MonthDays, d = d - f], \[IndentingNewLine]M = 
            0\[IndentingNewLine]]; \[IndentingNewLine]f = 
          calTransformTo[d, calTo]; \[IndentingNewLine]If[
          Head@f =!= calTransformTo, Return[f]]; \[IndentingNewLine]f = 
          d // FractionalPart; 
        h = f\ 24; \[IndentingNewLine]f = h // FractionalPart; 
        m = f\ 60; \[IndentingNewLine]f = m // FractionalPart; 
        s = f\ 60; \[IndentingNewLine]{d, m, h} = {d, m, h} // 
            Floor; \[IndentingNewLine]{y, M, d, h, m, 
          s}\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(calendarChange2[dtp_, calFrom_, 
        calTo_] := \[IndentingNewLine]Module[{dt, dn, 
          dnc}, \[IndentingNewLine]If[calFrom === calTo, 
          Return[dtp]]; \[IndentingNewLine]dt = 
          fixdate[dtp, calFrom]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]dn = 
          DayNumberFrom2[dt, calFrom, calFrom]; \[IndentingNewLine]If[
          dn \[Equal] $Failed, Return[$Failed]]; \[IndentingNewLine]dnc = 
          DayNumberChange2[dn, calFrom, calTo]; \[IndentingNewLine]If[
          dnc \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]DayNumberTo2[dnc, calTo, 
          calTo]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["note preffer From epoch over To and dn before date", "Text"],

Cell["\<\
see Months.notes.nb  epochs note 1.  fixme? unlike NewEpoch this \
likes a rigid type list.\
\>", "Text"],

Cell["\<\
if To is +5 ahead  of From, From-5 converts to To, left on timeline
if To is -5 behind of From, From+5 converts to To, right on timeline\
\>", \
"Text"],

Cell["\<\
if To is +5 ahead of From, From-5
\tif To and From are neg,
\t-from - 5 , if x is neg\
\>", "Text"],

Cell[BoxData[
    \(DayNumberChange2[dtp_?NumericQ, calFrom_, calTo_, 
        types_: {CDN, JDN, date}] := \[IndentingNewLine]Module[{x, 
          y}, \[IndentingNewLine]If[calFrom === calTo, 
          Return[dtp]]; \[IndentingNewLine]If[
          Intersection[types, {JDN}] =!= {}, \[IndentingNewLine]y = 
            Scan[\((\[IndentingNewLine]If[
                    Head@Epoch[calFrom, calFrom, #] =!= Epoch && 
                      Head@Epoch[calTo, calTo, #] =!= 
                        Epoch, \[IndentingNewLine]x = 
                      DayNumberDifference2[Epoch[calFrom, calFrom, #], 
                        Epoch[calTo, calTo, #], calTo]; \[IndentingNewLine]If[
                      NumericQ@x, 
                      Return[dtp - 
                          x]]]\[IndentingNewLine])\) &, {JDN}]]; \
\[IndentingNewLine]If[NumericQ@y, Return[y]]; \[IndentingNewLine]y = 
          Scan[\((\[IndentingNewLine]If[
                  Head@Epoch[calFrom, calTo, #] =!= Epoch && 
                    Head@Epoch[calTo, calTo, #] =!= 
                      Epoch, \[IndentingNewLine]x = 
                    DayNumberDifference2[Epoch[calFrom, calTo, #], 
                      Epoch[calTo, calTo, #], calTo]; \[IndentingNewLine]If[
                    NumericQ@x, Return[dtp - x]]]\[IndentingNewLine])\) &, 
            types]; \[IndentingNewLine]If[NumericQ@y, 
          Return[y]]; \[IndentingNewLine] (*\ 
          reverse\ lookup\ may\ have\ data\ the\ other\ does\ not\ yet\ so\ \
check\ *) \[IndentingNewLine]y = 
          Scan[\((\[IndentingNewLine]If[
                  Head@Epoch[calFrom, calFrom, #] =!= Epoch && 
                    Head@Epoch[calFrom, calTo, #] =!= 
                      Epoch, \[IndentingNewLine]x = 
                    DayNumberDifference2[Epoch[calFrom, calFrom, #], 
                      Epoch[calFrom, calTo, #], 
                      calFrom]; \[IndentingNewLine]If[NumericQ@x, 
                    Return[dtp - x]]]\[IndentingNewLine])\) &, 
            types]; \[IndentingNewLine]If[NumericQ@y, 
          Return[y]]; \
\[IndentingNewLine]Return[$Failed]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
NewEpochFrom needs this. fill in missing data for From that is \
obviously gleened.  user may have date and need JDN for some equation.  other \
funs do no cross-referencing or data expansion, assume this is unwanted, ie \
keeping epochs in jdn only.  likes rigid type list.\
\>", "Text"],

Cell[BoxData[
    \(DayNumberChange2Update[dtp_?NumericQ, calFrom_, calTo_, 
        types_: {CDN, JDN, date}] := \[IndentingNewLine]Module[{x, y, 
          dnd2}, \[IndentingNewLine]If[
          Head@Epoch[calFrom, calFrom, CDN] === Epoch && 
            Head@Epoch[calFrom, calFrom, date] =!= 
              Epoch, \[IndentingNewLine]x = 
            DayNumberFrom2[Epoch[calFrom, calFrom, date], calFrom, 
              calFrom]; \[IndentingNewLine]If[x =!= $Failed, 
            x = DayNumberTo2[x, calFrom, calFrom]]]; \[IndentingNewLine]If[
          Head@Epoch[calFrom, calFrom, CDN] =!= Epoch && 
            Head@Epoch[calFrom, calFrom, date] === 
              Epoch, \[IndentingNewLine]x = 
            DayNumberTo2[Epoch[calFrom, calFrom, CDN], calFrom, 
              calFrom]; \[IndentingNewLine]If[x =!= $Failed, 
            Epoch[calFrom, calFrom, date] = x]]; \[IndentingNewLine]If[
          Head@Epoch[calFrom, calTo, CDN] === Epoch && 
            Head@Epoch[calFrom, calTo, date] =!= 
              Epoch, \[IndentingNewLine]x = 
            DayNumberFrom2[Epoch[calFrom, calTo, date], calTo, 
              calTo]; \[IndentingNewLine]If[x =!= $Failed, 
            x = DayNumberTo2[x, calTo, calTo]]]; \[IndentingNewLine]If[
          Head@Epoch[calFrom, calTo, CDN] =!= Epoch && 
            Head@Epoch[calFrom, calTo, date] === 
              Epoch, \[IndentingNewLine]x = 
            DayNumberTo2[Epoch[calFrom, calTo, CDN], calTo, 
              calTo]; \[IndentingNewLine]If[x =!= $Failed, 
            Epoch[calFrom, calTo, date] = 
              x]]; \[IndentingNewLine]\[IndentingNewLine]y = 
          DayNumberChange2[dtp, calFrom, calTo, types]; \[IndentingNewLine]If[
          y \[Equal] $Failed, Return[$Failed]]; \[IndentingNewLine]dnd2 = 
          dtp - y;  (*\ delta, \ dnd2\ diff\ from\ to, \ 
          x\ in\ dnd2\ *) \[IndentingNewLine]\[IndentingNewLine]If[
          Head@Epoch[calFrom, calFrom, JDN] === Epoch && 
            Head@Epoch[calTo, calTo, JDN] =!= Epoch, \[IndentingNewLine]Epoch[
              calFrom, calFrom, JDN] = 
            Epoch[calTo, calTo, JDN] - dnd2]; \[IndentingNewLine] (*\ 
          policy\ if\ From\ has\ JDN\ already\ dont\ repeat\ it\ for\ every\ \
entry\ *) \[IndentingNewLine] (*\ \(If\)\([\)\(Epoch[calFrom, calTo, JDN] === 
              Epoch\ same\ as\ From\)\(\ \)*) \[IndentingNewLine]If[
          Head@Epoch[calFrom, calTo, CDN] === Epoch, 
          Epoch[calFrom, calTo, CDN] = 
            Epoch[calFrom, calFrom, CDN] + dnd2]; \[IndentingNewLine]If[
          Head@Epoch[calFrom, calTo, date] === 
            Epoch, \[IndentingNewLine]x = $Failed; \[IndentingNewLine]If[
            Epoch[calFrom, calTo, CDN] =!= Epoch, 
            x = Epoch[calFrom, calTo, CDN] + dnd2]; \[IndentingNewLine]If[
            Epoch[calFrom, calTo, JDN] =!= Epoch, 
            x = Epoch[calFrom, calTo, JDN]]; \[IndentingNewLine]If[
            x =!= $Failed, 
            Epoch[calFrom, calTo, date] = 
              DayNumberTo2[x, calTo, 
                calTo]];\[IndentingNewLine]]; \[IndentingNewLine]Return[
          y]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["DayNumberDifference", "Subsubsection"],

Cell[BoxData[
    \(DayNumberDifference[dt1_, dt2_, cal_] := 
      Module[{dta, dtb}, \[IndentingNewLine]If[NumericQ@dt1, dta = dt1, 
          dta = DayNumberFrom[dt1, cal, julian]]; \[IndentingNewLine]If[
          NumericQ@dt2, dtb = dt2, 
          dtb = DayNumberFrom[dt2, cal, julian]]; \[IndentingNewLine]If[
          dta \[Equal] $Failed || dtb \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]If[
          Sign@dta === Sign@dtb, \((dtb - dta)\), 
          If[dtb > dta, 
              1, \(-1\)] \((Abs@dta + 
                Abs@dtb)\)]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberDifference2[dt1_, dt2_, cal_] := 
      Module[{dta, dtb, x, sign}, \[IndentingNewLine]If[NumericQ@dt1, 
          dta = dt1, 
          dta = DayNumberFrom2[dt1, cal, cal]]; \[IndentingNewLine]If[
          NumericQ@dt2, dtb = dt2, 
          dtb = DayNumberFrom2[dt2, cal, cal]]; \[IndentingNewLine]If[
          dta \[Equal] $Failed || dtb \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]If[
          Sign@dta === Sign@dtb, \((dtb - dta)\), 
          If[dtb > dta, 
              1, \(-1\)] \((Abs@dta + 
                Abs@dtb)\)]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
If[Sign@dta===Sign@dtb,Sign@dta \
(dtb-dta),If[dtb>dta,1,-1](Abs@dta+Abs@dtb)]\
\>", "Text"],

Cell["\<\
old unsigned
DayNumberDifference2[dt1_,dt2_,cal_]:=Module[{dta,dtb},
    dta=DayNumberFrom2[dt1,cal,cal];
    dtb=DayNumberFrom2[dt2,cal,cal];
    If[Sign@dta===Sign@dtb,Abs@(dta-dtb),Abs@(Abs@dta+Abs@dtb)]
    ]\
\>", "Text",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["daystoSunday  daysof ...", "Subsubsection"],

Cell["\<\
# days from Sunday, January 1 2012 (or any year who's 1st day is \
Sunday)\
\>", "Text"],

Cell["\<\
\tthis is only suitable if epoch is close for printing pretty date
\t\tthese are not rhobust
\twill be broken for anomalistic\
\>", "Text"],

Cell[BoxData[
    \(daystoSunday[dtp_, 
        cal_] := \[IndentingNewLine]Module[{y, M, d, h, m, s, dt, dd, yy, i, 
          Sunday, ds}, \[IndentingNewLine]dt = 
          fixdate[dtp, cal]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]ds = 
          Epoch[cal, cal, daystart]; 
        If[\[Not] NumericQ@ds, 
          ds = 0]; \[IndentingNewLine]{y, M, d, h, m, s} = 
          dt; \[IndentingNewLine]Sunday = \(Epoch[SundayYear, cal, 
              date]\)[\([1]\)]; \[IndentingNewLine]dd = 0; 
        i = 0; \[IndentingNewLine]yy = 
          IncrementToward[Sunday, y]; \[IndentingNewLine]If[
          y > Sunday && y =!= Sunday && y =!= \((Sunday - 1)\), 
          dd = DaysInYear[Sunday, cal]]; \[IndentingNewLine]While[\(++i\) < 
            10^7, \[IndentingNewLine]If[yy == Sunday, 
            Break[]]; \[IndentingNewLine]dd += 
            DaysInYear[yy, cal]; \[IndentingNewLine]yy = 
            IncrementToward[Sunday, 
              yy];\[IndentingNewLine]]; \[IndentingNewLine]i = 
          YearDay[y, M, d, cal]; \[IndentingNewLine]If[y < Sunday, 
          i = DaysInYear[y, cal] - i + 1, \(--i\)]; \[IndentingNewLine]If[
          y < Sunday && h < Mod[ds\ 24, 24], \(++i\)]; \[IndentingNewLine]If[
          y >= Sunday && 
            h < Mod[ds\ 24, 24], \(--i\)]; \[IndentingNewLine]dd + 
          i\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
\t\tdaystart was already counted if > 1 day, but daily hour still \
matters\
\>", "Text"],

Cell["\<\
d = daystoSunday[year, month, day, gregorian];++d;dow = Mod[d, \
7];If[year < SundayEpoch && dow > 0, dow = 7 - dow];If[dow == 0, dow = 7];dow\
\
\>", "Text"],

Cell[BoxData[
    \(dtsdayofweek[dtp_, 
        cal_] := \[IndentingNewLine]Module[{y, M, d, h, m, s, dt, dd, i, dow, 
          ds}, \[IndentingNewLine]dt = 
          fixdate[dtp, cal]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]{y, M, d, h, m, s} = 
          dt; \[IndentingNewLine]dd = 
          daystoSunday[dt, cal]; \[IndentingNewLine]If[
          y < \(Epoch[SundayYear, cal, date]\)[\([1]\)], 
          dd = \(-dd\)]; \[IndentingNewLine]Mod[dd, 7] + 
          1\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
dtsdayofweek[year_,month_,day_,gregorian]:=Mod[If[year<SundayEpoch,\
7,0]+daystoSunday[year,month,day,gregorian],7]+1\
\>", "Text",
  InitializationCell->True],

Cell[BoxData[
    \(dtsdayofweekName[dt_, 
        cal_] := \(DayNamesStrings[
          cal]\)[\([dtsdayofweek[dt, cal]]\)]\)], "Input",
  InitializationCell->True],

Cell["don't use this", "Text"],

Cell[BoxData[
    \(dtsdaysbetween[dtp1_, dtp2_, cal_] := 
      Module[{y, M, d, h, m, s, y2, M2, d2, h2, m2, s2, dt1, dt2, di, df, 
          Sunday}, \[IndentingNewLine]Sunday = \(Epoch[SundayYear, cal, 
              date]\)[\([1]\)]; \[IndentingNewLine]dt1 = 
          fixdate[dtp1, cal]; \[IndentingNewLine]If[dt1 \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]dt2 = 
          fixdate[dtp2, cal]; \[IndentingNewLine]If[dt2 \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]{y, M, d, h, m, s} = 
          dt1; \[IndentingNewLine]{y2, M2, d2, h2, m2, s2} = 
          dt2; \[IndentingNewLine]di = 
          daystoSunday[dt1, cal]; \[IndentingNewLine]df = 
          daystoSunday[dt2, cal]; \[IndentingNewLine]If[
          Sign[y - Sunday] =!= Sign[y2 - Sunday], \[IndentingNewLine]di + df, 
          Abs[di - df]\[IndentingNewLine]]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["pretty  date", "Subsubsection"],

Cell["this could be per cal.  no need its not meant to do much", "Text"],

Cell["\<\
GMT is wrong if your TZ is not for GMT. post ante meridium \
(meridian, sun at noon)\
\>", "Text"],

Cell[BoxData[
    \(PrettyDate[dtp_List, 
        cal_] := \[IndentingNewLine]Module[{y, M, d, h, m, s, dt, x, dow, 
          tz}, \[IndentingNewLine]dt = 
          fixdate[dtp, cal]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]{y, M, d, h, m, s} = 
          dt; \[IndentingNewLine]tz = TimeZone[]; \[IndentingNewLine]dow = 
          dayofweekName[dt, 
            cal]; \[IndentingNewLine]dow <> "\<, \>" \
<> \[IndentingNewLine]\(MonthNamesStrings[
              cal]\)[\([M]\)] <> "\< \>" <> \[IndentingNewLine]ToString[
            d] <> "\< \>" <> \[IndentingNewLine]ToString[
            y] <> "\<.  \>" <> \[IndentingNewLine]ToString[
            If[\((h - 12)\) > 0, h - 12, 
              h]] <> "\<:\>" <> \[IndentingNewLine]If[
            m < 10, "\<0\>", "\<\>"] <> \[IndentingNewLine]ToString[
            m] <> "\<:\>" <> \[IndentingNewLine]If[
            s < 10, "\<0\>", "\<\>"] <> \[IndentingNewLine]ToString[
            s] <> "\< \>" <> \[IndentingNewLine]ToString[
            If[h \[GreaterEqual] 
                12, "\<p.m.\>", "\<a.m.\>"]] <> "\< \>" \
<> \[IndentingNewLine]ToString[
            Floor[tz]] <> "\<\[ThinSpace]\>" <> "\<GMT\>"\[IndentingNewLine]]\
\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(PrettyDate[cal_] := 
      PrettyDate[calendarChange2[Date[], gregorian, cal], cal]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MoonPhase[dtp_?NumericQ, 
        numeric_:  False] := \[IndentingNewLine]Module[{x}, 
        x = Mod[dtp, synodicMonth]; 
        If[x > \((7  synodicMonth/8)\) || x < \((synodicMonth/8)\), 
          Return[If[numeric, 1, "\<New Moon\>"]]]; \[IndentingNewLine]If[
          x >= \((synodicMonth/8)\) && x < \((3  synodicMonth/8)\), 
          Return[If[numeric, 
              2, "\<Waxing Crecent Moon\>"]]]; \[IndentingNewLine]If[
          x >= \((3\ synodicMonth/8)\) && x < \((5  synodicMonth/8)\), 
          Return[If[numeric, 3, "\<Full Moon\>"]]]; \[IndentingNewLine]If[
          x >= \((5\ synodicMonth/8)\) && x < \((7  synodicMonth/8)\), 
          Return[If[numeric, 
              4, "\<Wanning Crecent Moon\>"]]]; \
\[IndentingNewLine]Return[$Failed]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MoonPhase[dtp_List, cal_, 
        numeric_:  False] := \[IndentingNewLine]Module[{y, M, d, h, m, s, dt, 
          x}, \[IndentingNewLine]dt = 
          fixdate[dtp, cal]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]dt = 
          calendarChange2[dt, cal, synodic]; \[IndentingNewLine]{y, M, d, h, 
            m, s} = dt; \[IndentingNewLine]MoonPhase[
          d + h/24 + m/60 + s/60^2]\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MoonPhase[cal_] := 
      MoonPhase[calendarChange2[Date[], gregorian, cal], synodic]\)], "Input",\

  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Option  Arguement  Defaults  for  functions ,  calendar  or  \
DayNumberBy\
\>", "Subsubsection"],

Cell["\<\
these are for ease of use.  be wary DayNumber funs have diff \
default than Month ones.  a little confusing si vu ple\
\>", "Text"],

Cell[BoxData[
    \(MonthsInYear[] := 
      MonthsInYear[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthsInYear[year_] := 
      MonthsInYear[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayStart[] := DayStart[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNames[] := DayNames[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNamesStrings[] := 
      DayNamesStrings[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthNames[] := MonthNames[calendar\  /. \ Options[Months]]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(MonthNamesStrings[] := 
      MonthNamesStrings[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(fixdate[dtp_] := fixdate[dtp, calendar /. Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(fixdate[dtp_, x_] := $Failed\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayTab[{year, ___}] := 
      DayTab[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayTab[year_] := DayTab[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayTab[] := DayTab[calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(leapdays[year_] := 
      leapdays[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(leapdays[{year_, ___}] := 
      leapdays[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearFromDays[days_] := 
      YearFromDays[days, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearFromDays[{days_, ___}] := 
      YearFromDays[days, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(IsLeapYear[year_] := 
      IsLeapYear[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(IsLeapYear[{year_, ___}] := 
      IsLeapYear[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysInMonth[year_, month_] := 
      DaysInMonth[year, month, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysInMonth[{year_, month_, ___}] := 
      DaysInMonth[year, month, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthTab[year_] := 
      MonthTab[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthTab[{year_, ___}] := 
      MonthTab[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthDays[year_, month_] := 
      MonthDays[year, month, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthDays[{year_, month_, ___}] := 
      MonthDays[year, month, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysInYear[year_] := 
      DaysInYear[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysInYear[{year_, ___}] := 
      DaysInYear[year, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearDay[year_, month_, day_] := 
      YearDay[year, month, day, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearDay[{year_, month_, day_, ___}] := 
      YearDay[year, month, day, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthOfYearDay[year_, yearday_] := 
      MonthOfYearDay[year, yearday, 
        calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthOfYearDay[{year_, yearday_, ___}] := 
      MonthOfYearDay[year, yearday, 
        calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dayofweek[dt_] := 
      dayofweek[dt, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dayofweekName[dt_] := 
      dayofweekName[dt, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToSeconds[dtp_] := 
      ToSeconds[dtp, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(FromSeconds[secs_] := 
      FromSeconds[secs, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToSeconds2[dtp_] := 
      ToSeconds2[dtp, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(FromSeconds2[secs_] := 
      FromSeconds2[secs, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToSecondsmm2[dtp_] := 
      ToSecondsmm2[dtp, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(FromSecondsmm2[secs_] := 
      FromSecondsmm2[secs, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(daystoSunday[dt_] := 
      daystoSunday[dt, calendar /. Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dtsdayofweek[dt_] := 
      dtsdayofweek[dt, calendar /. Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dtsdayofweekName[dt_] := 
      dtsdayofweekName[dt, \ calendar /. Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dtsdaysbetween[dt1_, dt2_] := 
      dtsdaysbetween[dt1, dt2, calendar /. Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(calTransformFrom[days_] := 
      calTransformFrom[days, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(calTransformTo[days_] := 
      calTransformTo[days, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(PrettyDate[dtp_List] := 
      PrettyDate[dtp, calendar\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(PrettyDate[] := PrettyDate[calendar /. Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MoonPhase[] := MoonPhase[synodic]\)], "Input",
  InitializationCell->True],

Cell["only default here would be 0, not helpful", "Text"],

Cell[BoxData[
    \(Epoch[] := {epochs[], types[]}\)], "Input",
  InitializationCell->True],

Cell["julian is default for Day Number things", "Text"],

Cell[BoxData[
    \(DayNumberFrom[dtp_, tofrom_] := 
      DayNumberFrom[dtp, tofrom, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberTo[dtp_, tofrom_] := 
      DayNumberTo[dtp, tofrom, DayNumberBy\  /. \ Options[Months]]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(DayNumberFrom[dtp_] := 
      DayNumberFrom[dtp, DayNumberBy\  /. \ Options[Months], 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberTo[dtp_] := 
      DayNumberTo[dtp, DayNumberBy\  /. \ Options[Months], 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberFrom2[dtp_, tofrom_] := 
      DayNumberFrom2[dtp, tofrom, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberTo2[dtp_, tofrom_] := 
      DayNumberTo2[dtp, tofrom, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberFrom2[dtp_] := 
      DayNumberFrom2[dtp, DayNumberBy\  /. \ Options[Months], 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberTo2[dtp_] := 
      DayNumberTo2[dtp, DayNumberBy\  /. \ Options[Months], 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberChange2[dtp_] := 
      DayNumberChange2[dtp, DayNumberBy\  /. \ Options[Months], 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberChange2[dtp_, tofrom_] := 
      DayNumberChange2[dtp, tofrom, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberChange2Update[dtp_] := 
      DayNumberChange2Update[dtp, DayNumberBy\  /. \ Options[Months], 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberChange2Update[dtp_, tofrom_] := 
      DayNumberChange2Update[dtp, tofrom, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberDifference[dt1_, dt2_] := 
      DayNumberDifference[dt1, dt2, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberDifference2[dt1_, dt2_] := 
      DayNumberDifference[dt1, dt2, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearToCenturyFraction[jdt_] := 
      YearToCenturyFraction[jdt, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysToCenturyFraction[jdt_] := 
      DaysToCenturyFraction[jdt, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearFromCenturyFraction[jdn_] := 
      YearFromCenturyFraction[jdn, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysFromCenturyFraction[jdn_] := 
      DaysFromCenturyFraction[jdn, 
        DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToCenturyFraction[jdt_] := 
      ToCenturyFraction[jdt, DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToCenturyFraction2[jdt_] := 
      ToCenturyFraction[jdt, DayNumberBy\  /. \ Options[Months]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["End  Months", "Subsubsection"],

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Needs["\<Months`gregorian`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Needs["\<Months`julian`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Needs["\<Months`anomalistic`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Needs["\<Months`synodic`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(Needs["\<Months`squared`\>"]\)], "Input",
  InitializationCell->True],

Cell["Needs[\"Months`lunisolar`\"]", "Text",
  InitializationCell->True]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowSize->{771, 606},
WindowMargins->{{66, Automatic}, {14, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 37, 0, 42, "Subsubsection"],
Cell[1779, 53, 30, 0, 31, "Text"],
Cell[1812, 55, 40, 0, 31, "Text"],
Cell[1855, 57, 94, 3, 31, "Text"],
Cell[1952, 62, 1300, 40, 677, "Text"],
Cell[3255, 104, 44, 0, 31, "Text"],
Cell[3302, 106, 40, 0, 31, "Text"],
Cell[3345, 108, 39, 0, 31, "Text"],
Cell[3387, 110, 158, 6, 99, "Text"],
Cell[3548, 118, 57, 0, 31, "Text"],
Cell[3608, 120, 36, 0, 31, "Text"],
Cell[3647, 122, 33, 0, 31, "Text"],
Cell[3683, 124, 531, 10, 218, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4251, 139, 49, 0, 42, "Subsubsection"],
Cell[4303, 141, 65, 0, 31, "Text"],
Cell[4371, 143, 177, 4, 48, "Text",
  InitializationCell->True],
Cell[4551, 149, 93, 2, 27, "Input",
  InitializationCell->True],
Cell[4647, 153, 31, 0, 31, "Text"],
Cell[4681, 155, 137, 3, 27, "Input",
  InitializationCell->True],
Cell[4821, 160, 30, 0, 31, "Text"],
Cell[4854, 162, 151, 3, 27, "Input",
  InitializationCell->True],
Cell[5008, 167, 418, 7, 91, "Input",
  InitializationCell->True],
Cell[5429, 176, 8677, 137, 1723, "Input",
  InitializationCell->True],
Cell[14109, 315, 4351, 78, 891, "Input",
  InitializationCell->True],
Cell[18463, 395, 154, 4, 48, "Text"],
Cell[18620, 401, 369, 6, 75, "Input",
  InitializationCell->True],
Cell[18992, 409, 93, 2, 27, "Input",
  InitializationCell->True],
Cell[19088, 413, 94, 2, 27, "Input",
  InitializationCell->True],
Cell[19185, 417, 92, 2, 27, "Input",
  InitializationCell->True],
Cell[19280, 421, 115, 3, 31, "Text"],
Cell[19398, 426, 304, 5, 75, "Input",
  InitializationCell->True],
Cell[19705, 433, 529, 9, 91, "Input",
  InitializationCell->True],
Cell[20237, 444, 267, 5, 43, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[20541, 454, 35, 0, 42, "Subsubsection"],
Cell[20579, 456, 85, 2, 27, "Input",
  InitializationCell->True],
Cell[20667, 460, 577, 9, 123, "Input",
  InitializationCell->True],
Cell[21247, 471, 160, 5, 48, "Text",
  InitializationCell->True],
Cell[21410, 478, 743, 14, 155, "Input",
  InitializationCell->True],
Cell[22156, 494, 181, 4, 48, "Text"],
Cell[22340, 500, 74, 0, 31, "Text"],
Cell[22417, 502, 75, 0, 31, "Text"],
Cell[22495, 504, 493, 9, 91, "Input",
  InitializationCell->True],
Cell[22991, 515, 475, 10, 75, "Input",
  InitializationCell->True],
Cell[23469, 527, 71, 1, 31, "Text",
  InitializationCell->True],
Cell[23543, 530, 592, 11, 107, "Input",
  InitializationCell->True],
Cell[24138, 543, 147, 4, 48, "Text"],
Cell[24288, 549, 1197, 23, 203, "Input",
  InitializationCell->True],
Cell[25488, 574, 62, 0, 31, "Text"],
Cell[25553, 576, 597, 11, 107, "Input",
  InitializationCell->True],
Cell[26153, 589, 134, 4, 48, "Text"],
Cell[26290, 595, 373, 6, 75, "Input"],
Cell[26666, 603, 240, 6, 82, "Text"],
Cell[26909, 611, 115, 2, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[27061, 618, 39, 0, 42, "Subsubsection"],
Cell[27103, 620, 88, 2, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[27228, 627, 37, 0, 42, "Subsubsection"],
Cell[27268, 629, 227, 4, 48, "Text"],
Cell[27498, 635, 233, 5, 59, "Input",
  InitializationCell->True],
Cell[27734, 642, 241, 5, 59, "Input",
  InitializationCell->True],
Cell[27978, 649, 240, 5, 59, "Input",
  InitializationCell->True],
Cell[28221, 656, 240, 5, 59, "Input",
  InitializationCell->True],
Cell[28464, 663, 249, 5, 43, "Input",
  InitializationCell->True],
Cell[28716, 670, 73, 0, 31, "Text"],
Cell[28792, 672, 927, 18, 187, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[29756, 695, 43, 0, 42, "Subsubsection"],
Cell[29802, 697, 168, 4, 27, "Input",
  InitializationCell->True],
Cell[29973, 703, 168, 4, 27, "Input",
  InitializationCell->True],
Cell[30144, 709, 133, 3, 27, "Input",
  InitializationCell->True],
Cell[30280, 714, 133, 3, 27, "Input",
  InitializationCell->True],
Cell[30416, 719, 102, 2, 27, "Input",
  InitializationCell->True],
Cell[30521, 723, 104, 2, 27, "Input",
  InitializationCell->True],
Cell[30628, 727, 112, 2, 27, "Input",
  InitializationCell->True],
Cell[30743, 731, 114, 2, 27, "Input",
  InitializationCell->True],
Cell[30860, 735, 76, 0, 31, "Text"],
Cell[30939, 737, 290, 5, 75, "Input",
  InitializationCell->True],
Cell[31232, 744, 176, 4, 65, "Text"],
Cell[31411, 750, 155, 3, 27, "Input",
  InitializationCell->True],
Cell[31569, 755, 134, 3, 27, "Input",
  InitializationCell->True],
Cell[31706, 760, 157, 3, 27, "Input",
  InitializationCell->True],
Cell[31866, 765, 137, 3, 27, "Input",
  InitializationCell->True],
Cell[32006, 770, 146, 3, 27, "Input",
  InitializationCell->True],
Cell[32155, 775, 148, 3, 27, "Input",
  InitializationCell->True],
Cell[32306, 780, 101, 3, 31, "Text"],
Cell[32410, 785, 114, 3, 31, "Text"],
Cell[32527, 790, 154, 4, 27, "Input",
  InitializationCell->True],
Cell[32684, 796, 200, 4, 27, "Input",
  InitializationCell->True],
Cell[32887, 802, 99, 3, 31, "Text"],
Cell[32989, 807, 259, 5, 85, "Input",
  InitializationCell->True],
Cell[33251, 814, 137, 3, 27, "Input",
  InitializationCell->True],
Cell[33391, 819, 160, 4, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[33588, 828, 37, 0, 42, "Subsubsection"],
Cell[33628, 830, 243, 5, 82, "Text"],
Cell[33874, 837, 292, 6, 59, "Input",
  InitializationCell->True],
Cell[34169, 845, 169, 4, 27, "Input",
  InitializationCell->True],
Cell[34341, 851, 168, 4, 27, "Input",
  InitializationCell->True],
Cell[34512, 857, 8089, 136, 1659, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[42638, 998, 37, 0, 42, "Subsubsection"],
Cell[42678, 1000, 490, 10, 116, "Text"],
Cell[43171, 1012, 1444, 25, 331, "Input",
  InitializationCell->True],
Cell[44618, 1039, 44, 0, 31, "Text"],
Cell[44665, 1041, 344, 7, 82, "Text"],
Cell[45012, 1050, 2817, 48, 699, "Input",
  InitializationCell->True],
Cell[47832, 1100, 745, 13, 187, "Input",
  InitializationCell->True],
Cell[48580, 1115, 66, 0, 31, "Text"],
Cell[48649, 1117, 115, 3, 31, "Text"],
Cell[48767, 1122, 162, 4, 48, "Text"],
Cell[48932, 1128, 109, 4, 65, "Text"],
Cell[49044, 1134, 2170, 38, 411, "Input",
  InitializationCell->True],
Cell[51217, 1174, 298, 5, 65, "Text"],
Cell[51518, 1181, 3171, 54, 539, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[54726, 1240, 44, 0, 42, "Subsubsection"],
Cell[54773, 1242, 628, 12, 107, "Input",
  InitializationCell->True],
Cell[55404, 1256, 645, 13, 107, "Input",
  InitializationCell->True],
Cell[56052, 1271, 102, 3, 31, "Text"],
Cell[56157, 1276, 264, 8, 116, "Text",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[56458, 1289, 49, 0, 42, "Subsubsection"],
Cell[56510, 1291, 98, 3, 31, "Text"],
Cell[56611, 1296, 149, 4, 65, "Text"],
Cell[56763, 1302, 1442, 26, 347, "Input",
  InitializationCell->True],
Cell[58208, 1330, 99, 3, 31, "Text"],
Cell[58310, 1335, 168, 4, 48, "Text"],
Cell[58481, 1341, 584, 11, 155, "Input",
  InitializationCell->True],
Cell[59068, 1354, 169, 4, 48, "Text",
  InitializationCell->True],
Cell[59240, 1360, 165, 4, 27, "Input",
  InitializationCell->True],
Cell[59408, 1366, 30, 0, 31, "Text"],
Cell[59441, 1368, 919, 15, 235, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[60397, 1388, 37, 0, 42, "Subsubsection"],
Cell[60437, 1390, 72, 0, 31, "Text"],
Cell[60512, 1392, 108, 3, 31, "Text"],
Cell[60623, 1397, 1282, 24, 315, "Input",
  InitializationCell->True],
Cell[61908, 1423, 144, 3, 27, "Input",
  InitializationCell->True],
Cell[62055, 1428, 831, 15, 155, "Input",
  InitializationCell->True],
Cell[62889, 1445, 519, 9, 139, "Input",
  InitializationCell->True],
Cell[63411, 1456, 148, 4, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[63596, 1465, 107, 3, 42, "Subsubsection"],
Cell[63706, 1470, 141, 3, 48, "Text"],
Cell[63850, 1475, 131, 3, 27, "Input",
  InitializationCell->True],
Cell[63984, 1480, 142, 3, 27, "Input",
  InitializationCell->True],
Cell[64129, 1485, 116, 2, 27, "Input",
  InitializationCell->True],
Cell[64248, 1489, 116, 2, 27, "Input",
  InitializationCell->True],
Cell[64367, 1493, 137, 3, 27, "Input",
  InitializationCell->True],
Cell[64507, 1498, 122, 3, 27, "Input",
  InitializationCell->True],
Cell[64632, 1503, 141, 3, 27, "Input",
  InitializationCell->True],
Cell[64776, 1508, 119, 2, 27, "Input",
  InitializationCell->True],
Cell[64898, 1512, 89, 2, 27, "Input",
  InitializationCell->True],
Cell[64990, 1516, 130, 3, 27, "Input",
  InitializationCell->True],
Cell[65123, 1521, 117, 2, 27, "Input",
  InitializationCell->True],
Cell[65243, 1525, 112, 2, 27, "Input",
  InitializationCell->True],
Cell[65358, 1529, 134, 3, 27, "Input",
  InitializationCell->True],
Cell[65495, 1534, 141, 3, 27, "Input",
  InitializationCell->True],
Cell[65639, 1539, 142, 3, 27, "Input",
  InitializationCell->True],
Cell[65784, 1544, 149, 3, 27, "Input",
  InitializationCell->True],
Cell[65936, 1549, 138, 3, 27, "Input",
  InitializationCell->True],
Cell[66077, 1554, 145, 3, 27, "Input",
  InitializationCell->True],
Cell[66225, 1559, 155, 3, 27, "Input",
  InitializationCell->True],
Cell[66383, 1564, 162, 3, 27, "Input",
  InitializationCell->True],
Cell[66548, 1569, 134, 3, 27, "Input",
  InitializationCell->True],
Cell[66685, 1574, 141, 3, 27, "Input",
  InitializationCell->True],
Cell[66829, 1579, 151, 3, 27, "Input",
  InitializationCell->True],
Cell[66983, 1584, 158, 3, 27, "Input",
  InitializationCell->True],
Cell[67144, 1589, 138, 3, 27, "Input",
  InitializationCell->True],
Cell[67285, 1594, 145, 3, 27, "Input",
  InitializationCell->True],
Cell[67433, 1599, 158, 3, 27, "Input",
  InitializationCell->True],
Cell[67594, 1604, 165, 3, 27, "Input",
  InitializationCell->True],
Cell[67762, 1609, 174, 4, 27, "Input",
  InitializationCell->True],
Cell[67939, 1615, 181, 4, 43, "Input",
  InitializationCell->True],
Cell[68123, 1621, 132, 3, 27, "Input",
  InitializationCell->True],
Cell[68258, 1626, 140, 3, 27, "Input",
  InitializationCell->True],
Cell[68401, 1631, 134, 3, 27, "Input",
  InitializationCell->True],
Cell[68538, 1636, 140, 3, 27, "Input",
  InitializationCell->True],
Cell[68681, 1641, 136, 3, 27, "Input",
  InitializationCell->True],
Cell[68820, 1646, 142, 3, 27, "Input",
  InitializationCell->True],
Cell[68965, 1651, 140, 3, 27, "Input",
  InitializationCell->True],
Cell[69108, 1656, 146, 3, 27, "Input",
  InitializationCell->True],
Cell[69257, 1661, 134, 3, 27, "Input",
  InitializationCell->True],
Cell[69394, 1666, 134, 3, 27, "Input",
  InitializationCell->True],
Cell[69531, 1671, 144, 3, 27, "Input",
  InitializationCell->True],
Cell[69678, 1676, 151, 3, 27, "Input",
  InitializationCell->True],
Cell[69832, 1681, 150, 3, 27, "Input",
  InitializationCell->True],
Cell[69985, 1686, 146, 3, 27, "Input",
  InitializationCell->True],
Cell[70134, 1691, 140, 3, 27, "Input",
  InitializationCell->True],
Cell[70277, 1696, 116, 2, 27, "Input",
  InitializationCell->True],
Cell[70396, 1700, 94, 2, 27, "Input",
  InitializationCell->True],
Cell[70493, 1704, 57, 0, 31, "Text"],
Cell[70553, 1706, 91, 2, 27, "Input",
  InitializationCell->True],
Cell[70647, 1710, 55, 0, 31, "Text"],
Cell[70705, 1712, 171, 4, 27, "Input",
  InitializationCell->True],
Cell[70879, 1718, 160, 4, 27, "Input",
  InitializationCell->True],
Cell[71042, 1724, 190, 4, 43, "Input",
  InitializationCell->True],
Cell[71235, 1730, 186, 4, 43, "Input",
  InitializationCell->True],
Cell[71424, 1736, 173, 4, 27, "Input",
  InitializationCell->True],
Cell[71600, 1742, 169, 4, 27, "Input",
  InitializationCell->True],
Cell[71772, 1748, 192, 4, 43, "Input",
  InitializationCell->True],
Cell[71967, 1754, 188, 4, 43, "Input",
  InitializationCell->True],
Cell[72158, 1760, 196, 4, 43, "Input",
  InitializationCell->True],
Cell[72357, 1766, 177, 4, 27, "Input",
  InitializationCell->True],
Cell[72537, 1772, 208, 4, 43, "Input",
  InitializationCell->True],
Cell[72748, 1778, 189, 4, 43, "Input",
  InitializationCell->True],
Cell[72940, 1784, 177, 4, 27, "Input",
  InitializationCell->True],
Cell[73120, 1790, 178, 4, 27, "Input",
  InitializationCell->True],
Cell[73301, 1796, 170, 4, 27, "Input",
  InitializationCell->True],
Cell[73474, 1802, 170, 4, 27, "Input",
  InitializationCell->True],
Cell[73647, 1808, 174, 4, 27, "Input",
  InitializationCell->True],
Cell[73824, 1814, 174, 4, 27, "Input",
  InitializationCell->True],
Cell[74001, 1820, 153, 3, 27, "Input",
  InitializationCell->True],
Cell[74157, 1825, 154, 3, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[74348, 1833, 36, 0, 42, "Subsubsection"],
Cell[74387, 1835, 66, 2, 27, "Input",
  InitializationCell->True],
Cell[74456, 1839, 73, 2, 27, "Input",
  InitializationCell->True],
Cell[74532, 1843, 91, 2, 27, "Input",
  InitializationCell->True],
Cell[74626, 1847, 88, 2, 27, "Input",
  InitializationCell->True],
Cell[74717, 1851, 93, 2, 27, "Input",
  InitializationCell->True],
Cell[74813, 1855, 89, 2, 27, "Input",
  InitializationCell->True],
Cell[74905, 1859, 89, 2, 27, "Input",
  InitializationCell->True],
Cell[74997, 1863, 72, 1, 31, "Text",
  InitializationCell->True]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

