(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     20196,        550]*)
(*NotebookOutlinePosition[     20865,        574]*)
(*  CellTagsIndexPosition[     20821,        570]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Begin julian", "Subsection"],

Cell[BoxData[
    \(\(Clear["\<Months`julian`\>"];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(BeginPackage["\<Months`julian`\>", \
"\<Months`Common`MonthsCommon`\>"]\)], "Input",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["data", "Subsubsection"],

Cell["\<\
\tthis set up for using a very popular Julian Day number
\talgorithm, it's not historical accurate\
\>", "Text"],

Cell[BoxData[
    \(\(JDNMonths = 12;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(DayStart[julian] = 12;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(leapammount[julian] = 1;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Epoch[julianDayNumber, julianDayNumber, 
          text] = "\<julianDayNumber julian yr -4712\>";\)\), "\
\[IndentingNewLine]", 
    \(\(Epsolaroch[julianDayNumber, julianDayNumber, type] = 
        solar;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julianDayNumber, julianDayNumber, daystart] = 
        1/2;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julianDayNumber, julianDayNumber, CDN] = 
        0;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julianDayNumber, julianDayNumber, JDN] = 
        0;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julianDayNumber, julianDayNumber, date] = {0, 1, 1, 0, 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julianDayNumber, gregorian, date] = {\(-4713\), 11, 24, 
          DayStart[julian], 0, 0};\)\), "\n", 
    \(\(Epoch[julianDayNumber, julian, date] = {\(-4712\), 1, 1, 
          DayStart[julian], 0, 0};\)\)}], "Input",
  InitializationCell->True],

Cell["see notes nb", "Text"],

Cell[BoxData[{
    \(\(Epoch[julian, julian, 
          text] = "\<julian\>";\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julian, julian, type] = solar;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julian, julian, daystart] = 1/2;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julian, julian, date] = {0, 1, 1, DayStart[julian], 0, 
          0};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julian, julian, CDN] = 0;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julian, julian, JDN] = 1721058;\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julian, julianDayNumber, date] = {4712, 1, 1, DayStart[julian], 
          0, 0};\)\), "\[IndentingNewLine]", 
    \(\(Epoch[julian, gregorian, date] = {\(-1\), 12, 30, DayStart[julian], 
          0, 0};\)\)}], "Input",
  InitializationCell->True],

Cell["note: set this to months JDN thinks there are", "Text"],

Cell[BoxData[
    \(\(MonthsInYear[julian] = JDNMonths;\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(DayNames[julian] = {domenica, lunedi, martedi, mercoledi, giovedi, 
          venerdi, sabato};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(DayNamesStrings[julian] = Map[SymbolName, DayNames[]];\)\)], "Input",
  InitializationCell->True],

Cell["\<\
Mercedonius is Intercalaris (inter), go after feb for old leap \
years, see notes nb\
\>", "Text"],

Cell[BoxData[
    \(\(MonthNames[julian] = {Ianuarius, Februarius, Martius, Aprilis, Maius, 
          Iunius, Quintilis, Sextilis, September, October, November, 
          December};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(MonthNamesStrings[julian] = 
        Map[SymbolName, MonthNames[julian]];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(DayTab[
          julian] = {{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, {31, 
            29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 
            31} (*leap\ year*) };\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthDayTab[julian] = 
      Module[{Private`d, 
          Private`m}, \[IndentingNewLine]{Flatten@
            Table[Table[
                Private`m, {Private`d, \(DayTab[julian]\)[\([1, 
                      Private`m]\)]}], {Private`m, 1, 
                MonthsInYear[julian]}], \[IndentingNewLine]Flatten@
            Table[Table[
                Private`m, {Private`d, \(DayTab[julian]\)[\([2, 
                      Private`m]\)]}], {Private`m, 1, 
                MonthsInYear[julian]}]\[IndentingNewLine]}]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(GreatestDay[julian] = 31; LesserYear[julian] = 365; 
    GreaterYear[julian] = 366;\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["utils", "Subsubsection"],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(fixdate[dtp_, julian] := 
      Module[{y, M, d, h, m, s, x, dt, S, calgreg, 
            caljul}, \[IndentingNewLine]dt = 
            fixdatebegin[dtp]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
            Return[$Failed]]; \[IndentingNewLine]caljul = {0, 1, 1, 
              DayStart[julian], 0, 0}; \[IndentingNewLine]While[
            Length@dt < 6, 
            AppendTo[dt, 
              caljul[\([Length@dt + 1]\)]]]; \[IndentingNewLine]{y, M, d, h, 
              m, s} = dt; \[IndentingNewLine]x = 
            MonthsInYear[julian]; \[IndentingNewLine]If[M < 1 || M > x, 
            Message[Months::"\<unformatted\>", M]; 
            Return[$Failed]]; \[IndentingNewLine]x = 
            DaysInMonth[{y, M}, julian]; \[IndentingNewLine]If[
            d < 1 || d > x, Message[Months::"\<unformatted\>", d]; 
            Return[$Failed]]; \[IndentingNewLine]If[
            Quotient[h, 24] \[NotEqual] 0, 
            Message[Months::"\<unformatted\>", h]; 
            Return[$Failed]]; \[IndentingNewLine]If[
            Quotient[m, 60] \[NotEqual] 0, 
            Message[Months::"\<unformatted\>", m]; 
            Return[$Failed]]; \[IndentingNewLine]If[
            Quotient[s, 60] \[NotEqual] 0, 
            Message[Months::"\<unformatted\>", s]; 
            Return[$Failed]]; \[IndentingNewLine]dt\[IndentingNewLine]] /; 
        ListQ@dtp || dtp \[Element] Integers\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["julian calendar funs", "Subsubsection"],

Cell["\<\
\t(was/is) possible to try old table see notes nb for iterative \
code
\t\tsection: \"what is JDN's calendar layout ?\"\
\>", "Text"],

Cell["\tnote this is how JDN alg. produces it", "Text"],

Cell[BoxData[
    \(MonthsInYear[year_, julian] := JDNMonths\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(leapdays[yp_?NumericQ, julian] := 
      Module[{y}, 
        y = Floor[Abs@yp]; \[IndentingNewLine]If[Sign[yp] === 0, 1, 
            Sign[yp]] \((Quotient[y, 4] + If[yp < 0, 0, 1])\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(leapdays[{y_, ___}, julian] := leapdays[y, julian]\)], "Input",
  InitializationCell->True],

Cell["(*see note 4 in months.notes.nb DayNumberTo2*)", "Text"],

Cell[BoxData[
    \(YearFromDays[days_?NumericQ, julian] := 
      Module[{y, 
          ave}, \[IndentingNewLine]ave = \((LesserYear[julian] + 
              1/4)\); \[IndentingNewLine]If[
          days \[GreaterEqual] 0, \[IndentingNewLine]Quotient[Floor@days - 1, 
            ave], \[IndentingNewLine]y = 
            Quotient[Floor@\(Abs@days\) + LesserYear[julian] - 1, 
              ave]; \[IndentingNewLine]y = \(-y\); 
          If[y === 0, 
            y = \(-1\)]; \[IndentingNewLine]y\[IndentingNewLine]]]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(YearFromDays[{days_, ___}, julian] := 
      YearFromDays[days, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(IsLeapYear[year_?NumericQ, julian] := 
      Module[{y}, y = Floor@year; Mod[y, 4] \[Equal] 0]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(IsLeapYear[{year_, ___}, julian] := IsLeapYear[year, julian]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(DaysInMonth[year_Integer, month_Integer, 
        julian] := \(DayTab[julian]\)[\([If[IsLeapYear[year, julian], 2, 1], 
            Floor@month]\)] /; 
        1 <= month \[LessEqual] MonthsInYear[julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysInMonth[{year_, month_, ___}, julian] := 
      DaysInMonth[year, month, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthTab[year_?NumericQ, 
        julian] := \(DayTab[
          julian]\)[\([If[IsLeapYear[Floor@year, julian], 2, 
            1]]\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthTab[{year_, ___}, julian] := MonthTab[year, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthTab[year_?NumericQ, julian] := 
      Module[{M}, 
        Table[DaysInMonth[Floor@year, M, julian], {M, 1, 
            MonthsInYear[julian]}]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthTab[{year_, ___}, julian] := MonthTab[year, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthDays[year_Integer, month_Integer, julian] := 
      Plus[Sequence @@ Take[MonthTab[Floor@year, julian], Floor@month]] /; 
        0 <= month \[LessEqual] MonthsInYear[julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthDays[{year_, month_, ___}, julian] := 
      MonthDays[year, month, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysInYear[year_?NumericQ, julian] := 
      If[IsLeapYear[year, julian], 366, 365]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DaysInYear[{year_, ___}, julian] := DaysInYear[year, julian]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(YearDay[year_Integer, month_Integer, day_?NumericQ, 
        julian] := \[IndentingNewLine]MonthDays[year, month - 1, julian] + 
          day /; 1 \[LessEqual] month \[LessEqual] MonthsInYear[julian] && 
          1 \[LessEqual] day \[LessEqual] 
            DaysInMonth[year, month, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(YearDay[{year_, month_, day_, ___}, julian] := 
      YearDay[year, month, day, julian]\)], "Input",
  InitializationCell->True],

Cell["\<\
this always works (even with \"real julian\" above see notes nb \
about) and yet isn't too slow.  tentative months must be on low side\
\>", \
"Text"],

Cell[BoxData[
    \(MonthOfYearDay[year_Integer, yearday_?NumericQ, 
        julian] := \[IndentingNewLine]Module[{month, x, 
            i}, \[IndentingNewLine]month = 
            Floor[yearday/GreatestDay[julian]]; \[IndentingNewLine]If[
            month \[Equal] 0, Return[1]]; \[IndentingNewLine]Do[
            If[MonthDays[year, month, julian] \[GreaterEqual] yearday, 
              Break[]]; \(++month\), {i, 1, 
              MonthsInYear[
                julian]}]; \[IndentingNewLine]month\[IndentingNewLine]] /; 
        1 \[LessEqual] Floor@yearday \[LessEqual] 
          DaysInYear[year, julian]\)], "Input"],

Cell[BoxData[
    \(MonthOfYearDay[year_Integer, yearday_?NumericQ, 
        julian] := \(MonthDayTab[
            julian]\)[\([If[IsLeapYear[Floor@year, julian], 2, 1], 
            yearday]\)] /; 
        1 \[LessEqual] Floor@yearday \[LessEqual] 
          DaysInYear[year, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(MonthOfYearDay[{year_, yearday_, ___}, julian] := 
      MonthOfYearDay[year, yearday, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dayofweek[dtp_, julian] := 
      Module[{dt}, \[IndentingNewLine]dt = fixdate[dtp, julian]; 
        If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]Mod[
            Floor@\(julianToJDN@DayNumberFrom2[dt, julian, julian]\), 7] + 
          1]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(dayofweekName[dt_, 
        julian] := \(DayNamesStrings[
          julian]\)[\([dayofweek[dt, julian]]\)]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToSeconds[dtp_, julian] := 
      DayNumberFrom[dtp, julian, julian]\ 24\ 60^2\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(FromSeconds[secs_?NumericQ, julian] := 
      DayNumberTo[secs/\((24\ 60^2)\), julian, julian]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(ToSeconds2[dtp_, julian] := 
      DayNumberFrom2[dtp, julian, julian]\ 24\ 60^2\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(FromSeconds2[secs_?NumericQ, julian] := 
      DayNumberTo2[secs/\((24\ 60^2)\), julian, julian]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["julian day number", "Subsubsection"],

Cell["\<\
see section: lack of tests  julian  , Julian Day Number ,  Julian , \
 and epochs\
\>", "Text"],

Cell[BoxData[
    \(DayNumberFrom[dtp_, gregorian, julian] := 
      Module[{dt, A, B, C, D, E, F, JD, y, M, d, h, m, 
          s}, \[IndentingNewLine]dt = 
          fixdate[dtp, gregorian]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]{y, M, d, h, m, s} = 
          dt; \[IndentingNewLine]If[M \[LessEqual] 2, \(--y\); 
          M += JDNMonths]; \[IndentingNewLine]Y = y; 
        D = d; \[IndentingNewLine]A = Y/100 // Floor; \[IndentingNewLine]B = 
          A/4 // Floor; \[IndentingNewLine]C = 
          2 - A + B; \[IndentingNewLine]E = \((365 + 1/4)\) \((Y + 4716)\) // 
            Floor; \[IndentingNewLine]F = \((306001/10000)\) \((M + 1)\) // 
            Floor; \[IndentingNewLine]JD = 
          C + D + E + 
            F - \((1524 + 
                1/2)\); \[IndentingNewLine]JD + \((h + \((m + s/60)\)/60)\)/
            24\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberFrom[dtp_, julian, julian] := 
      Module[{dt, A, B, C, D, E, F, JD, y, M, d, h, m, 
          s}, \[IndentingNewLine]dt = 
          fixdate[dtp, julian]; \[IndentingNewLine]If[dt \[Equal] $Failed, 
          Return[$Failed]]; \[IndentingNewLine]{y, M, d, h, m, s} = 
          dt; \[IndentingNewLine]If[M \[LessEqual] 2, \(--y\); 
          M += JDNMonths];  (*\ 
          why\ greg\ \(\(months\)\(?\)\)\ *) \[IndentingNewLine]Y = y; 
        D = d; \[IndentingNewLine] (*\[IndentingNewLine]A = 
            Y/100 // Floor; \[IndentingNewLine]B = 
            A/4 // Floor; \[IndentingNewLine]C = 
            2 - A + B;\[IndentingNewLine]*) \[IndentingNewLine]A = 0; B = 0; 
        C = 0; \[IndentingNewLine]E = \((365 + 1/4)\) \((Y + 4716)\) // 
            Floor; \[IndentingNewLine]F = \((306001/10000)\) \((M + 1)\) // 
            Floor; \[IndentingNewLine]JD = 
          C + D + E + 
            F - \((1524 + 
                1/2)\); \[IndentingNewLine]JD + \((h + \((m + s/60)\)/60)\)/
            24\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(DayNumberTo[jdc_?NumericQ, gregorian, julian] := 
      Module[{dt, Q, Z, W, X, A, B, C, D, E, F, JD, y, M, d, h, m, 
          s}, \[IndentingNewLine]Q = jdc + 1/2; \[IndentingNewLine]Z = 
          Q // Floor; \[IndentingNewLine]W = \((Z - \((1867216 + 
                      1/4)\))\)/\((36524 + 1/4)\) // 
            Floor; \[IndentingNewLine]X = 
          W/4 // Floor; \[IndentingNewLine]A = 
          Z + 1 + W - X; \[IndentingNewLine]B = 
          A + 1524; \[IndentingNewLine]C = \((B - \((1221/10)\))\)/\((365 + 
                  1/4)\) // Floor; \[IndentingNewLine]D = \((365 + 1/4)\)\ C // 
            Floor; \[IndentingNewLine]E = \((B - D)\)/\((306001/10000)\) // 
            Floor; \[IndentingNewLine]F = \((306001/10000)\)\ E // 
            Floor; \[IndentingNewLine]d = 
          B - D - F + \((Q - Z)\); \[IndentingNewLine]M = 
          If[0 < E - 1 \[LessEqual] JDNMonths, E - 1, 
            E - \((JDNMonths + 1)\)]; \[IndentingNewLine]y = 
          If[M <= 2, C - 4715, C - 4716]; \[IndentingNewLine]h = 
          FractionalPart[d]\ 24; \[IndentingNewLine]d = 
          d // Floor; \[IndentingNewLine]m = 
          FractionalPart[h]\ 60; \[IndentingNewLine]h = 
          h // Floor; \[IndentingNewLine]s = 
          FractionalPart[m]\ 60; \[IndentingNewLine]m = 
          m // Floor; \[IndentingNewLine]{y, M, d, h, m, 
          s}\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
\t.5  is because the julian Day begins at noon, Greenwich \
time\
\>", "Text"],

Cell[BoxData[
    \(DayNumberTo[jdc_?NumericQ, julian, julian] := 
      Module[{dt, Q, Z, W, X, A, B, C, D, E, F, JD, y, M, d, h, m, 
          s}, \[IndentingNewLine]Q = jdc + 1/2; \[IndentingNewLine]Z = 
          Q // Floor; \[IndentingNewLine] (*\[IndentingNewLine]W = \((Z - \
\((1867216 + 1/4)\))\)/\((36524 + 1/4)\) // Floor; \[IndentingNewLine]X = 
            W/4 // Floor; \[IndentingNewLine]A = 
            Z + 1 + W - X;\[IndentingNewLine]*) \[IndentingNewLine]W = 0; 
        X = 0; \[IndentingNewLine]A = Z; \[IndentingNewLine]B = 
          A + 1524; \[IndentingNewLine]C = \((B - 1221/10)\)/\((365 + 1/4)\) // 
            Floor; \[IndentingNewLine]D = \((365 + 1/4)\)\ C // 
            Floor; \[IndentingNewLine]E = \((B - D)\)/\((306001/10000)\) // 
            Floor; \[IndentingNewLine]F = \((306001/10000)\) E // 
            Floor; \[IndentingNewLine]d = 
          B - D - F + \((Q - Z)\); \[IndentingNewLine]M = 
          If[0 < E - 1 \[LessEqual] JDNMonths, E - 1, 
            E - \((JDNMonths + 1)\)]; \[IndentingNewLine]y = 
          If[M \[LessEqual] 2, C - 4715, C - 4716]; \[IndentingNewLine]h = 
          FractionalPart[d]\ 24; \[IndentingNewLine]d = 
          d // Floor; \[IndentingNewLine]m = 
          FractionalPart[h]\ 60; \[IndentingNewLine]h = 
          h // Floor; \[IndentingNewLine]s = 
          FractionalPart[m]\ 60; \[IndentingNewLine]m = 
          m // Floor; \[IndentingNewLine]{y, M, d, h, m, 
          s}\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["julian  J  MeanTropicalYear  T", "Subsubsection"],

Cell["see notes for  julian  J  MeanTropicalYear  T", "Text"],

Cell[BoxData[
    \(MeanTropicalYearTTfunc[T_?NumericQ, julian, 
        epoch_] := \[IndentingNewLine]If[epoch =!= Epoch[TT], 
        Return[$Failed], \[IndentingNewLine]365.2421896698 - 
          6.15359\ 10^\(-6\)\ T - 7.29\ 10^\(-10\)\ T^2 + 
          2.64\ 10^\(-10\)\ T^3\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["End julian", "Subsubsection"],

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowSize->{600, 569},
WindowMargins->{{Automatic, 148}, {Automatic, 73}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 34, 0, 45, "Subsection"],
Cell[1776, 53, 93, 2, 27, "Input",
  InitializationCell->True],
Cell[1872, 57, 132, 3, 27, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[2029, 64, 29, 0, 42, "Subsubsection"],
Cell[2061, 66, 122, 3, 48, "Text"],
Cell[2186, 71, 80, 2, 27, "Input",
  InitializationCell->True],
Cell[2269, 75, 87, 2, 27, "Input",
  InitializationCell->True],
Cell[2359, 79, 89, 2, 27, "Input",
  InitializationCell->True],
Cell[2451, 83, 931, 18, 171, "Input",
  InitializationCell->True],
Cell[3385, 103, 28, 0, 31, "Text"],
Cell[3416, 105, 776, 13, 155, "Input",
  InitializationCell->True],
Cell[4195, 120, 61, 0, 31, "Text"],
Cell[4259, 122, 98, 2, 27, "Input",
  InitializationCell->True],
Cell[4360, 126, 160, 3, 43, "Input",
  InitializationCell->True],
Cell[4523, 131, 119, 2, 27, "Input",
  InitializationCell->True],
Cell[4645, 135, 108, 3, 48, "Text"],
Cell[4756, 140, 226, 4, 59, "Input",
  InitializationCell->True],
Cell[4985, 146, 138, 3, 27, "Input",
  InitializationCell->True],
Cell[5126, 151, 236, 5, 43, "Input",
  InitializationCell->True],
Cell[5365, 158, 578, 12, 171, "Input",
  InitializationCell->True],
Cell[5946, 172, 144, 3, 43, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[6127, 180, 30, 0, 42, "Subsubsection"],
Cell[6160, 182, 88, 2, 27, "Input",
  InitializationCell->True],
Cell[6251, 186, 1454, 27, 315, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[7742, 218, 45, 0, 42, "Subsubsection"],
Cell[7790, 220, 143, 4, 48, "Text"],
Cell[7936, 226, 55, 0, 31, "Text"],
Cell[7994, 228, 101, 2, 27, "Input",
  InitializationCell->True],
Cell[8098, 232, 246, 5, 59, "Input",
  InitializationCell->True],
Cell[8347, 239, 111, 2, 27, "Input",
  InitializationCell->True],
Cell[8461, 243, 62, 0, 31, "Text"],
Cell[8526, 245, 562, 12, 139, "Input",
  InitializationCell->True],
Cell[9091, 259, 132, 3, 27, "Input",
  InitializationCell->True],
Cell[9226, 264, 155, 3, 43, "Input",
  InitializationCell->True],
Cell[9384, 269, 123, 3, 27, "Input",
  InitializationCell->True],
Cell[9510, 274, 265, 5, 59, "Input",
  InitializationCell->True],
Cell[9778, 281, 145, 3, 43, "Input",
  InitializationCell->True],
Cell[9926, 286, 196, 5, 43, "Input",
  InitializationCell->True],
Cell[10125, 293, 117, 2, 27, "Input",
  InitializationCell->True],
Cell[10245, 297, 210, 5, 59, "Input",
  InitializationCell->True],
Cell[10458, 304, 117, 2, 27, "Input",
  InitializationCell->True],
Cell[10578, 308, 240, 4, 59, "Input",
  InitializationCell->True],
Cell[10821, 314, 141, 3, 27, "Input",
  InitializationCell->True],
Cell[10965, 319, 144, 3, 43, "Input",
  InitializationCell->True],
Cell[11112, 324, 123, 3, 27, "Input",
  InitializationCell->True],
Cell[11238, 329, 353, 6, 75, "Input",
  InitializationCell->True],
Cell[11594, 337, 148, 3, 43, "Input",
  InitializationCell->True],
Cell[11745, 342, 160, 4, 48, "Text"],
Cell[11908, 348, 625, 11, 139, "Input"],
Cell[12536, 361, 325, 7, 59, "Input",
  InitializationCell->True],
Cell[12864, 370, 155, 3, 43, "Input",
  InitializationCell->True],
Cell[13022, 375, 328, 7, 59, "Input",
  InitializationCell->True],
Cell[13353, 384, 167, 4, 43, "Input",
  InitializationCell->True],
Cell[13523, 390, 139, 3, 27, "Input",
  InitializationCell->True],
Cell[13665, 395, 155, 3, 43, "Input",
  InitializationCell->True],
Cell[13823, 400, 141, 3, 27, "Input",
  InitializationCell->True],
Cell[13967, 405, 157, 3, 43, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[14161, 413, 42, 0, 42, "Subsubsection"],
Cell[14206, 415, 105, 3, 48, "Text"],
Cell[14314, 420, 952, 17, 251, "Input",
  InitializationCell->True],
Cell[15269, 439, 1106, 20, 299, "Input",
  InitializationCell->True],
Cell[16378, 461, 1452, 25, 379, "Input",
  InitializationCell->True],
Cell[17833, 488, 88, 3, 31, "Text"],
Cell[17924, 493, 1527, 25, 443, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[19488, 523, 55, 0, 42, "Subsubsection"],
Cell[19546, 525, 61, 0, 31, "Text"],
Cell[19610, 527, 341, 6, 75, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[19988, 538, 35, 0, 42, "Subsubsection"],
Cell[20026, 540, 66, 2, 27, "Input",
  InitializationCell->True],
Cell[20095, 544, 73, 2, 27, "Input",
  InitializationCell->True]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

