(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     28569,        647]*)
(*NotebookOutlinePosition[     29415,        678]*)
(*  CellTagsIndexPosition[     29333,        672]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[":Mathematica Version: 4.0", "Text"],

Cell[":Package Version: 1.0", "Text"],

Cell[":Copyright: public domain", "Text"],

Cell[":Name: timeDeduce`", "Text"],

Cell[":Title: timeDeduce", "Text"],

Cell[":Author: John Hendrickson (debguy@sourceforge.net)", "Text"],

Cell[":History: None", "Text"],

Cell[":Keywords: timeDeduce time Date Calendar", "Text"],

Cell[":Requirements: None", "Text"],

Cell[":Sources: all code is origional by author", "Text"],

Cell["\<\
:Summary: see also timeDeduce.notes.nb

\tquick ex:
\t\t input: GMT,lon,tz,arieslon
\t\toutput: input + SHA,Ra,LHA ; in dn,tm or ang

\tdeduces what can be known without simulation
\t(uses equality based equations of time)

\tNOTE: theoretically done but hardly tested

\tby: John Hendrickson (debguy@sourceforge.net), 2013, public domain
\t\tcredit to many authors of math in general\
\>", "Text"],

Cell[CellGroupData[{

Cell["setup", "Subsubsection"],

Cell[BoxData[
    \(\(\( (*\ suggested\ , \ 
      see\ the\ mm\ book\ *) \)\(\[IndentingNewLine]\)\(If[
      False, \[IndentingNewLine] (*\ On[MemoryConserve]\ *)  (*\ 
        may\ be\ causing\ crashes\ *) \[IndentingNewLine] (*\ 
        turn\ off\ has\ similar\ spelling\ messages\ \
*) \[IndentingNewLine]Off[General::"\<spell\>"]; \[IndentingNewLine]Unprotect[
        In, Out]; \[IndentingNewLine]Clear[In, 
        Out]; \[IndentingNewLine]Protect[In, 
        Out]; \[IndentingNewLine]$HistoryLength = 
        5; \[IndentingNewLine]MemoryInUse[];\[IndentingNewLine]]\)\)\)], \
"Input"],

Cell["Needs[\"Miscellaneous`Geodesy`\"]", "Text"],

Cell[BoxData[
    \(Needs["\<Months`Months`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(BeginPackage["\<timeDeduce`\>", {"\<Months`Months`\>", \
"\<Months`Common`MonthsCommon`\>"}];\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["timeDeduce", "Subsection"],

Cell[CellGroupData[{

Cell["setup", "Subsubsection"],

Cell[BoxData[
    \(\(Clear[timeDeduce, timeConvert, timeTimeRules, timeAngleRules, 
        timeShow, timeTest4, timeTest2, timeTestEq, timeTest3, timeTest3b, 
        vars, varsT, varsHA, varsRA, tm, dn, ang];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeDeduce::usage := "\<timeDeduce[cal:gregorian,pass:2]  shows time \
in formats known given data (ex. GMT, Ra).  Directions: use timeReset[cal] \
then set which of longitudes, hour angles, times you know.  Then use \
timeDeduce to fill in deduced. both day number and time or longitude are \
output.  show outputs a Input cell.  see also: DayNumberChange2 ToDegrees \
Convert.\n\nnotes: T can be set tm or dn, L and HA can be set ang or dn.  For \
beginners: all should  refer to same day so output is not confusing. tm are \
usual date format for cal, dn are day numbers from day 0 of cal not JDN \
longitude or hour ang [-180,180] degrees. sign of tm dn is significant.  ex \
of setting times (only need be set): GMT[tm]={x,...}; GHAstar[dn]=.; \
lon[ang]=x;  star can be sun.  see also: timeDeduce.notes.nb\>";\)\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(\(\(\ \ \)\(timeShow::usage := "\<Use at any time to see current \
values.\>";\)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(\ \ \)\(timeReset::usage := "\<Start over, reset all time,ang to \
Null.\>";\)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(\ \ \)\(timeShow::usage := "\<Use timeShow at any time to see what \
is set\>";\)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(\ \ \)\(timeConvert::usage := "\<timeDeduce uses, but can be used \
at any time\>";\)\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeConvert::"\<unformatted\>" = "\<`1`\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeDeduce::"\<errata\>" = "\<RA[ang]<->SHA[dn], only HA used for \
dn\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeAngleRules::"\<dayless\>" = "\<`1` is from two \
longitudes\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeTest4::"\<extrapolated\>" = "\<`1` taken from diff. \
days\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeTest3::"\<extrapolated\>" = "\<`1` taken from diff. \
days\>";\)\)], "Input",
  InitializationCell->True],

Cell["Needs[\"Miscellaneous`Units`\"]", "Text"],

Cell[BoxData[
    \(\(Options[timeDeduce]\  = \ {daylessOk \[Rule] True, 
          timePasses \[Rule] 2, ShowCell \[Rule] False};\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(Clear[\[IndentingNewLine]vars, varsT, varsST, varsHA, varsRA, tm, dn, 
        ang, \[IndentingNewLine]midday, 
        dayless, \[IndentingNewLine]localtime, timezone, lon, 
        arieslon, \[IndentingNewLine]UTC, GMT, LMT, LAT, 
        EOT, \[IndentingNewLine]LST, LMST, AST, GST, AMST, 
        GMST, \[IndentingNewLine]GHAaries, 
        GHAstar, \[IndentingNewLine]LHAaries, LHAstar, LHAsun, SHAstar, 
        SHAlm, SHAgm, \[IndentingNewLine]RAstar, RAaries, RAlm, 
        RAgm\[IndentingNewLine]];\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["functions", "Subsubsection"],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeReset[cal_:  gregorian] := 
        Module[{}, \[IndentingNewLine] (*\(Clear[lat, gdate, jdn, gdn, star, 
                height];\)*) \[IndentingNewLine]Clear[\[IndentingNewLine]vars,\
 varsT, varsST, varsHA, varsRA, tm, dn, ang, \[IndentingNewLine]midday, 
            dayless, \[IndentingNewLine]localtime, timezone, lon, 
            arieslon, \[IndentingNewLine]UTC, GMT, LMT, LAT, 
            EOT, \[IndentingNewLine]LST, LMST, AST, GST, AMST, 
            GMST, \[IndentingNewLine]GHAaries, 
            GHAstar, \[IndentingNewLine]LHAaries, LHAstar, LHAsun, SHAstar, 
            SHAlm, SHAgm, \[IndentingNewLine]RAstar, RAaries, RAlm, 
            RAgm\[IndentingNewLine]]; \[IndentingNewLine] (*\(vars = 
                Sort@{lat, gdate, jdn, gdn, star, 
                    height};\)*) \[IndentingNewLine]varsT = 
            Sort@{GMT, LAT, LMT, localtime, timezone, UTC, 
                EOT}; \[IndentingNewLine]varsST = 
            Sort@{LST, LMST, AST, AMST, GMST, 
                GST}; \[IndentingNewLine]varsHA = 
            Sort@{GHAaries, GHAstar, LHAaries, LHAstar, LHAsun, SHAstar, 
                SHAlm, SHAgm}; \[IndentingNewLine]varsRA = 
            Sort@{RAstar, RAaries, RAlm, RAgm}; \[IndentingNewLine]varsL = 
            Sort@{arieslon, lon}; \[IndentingNewLine]varsStr = 
            Map[ToString, vars]; \[IndentingNewLine]varsTStr = 
            Map[ToString, varsT]; \[IndentingNewLine]varsSTStr = 
            Map[ToString, varsST]; \[IndentingNewLine]varsHAStr = 
            Map[ToString, varsHA]; \[IndentingNewLine]varsRAStr = 
            Map[ToString, varsRA]; \[IndentingNewLine]varsLStr = 
            Map[ToString, 
              varsL]; \[IndentingNewLine]Scan[\((#[tm] = Null; #[dn] = 
                  Null)\) &, 
            varsT]; \[IndentingNewLine]Scan[\((#[tm] = Null; #[dn] = 
                  Null)\) &, 
            varsST]; \[IndentingNewLine]Scan[\((#[ang] = Null; #[dn] = 
                  Null)\) &, 
            varsHA]; \[IndentingNewLine]Scan[\((#[ang] = Null; #[dn] = 
                  Null)\) &, 
            varsRA]; \[IndentingNewLine]Scan[\((#[ang] = Null; #[dn] = 
                  Null)\) &, varsL]; \[IndentingNewLine]If[
            Head@Epoch[cal, cal, daystart] === Epoch, 
            midday[dn] = 1/2, \[IndentingNewLine]midday[dn] = 
              1/2 - Epoch[cal, cal, 
                  daystart]]; \[IndentingNewLine] (*midday\ is\ 0\ for\ \
julian\ if\ attempted*) \[IndentingNewLine]dayless = 
            daylessOk /. 
              Options[timeDeduce]; \
\[IndentingNewLine]Null\[IndentingNewLine]];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(timeConvert[cal_:  gregorian] := 
        Module[{x}, \[IndentingNewLine]Scan[\((\[IndentingNewLine]If[#[dn] === 
                      Null && #[ang] =!= Null, \[IndentingNewLine]#[
                      dn] = \((\(-\((#[ang] - 180)\)\)/15)\)/
                      24]; \[IndentingNewLine]If[#[ang] === Null && #[dn] =!= 
                      Null, \[IndentingNewLine]#[
                      ang] = \((1 - #[dn])\)\ 15\ 24 - 
                      180]; \[IndentingNewLine]If[
                  Abs@#[dn] \[GreaterEqual] 1 || 
                    Abs@#[ang] \[GreaterEqual] 
                      180, \[IndentingNewLine]Message[
                    timeConvert::"\<unformatted\>", {#[dn], #[ang]}]; \ 
                  Return[$Failed]];\[IndentingNewLine])\) &, 
            varsL]; \[IndentingNewLine]Scan[\((\[IndentingNewLine]If[#[dn] === 
                      Null && #[ang] =!= Null, \[IndentingNewLine]#[
                      dn] = #[ang]/360; \[IndentingNewLine]#[
                      dn] = \((\((IntegerPart@#[dn] + 1 - 
                              FractionalPart@#[dn])\) 360/15)\)/
                      24]; \[IndentingNewLine]If[#[ang] === Null && #[dn] =!= 
                      Null, \[IndentingNewLine]#[ang] = 
                    FractionalPart@#[dn]\ 24\ 15; \[IndentingNewLine]#[ang] = 
                    IntegerPart@#[dn] + 
                      360 - #[
                        ang];\[IndentingNewLine]];\[IndentingNewLine])\) &, 
            varsRA]; \[IndentingNewLine]Scan[\((\[IndentingNewLine]If[#[dn] === 
                      Null && #[ang] =!= Null, \[IndentingNewLine]#[
                      dn] = \((#[ang]/15)\)/
                      24]; \[IndentingNewLine]If[#[ang] === Null && #[dn] =!= 
                        Null, \[IndentingNewLine]#[ang] = #[
                        dn]]\ 15\ 24;\[IndentingNewLine])\) &, 
            varsHA]; \[IndentingNewLine]Scan[\((\[IndentingNewLine]If[#[dn] === 
                      Null && #[tm] =!= Null, \[IndentingNewLine]#[dn] = 
                    DayNumberFrom2[#[tm], cal, 
                      cal]]; \[IndentingNewLine]If[#[tm] === Null && #[dn] =!= 
                      Null, \[IndentingNewLine]#[tm] = 
                    DayNumberTo2[#[dn], cal, cal]];\[IndentingNewLine])\) &, 
            varsT]; \[IndentingNewLine]Scan[\((\[IndentingNewLine]If[#[dn] === 
                      Null && #[tm] =!= Null, \[IndentingNewLine]x = 
                    DayNumberFrom2[#[tm], cal, cal]; \[IndentingNewLine]#[
                      dn] = IntegerPart@x + 1 - 
                      FractionalPart@x]; \[IndentingNewLine]If[#[tm] === 
                      Null && #[dn] =!= Null, \[IndentingNewLine]x = #[dn]; 
                  x = IntegerPart@x + 1 - 
                      FractionalPart@x; \[IndentingNewLine]#[tm] = 
                    DayNumberTo2[x, cal, cal]];\[IndentingNewLine])\) &, 
            varsST]; \[IndentingNewLine]Null\[IndentingNewLine]];\)\)], \
"Input",
  InitializationCell->True],

Cell["\<\
note /; would cause mm to inf recurse and i doubt Solve could \
complete all tasks.

If[LMT[dn]===Null&&GMT[dn]=!=Null&&lon[dn]=!=Null,
    LMT[dn]:=GMT[dn]-lon[dn]+midday;once=True];
If[GMT[dn]===Null&&LMT[dn]=!=Null&&lon[dn]=!=Null,
    GMT[dn]:=LMT[dn]+lon[dn]-midday;once=True];
If[lon[dn]===Null&&LMT[dn]=!=Null&&GMT[dn]=!=Null,
    lon[dn]:=GMT[dn]-LMT[dn]+midday;once=True];\
\>", "Text"],

Cell["\<\
there is a dependancy tree of prefferred order (tm then ang) and \
also of not extrapolating lon.  but not enough to bother using sorting.  pass \
2 \"allows\" lon equated from diff days (usu. ok)\
\>", "Text"],

Cell["\<\
using dn many share same formula though dn takes \"getting used \
to\".  i don't think anything is \"easier\" all said and done (ie, if days \
are separated and added back later: why?), also dn works +- days\
\>", "Text"],

Cell["\<\
lon/midday or lon/lon (t3 t4) have no sign in arithmetic and do not \
belong to either t1,t2 so: un-generically adjust sign t3 t4
\tex: if GMT is neg, then lon, midday are when added\
\>", "Text"],

Cell[BoxData[
    \(timeTest4[{t1_, t2_, t3_, t4_}, pass_] := 
      Module[{once, sign}, 
        once = False; \[IndentingNewLine]If[t4[dn] == Null, 
          Return[once]]; \[IndentingNewLine]If[
          t1[dn] === Null && t2[dn] =!= Null && 
            t3[dn] =!= Null, \[IndentingNewLine]sign = 
            If[t2[dn] < 0, \(-1\), 1]; \[IndentingNewLine]t1[dn] = 
            t2[dn] - sign\ t3[dn] + sign\ t4[dn]; 
          once = True]; \[IndentingNewLine]If[
          t2[dn] === Null && t1[dn] =!= Null && 
            t3[dn] =!= Null, \[IndentingNewLine]sign = 
            If[t1[dn] < 0, \(-1\), 1]; \[IndentingNewLine]t2[dn] = 
            t1[dn] + sign\ t3[dn] - sign\ t4[dn]; 
          once = True]; \[IndentingNewLine]If[
          Floor@t2[dn] === 
              Floor@t1[dn] || \((pass === 2 && 
                dayless)\), \[IndentingNewLine]If[
            t3[dn] === Null && t1[dn] =!= Null && 
              t2[dn] =!= Null, \[IndentingNewLine]sign = 
              If[FractionalPart@t2[dn] - FractionalPart@t1[dn] < 0, \(-1\), 
                1]; \[IndentingNewLine]t3[dn] = 
              FractionalPart@t2[dn] - FractionalPart@t1[dn] + 
                sign\ t4[dn]; \[IndentingNewLine]If[
              Floor@t2[dn] =!= Floor@t1[dn], 
              Message[timeTest4::"\<extrapolated\>", 
                t4]]; \[IndentingNewLine]once = True]; \[IndentingNewLine]If[
            t4[dn] === Null && t2[dn] =!= Null && 
              t3[dn] =!= Null, \[IndentingNewLine]sign = 
              If[FractionalPart@t1[dn] - FractionalPart@t2[dn] < 0, \(-1\), 
                1]; \[IndentingNewLine]t4[dn] = 
              sign\ t3[dn] - FractionalPart@t2[dn] + 
                FractionalPart@t1[dn]; \[IndentingNewLine]If[
              Floor@t2[dn] =!= Floor@t1[dn], 
              Message[timeTest4::"\<extrapolated\>", 
                t4]]; \[IndentingNewLine]once = 
              True];\[IndentingNewLine]]; \[IndentingNewLine]once]\)], "Input",\

  InitializationCell->True],

Cell[BoxData[
    \(timeTest2[{t1_, t2_}] := 
      Module[{once}, 
        once = False; \[IndentingNewLine]If[
          t1[dn] === Null && t2[dn] =!= Null, \[IndentingNewLine]t1[dn] = 
            t2[dn]; once = True]; \[IndentingNewLine]If[
          t2[dn] === Null && t1[dn] =!= Null, \[IndentingNewLine]t2[dn] = 
            t1[dn]; once = True]; \[IndentingNewLine]once]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(timeTest3[{t1_, t2_, t3_}, pass_] := 
      Module[{once, sign}, 
        once = False; \[IndentingNewLine]If[
          t1[dn] === Null && t2[dn] =!= Null && 
            t3[dn] =!= Null, \[IndentingNewLine]sign = 
            If[t2[dn] < 0, \(-1\), 1]; \[IndentingNewLine]t1[dn] = 
            t2[dn] + sign\ t3[dn]; once = True]; \[IndentingNewLine]If[
          t2[dn] === Null && t1[dn] =!= Null && 
            t3[dn] =!= Null, \[IndentingNewLine]sign = 
            If[t1[dn] < 0, \(-1\), 1]; \[IndentingNewLine]t2[dn] = 
            t1[dn] - sign\ t3[dn]; once = True]; \[IndentingNewLine]If[
          t3[dn] === Null && t1[dn] =!= Null && 
            t2[dn] =!= Null, \[IndentingNewLine]If[
            Floor@t2[dn] === 
                Floor@t1[dn] || \((pass === 2 && 
                  dayless)\), \[IndentingNewLine]t3[dn] = 
              FractionalPart@t1[dn] - 
                FractionalPart@t2[dn]; \[IndentingNewLine]If[
              Floor@t2[dn] =!= Floor@t1[dn], 
              Message[timeTest3::"\<extrapolated\>", 
                t3]];\[IndentingNewLine]]; \[IndentingNewLine]once = 
            True]; \[IndentingNewLine]once]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(timeTest3b[{t1_, t2_, t3_}] := 
      Module[{once}, 
        once = False; \[IndentingNewLine]If[
          t1[dn] === Null && t2[dn] =!= Null && 
            t3[dn] =!= Null, \[IndentingNewLine]t1[dn] = t2[dn] + t3[dn]; 
          once = True]; \[IndentingNewLine]If[
          t2[dn] === Null && t1[dn] =!= Null && 
            t3[dn] =!= Null, \[IndentingNewLine]t2[dn] = t1[dn] - t3[dn]; 
          once = True]; \[IndentingNewLine]If[
          t3[dn] === Null && t1[dn] =!= Null && 
            t2[dn] =!= Null, \[IndentingNewLine]t3[dn] = 
            t1[dn] - t2[dn]; \[IndentingNewLine]once = 
            True]; \[IndentingNewLine]once]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(timeTestEq[{t1_, t2_}] := 
      Module[{once}, 
        once = False; \[IndentingNewLine]If[
          t1[dn] === Null && t2[dn] =!= Null, \[IndentingNewLine]t1[dn] = 
            t2[dn]; once = True]; \[IndentingNewLine]If[
          t2[dn] === Null && t1[dn] =!= Null, \[IndentingNewLine]t2[dn] = 
            t1[dn]; once = True]; \[IndentingNewLine]once]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(timeTimeRules[pass_, 
        cal_:  gregorian] := \[IndentingNewLine]Module[{once}, 
        once = False; \[IndentingNewLine]\[IndentingNewLine]If[
          timezone[dn] \[Equal] Null && 
            lon[dn] \[NotEqual] Null, \[IndentingNewLine]timezone[dn] = 
            IntegerPart@\((\(-\((\((Abs@lon[dn] - Abs@midday[dn])\)\ 24 - 
                      1/2)\)\))\)]; \[IndentingNewLine]If[
          localtime[dn] \[Equal] Null && \ timezone[dn] \[NotEqual] Null && 
            GMT[dn] \[NotEqual] Null, 
          localtime[dn] = GMT[dn] + timezone[dn]\ 24]; \[IndentingNewLine]If[
          GMT[dn] \[Equal] Null && \ timezone[dn] \[NotEqual] Null && 
            localtime[dn] \[NotEqual] Null, 
          GMT[dn] = 
            localtime[dn] - 
              timezone[
                  dn]\ 24]; \
\[IndentingNewLine]\[IndentingNewLine]Scan[\((once = 
                Or[once, timeTestEq[#]])\) &, {{UTC, 
              GMT}}]; \[IndentingNewLine]\[IndentingNewLine]Scan[\((once = 
                Or[once, timeTest4[#, pass]])\) &, {{LMT, GMT, lon, 
              midday}, {AST, GST, arieslon, midday}, {AMST, GMST, arieslon, 
              midday}, {LST, GST, lon, midday}, {LMST, GMST, lon, 
              midday}, {LST, AST, lon, 
              arieslon}}]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
          remember\ ST[dn] \[Rule] 
            T[dn]\ on\ input\ *) \[IndentingNewLine]Scan[\((once = 
                Or[once, timeTest3[#, pass]])\) &, {{GMST, GMT, 
              midday}}]; \[IndentingNewLine]Scan[\((once = 
                Or[once, timeTest3b[#]])\) &, {{GST, EOT, 
              GMST}}]; \[IndentingNewLine]\[IndentingNewLine]once\
\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell["\<\
notes:
\tEOT == GHAsun - GMHAsun
\tST(0) preceeds T(0), which is earlier arbitrary\
\>", "Text"],

Cell["\<\
\tis there a way to get ST to T or HA ?  eot?
\t? was sin cos better idea (too late, no days on either)\
\>", "Text"],

Cell[BoxData[
    \(timeAngleRules[ponce_, pass_, cal_:  gregorian] := 
      Module[{once, sign, test}, 
        once = False; \[IndentingNewLine]\[IndentingNewLine] (*remember\ RA[
                ang] \[Rule] 
            SHA[dn]\ on\ input . \ 
                RA\ does\ not\ always\ mean\ to\ \
aries*) \[IndentingNewLine]Scan[\((once = 
                Or[once, timeTestEq[#]])\) &, {{RAstar, SHAstar}, {RAlm, 
              SHAlm}, {RAgm, SHAgm}, {RAaries, 
              LHAaries}}]; \[IndentingNewLine]\[IndentingNewLine]If[
          LHAsun[dn] =!= Null && LHAstar[dn] === Null, 
          LHAstar[dn] = LHAsun[dn]; once = True]; \[IndentingNewLine]If[
          LHAsun === Null, \[IndentingNewLine]sign = 
            If[GST[dn] < 0, \(-1\), 1]; \[IndentingNewLine]If[
            GHAstar[dn] === GST[dn] - sign\ midday[dn], LHAsun = LHAstar; 
            once = True]; \[IndentingNewLine]sign = 
            If[LHAstar[dn] < 0, \(-1\), 1]; \[IndentingNewLine]If[
            LHAstar[dn] === LST[dn] - sign\ midday[dn], LHAsun = LHAstar; 
            once = True]; \[IndentingNewLine]If[SHAlm[dn] === SHAstar[dn], 
            LHAsun = LHAstar; 
            once = True];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ LAT\ preceeds\ LHAsun\ , \ 
          choice\ is\ arbitrary\ *) \[IndentingNewLine]Scan[\((once = 
                Or[once, timeTest3[#, pass]])\) &, {{LAT, LHAsun, 
              midday}}]; \[IndentingNewLine]Scan[\((once = 
                Or[once, timeTest3b[#]])\) &, {{GHAstar, GHAaries, 
              SHAstar}}]; \[IndentingNewLine]Scan[\((once = 
                Or[once, timeTest4[#, pass]])\) &, {{LHAstar, GHAstar, lon, 
              midday}}]; \[IndentingNewLine]\[IndentingNewLine]test[{x1_, 
              x2_}] := If[
            x2[dn] === Null && x1[dn] =!= Null, \[IndentingNewLine]x2[dn] = 
              IntegerPart@x1[dn] + 1 - 
                FractionalPart@x1[dn]; \[IndentingNewLine]once = 
              True]; \[IndentingNewLine]Scan[\((\[IndentingNewLine] (*\ 
                day\ should\ not\ change\ *) \[IndentingNewLine]test[{#[\([1]\
\)], #[\([2]\)]}]; \[IndentingNewLine]test[{#[\([2]\)], #[\([1]\)]}]; \
\[IndentingNewLine]sign = 
                If[\(#[\([1]\)]\)[dn] < 0, \(-1\), 
                  1]; \[IndentingNewLine]If[\(#[\([3]\)]\)[dn] === 
                    Null && \(#[\([1]\)]\)[dn] =!= 
                    Null && \(#[\([4]\)]\)[dn] =!= 
                    Null, \[IndentingNewLine]\(#[\([3]\)]\)[dn] = 
                  sign\ \(#[\([4]\)]\)[dn] + 
                    FractionalPart@\(#[\([1]\)]\)[
                        dn]; \[IndentingNewLine]If[\(#[\([3]\)]\)[dn] > 
                    1, \(#[\([3]\)]\)[dn] = \(#[\([3]\)]\)[dn] - 
                      1]; \[IndentingNewLine]once = 
                  True]; \[IndentingNewLine]If[\(#[\([4]\)]\)[dn] === 
                    Null && \(#[\([1]\)]\)[dn] =!= 
                    Null && \(#[\([3]\)]\)[dn] =!= 
                    Null, \[IndentingNewLine]\(#[\([4]\)]\)[dn] = 
                  sign\ \(#[\([3]\)]\)[dn] - 
                    FractionalPart@\(#[\([1]\)]\)[
                        dn]; \[IndentingNewLine]If[\(#[\([4]\)]\)[dn] < 
                    1, \(#[\([4]\)]\)[dn] = 
                    1 + \(#[\([4]\)]\)[dn]]; \[IndentingNewLine]once = 
                  True]; \[IndentingNewLine] (*\ 
                day\ may\ be\ deduced\ later, \ 
                so\ do\ last\ of\ \((first, 
                    any)\)\ pass\ *) \[IndentingNewLine]If[
                dayless && 
                  ponce === False, \[IndentingNewLine]\(If[
                    once \[Equal] 
                      False, \[IndentingNewLine]If[\(#[\([1]\)]\)[dn] === 
                          Null && \(#[\([3]\)]\)[dn] =!= 
                          Null && \(#[\([4]\)]\)[dn] =!= 
                          Null, \[IndentingNewLine]\(#[\([1]\)]\)[dn] = 
                        Abs@\(#[\([3]\)]\)[dn] - 
                          Abs@\(#[\([4]\)]\)[
                              dn]; \[IndentingNewLine]If[\(#[\([1]\)]\)[dn] < 
                          1, \(#[\([1]\)]\)[dn] = 
                          1 + \(#[\([1]\)]\)[dn]]; \[IndentingNewLine]Message[
                        timeAngleRules::"\<dayless\>", {#[\([1]\)], \
#[\([2]\)]}]; \[IndentingNewLine]once = 
                        True]\[IndentingNewLine]];\)\[IndentingNewLine]]; \
\[IndentingNewLine] (*\ 
                poll\ again\ *) \[IndentingNewLine]test[{#[\([1]\)], \
#[\([2]\)]}];\[IndentingNewLine])\) &, {{LHAaries, SHAlm, arieslon, 
              lon}, {GHAaries, SHAgm, arieslon, 
              midday}}]; \[IndentingNewLine]once\[IndentingNewLine]]\)], \
"Input",
  InitializationCell->True],

Cell[BoxData[
    \(timeDeduce[
        cal_:  gregorian] := \[IndentingNewLine]Module[{i, j, once, pass, 
          passes}, \[IndentingNewLine]passes = 
          timePasses /. 
            Options[timeDeduce]; \[IndentingNewLine]Do[\[IndentingNewLine]If[
            pass === 2 && lon[dn] =!= Null && arieslon[dn] =!= Null && 
              midday =!= Null, Continue[]]; \[IndentingNewLine]i = 
            timeConvert[cal]; \[IndentingNewLine]If[i \[Equal] $Failed, 
            Return[$Failed]]; \[IndentingNewLine] (*\ 
            times\ before\ angles\ assuming\ T\ is\ accurate\ and\ has\ dn\ \
*) \[IndentingNewLine]once = True; \[IndentingNewLine]While[once, 
            once = False; \[IndentingNewLine]While[timeTimeRules[pass, cal], 
              once = True]; \[IndentingNewLine]once = 
              Or[once, 
                timeAngleRules[once, pass, 
                  cal]]\[IndentingNewLine]]; \[IndentingNewLine]i = 
            timeConvert[cal];\[IndentingNewLine], {pass, 1, 
            passes}];\[IndentingNewLine]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(timeShow[
        cal_:  gregorian] := \[IndentingNewLine]Module[{i, j, cell, once, t1, 
          t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, 
          tab}, \[IndentingNewLine]tab = 
          Map[\((\[IndentingNewLine]Table[{\[IndentingNewLine]\(#[\([1]\)]\)[\
\([i]\)] <> "\<[\>" <> 
                      ToString@#[\([3]\)] <> "\<]\>", \
\[IndentingNewLine]InputForm@\((\(\(#[\([2]\)]\)[\([i]\)]\)[#[\([3]\)]])\)\
\[IndentingNewLine]}, {i, 1, 
                    Length@#[\([1]\)]}]\[IndentingNewLine])\) &, {\
\[IndentingNewLine]{varsLStr, varsL, dn}, {varsHAStr, varsHA, 
                dn}, {varsRAStr, varsRA, dn}, {varsTStr, varsT, 
                dn}, {varsSTStr, varsST, dn}, {varsLStr, varsL, 
                ang}, {varsHAStr, varsHA, ang}, {varsRAStr, varsRA, 
                ang}, {varsTStr, varsT, tm}, {varsSTStr, varsST, 
                tm}\[IndentingNewLine]}]; \[IndentingNewLine]tab = 
          Partition[Flatten@tab, 2]; \[IndentingNewLine]tab = 
          Prepend[tab, {"\<calendar\>", 
              cal}]; \[IndentingNewLine]\[IndentingNewLine]If[
          ShowCell /. Options[timeDeduce], \[IndentingNewLine]once = 
            False; \[IndentingNewLine]cell = 
            Cell[\[IndentingNewLine]StringJoin@
                Table[{If[once, "\<\n\>", once = True; \ "\<\>"] <> 
                      ToString@tab[\([i, 1]\)] <> "\<:=\>" <> 
                      ToString@\(InputForm@
                          tab[\([i, 
                              2]\)]\) <> "\<;\>"}\[IndentingNewLine], {i, 1, 
                    Length@tab}], \[IndentingNewLine]"\<Text\>"]; \
\[IndentingNewLine]NotebookWrite[EvaluationNotebook[], \ 
            cell]\[IndentingNewLine], \[IndentingNewLine]tab]\
\[IndentingNewLine]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["end package", "Subsubsection"],

Cell[BoxData[
    \(\(End[];\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(EndPackage[];\)\)], "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[270]:="],

Cell[BoxData[
    \(\(On[General::"\<spell\>"];\)\)], "Input"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
AutoGeneratedPackage->None,
WindowSize->{680, 575},
WindowMargins->{{113, Automatic}, {181, Automatic}}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "(lic2) In[270]:="->{
    Cell[28366, 637, 110, 3, 27, "Input",
      InitializationCell->True,
      CellTags->"(lic2) In[270]:="]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"(lic2) In[270]:=", 29196, 664}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1717, 49, 41, 0, 31, "Text"],
Cell[1761, 51, 37, 0, 31, "Text"],
Cell[1801, 53, 41, 0, 31, "Text"],
Cell[1845, 55, 34, 0, 31, "Text"],
Cell[1882, 57, 34, 0, 31, "Text"],
Cell[1919, 59, 66, 0, 31, "Text"],
Cell[1988, 61, 30, 0, 31, "Text"],
Cell[2021, 63, 56, 0, 31, "Text"],
Cell[2080, 65, 35, 0, 31, "Text"],
Cell[2118, 67, 57, 0, 31, "Text"],
Cell[2178, 69, 408, 14, 235, "Text"],

Cell[CellGroupData[{
Cell[2611, 87, 30, 0, 42, "Subsubsection"],
Cell[2644, 89, 596, 11, 187, "Input"],
Cell[3243, 102, 49, 0, 31, "Text"],
Cell[3295, 104, 88, 2, 27, "Input",
  InitializationCell->True],
Cell[3386, 108, 158, 3, 27, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[3581, 116, 32, 0, 45, "Subsection"],

Cell[CellGroupData[{
Cell[3638, 120, 30, 0, 42, "Subsubsection"],
Cell[3671, 122, 254, 4, 59, "Input",
  InitializationCell->True],
Cell[3928, 128, 882, 13, 219, "Input",
  InitializationCell->True],
Cell[4813, 143, 143, 3, 27, "Input",
  InitializationCell->True],
Cell[4959, 148, 145, 3, 27, "Input",
  InitializationCell->True],
Cell[5107, 153, 148, 3, 27, "Input",
  InitializationCell->True],
Cell[5258, 158, 152, 3, 27, "Input",
  InitializationCell->True],
Cell[5413, 163, 108, 2, 27, "Input",
  InitializationCell->True],
Cell[5524, 167, 139, 3, 27, "Input",
  InitializationCell->True],
Cell[5666, 172, 132, 3, 27, "Input",
  InitializationCell->True],
Cell[5801, 177, 131, 3, 27, "Input",
  InitializationCell->True],
Cell[5935, 182, 131, 3, 27, "Input",
  InitializationCell->True],
Cell[6069, 187, 47, 0, 31, "Text"],
Cell[6119, 189, 173, 3, 27, "Input",
  InitializationCell->True],
Cell[6295, 194, 573, 10, 171, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[6905, 209, 34, 0, 42, "Subsubsection"],
Cell[6942, 211, 88, 2, 27, "Input",
  InitializationCell->True],
Cell[7033, 215, 2665, 46, 587, "Input",
  InitializationCell->True],
Cell[9701, 263, 3026, 48, 651, "Input",
  InitializationCell->True],
Cell[12730, 313, 405, 10, 150, "Text"],
Cell[13138, 325, 220, 4, 65, "Text"],
Cell[13361, 331, 231, 4, 65, "Text"],
Cell[13595, 337, 206, 4, 65, "Text"],
Cell[13804, 343, 2007, 38, 363, "Input",
  InitializationCell->True],
Cell[15814, 383, 419, 8, 107, "Input",
  InitializationCell->True],
Cell[16236, 393, 1216, 23, 251, "Input",
  InitializationCell->True],
Cell[17455, 418, 708, 14, 155, "Input",
  InitializationCell->True],
Cell[18166, 434, 420, 8, 107, "Input",
  InitializationCell->True],
Cell[18589, 444, 1756, 32, 379, "Input",
  InitializationCell->True],
Cell[20348, 478, 106, 4, 65, "Text"],
Cell[20457, 484, 127, 3, 48, "Text"],
Cell[20587, 489, 4734, 82, 859, "Input",
  InitializationCell->True],
Cell[25324, 573, 1084, 19, 283, "Input",
  InitializationCell->True],
Cell[26411, 594, 1805, 32, 459, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[28253, 631, 36, 0, 42, "Subsubsection"],
Cell[28292, 633, 71, 2, 27, "Input",
  InitializationCell->True],
Cell[28366, 637, 110, 3, 27, "Input",
  InitializationCell->True,
  CellTags->"(lic2) In[270]:="],
Cell[28479, 642, 62, 1, 27, "Input"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

